package com.aciedge.cotplitool.threads;

import com.aciedge.cotplitool.Model;
import com.aciedge.cotplitool.cot.CoTPLISender;

import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author aciedge.com
 * @author
 * <a href="mailto:tdnorbra@nps.edu?subject=com.aciedge.cotplitool.threads.CoTSendingThread">Terry
 * Norbraten, NPS MOVES</a>
 */
public class CoTSendingThread extends Thread {

    public static int currentCallsign = 1;

    private volatile boolean exit = false;

    @Override
    public void run() {
        this.send();
    }

    public void exit() {
        this.exit = true;
    }

    private void send() {
        CoTPLISender sender = null;

        long startLoopTime, endLoopTime;
        String cotUID;

        StringBuilder time = new StringBuilder();
        StringBuilder start = new StringBuilder();
        StringBuilder stale = new StringBuilder();

        Calendar timeCalendar = new GregorianCalendar(TimeZone.getTimeZone("UTC"));
        
        // Correct time format for CoT SDK
        Calendar startCalendar = new GregorianCalendar(TimeZone.getTimeZone("UTC"));
        startCalendar.setTime(Calendar.getInstance().getTime());
        start.append(startCalendar.get(1));
        start.append("-");
        start.append(twoDigit(startCalendar.get(2) + 1));
        start.append("-");
        start.append(twoDigit(startCalendar.get(5)));
        start.append("T");
        start.append(twoDigit(startCalendar.get(11)));
        start.append(":");
        start.append(twoDigit(startCalendar.get(12)));
        start.append(":");
        start.append(twoDigit(startCalendar.get(13)));
        start.append(".");
        start.append(twoDigit(startCalendar.get(14) / 10));
        start.append("Z");
        
        Calendar staleCalendar = new GregorianCalendar(TimeZone.getTimeZone("UTC"));

        try {
            sender = new CoTPLISender(Model.protocol, Model.cotDestinationIP, Model.cotDestinationPort);

            for (long amountPerMessage = 1000L / (long) Model.cotNumberPLIPerSec; !this.exit; ++currentCallsign) {
                startLoopTime = System.currentTimeMillis();
                if (currentCallsign > Model.cotNumberUniqueEntities) {
                    currentCallsign = 1;
                }

                cotUID = "PERFORMANCE-TEST-" + String.valueOf(currentCallsign);
                
                timeCalendar.setTime(Calendar.getInstance().getTime());
                time.append(timeCalendar.get(1));
                time.append("-");
                time.append(twoDigit(timeCalendar.get(2) + 1));
                time.append("-");
                time.append(twoDigit(timeCalendar.get(5)));
                time.append("T");
                time.append(twoDigit(timeCalendar.get(11)));
                time.append(":");
                time.append(twoDigit(timeCalendar.get(12)));
                time.append(":");
                time.append(twoDigit(timeCalendar.get(13)));
                time.append(".");
                time.append(twoDigit(timeCalendar.get(14) / 10));
                time.append("Z");
                
                // Sets to 2 min. ahead of time
                staleCalendar.setTimeInMillis(Calendar.getInstance().getTimeInMillis() + 120000L);
                stale.append(staleCalendar.get(1));
                stale.append("-");
                stale.append(twoDigit(staleCalendar.get(2) + 1));
                stale.append("-");
                stale.append(twoDigit(staleCalendar.get(5)));
                stale.append("T");
                stale.append(twoDigit(staleCalendar.get(11)));
                stale.append(":");
                stale.append(twoDigit(staleCalendar.get(12)));
                stale.append(":");
                stale.append(twoDigit(staleCalendar.get(13)));
                stale.append(".");
                stale.append(twoDigit(staleCalendar.get(14) / 10));
                stale.append("Z");
                
                sender.sendPLI(cotUID, time.toString(), start.toString(), stale.toString(), String.valueOf(Math.random() * 180.0D - 90.0D), String.valueOf(Math.random() * 360.0D - 180.0D), cotUID);
                
                time.setLength(0);
                stale.setLength(0);
                
                endLoopTime = System.currentTimeMillis();

                try {
                    TimeUnit.MILLISECONDS.sleep(amountPerMessage - (endLoopTime - startLoopTime));
                } catch (InterruptedException var23) {
                    Logger.getLogger(CoTSendingThread.class.getName()).log(Level.SEVERE, null, var23);
                }

            }
        } catch (IOException var24) {
            Logger.getLogger(CoTSendingThread.class.getName()).log(Level.SEVERE, null, var24);
        } finally {
            try {
                if (sender != null) {
                    sender.close();
                }
            } catch (Exception var22) {
                Logger.getLogger(CoTSendingThread.class.getName()).log(Level.SEVERE, null, var22);
            }
        }

    }

    private static String twoDigit(int i) {
        return i >= 0 && i < 10 ? "0" + String.valueOf(i) : String.valueOf(i);
    }
}
