/*
Copyright (c) 1995-2023 held by the author(s).  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other materials provided with the
      distribution.
    * Neither the names of the Naval Postgraduate School (NPS)
      Modeling Virtual Environments and Simulation (MOVES) Institute
      (https://www.nps.edu and https://my.nps.edu/web/moves)
      nor the names of its contributors may be used to endorse or
      promote products derived from this software without specific
      prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
package edu.nps.moves.cot.file;

import java.io.IOException;
import java.nio.file.Path;

import mil.army.usareur.g3.mcsd.CoTExample.CustomCoTparser;

import org.apache.commons.collections4.IterableUtils;

import static org.hamcrest.core.IsEqual.equalTo;
import static org.hamcrest.MatcherAssert.assertThat;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.JUnitCore;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;

import org.xmlunit.builder.DiffBuilder;
import org.xmlunit.diff.Diff;
import org.xmlunit.diff.Difference;

/** Test that an output file exactly matches a known reference 
 *
 * @author <a href="mailto:tdnorbra@nps.edu?subject=edu.nps.moves.cot.file.CotFileReaderTest">Terry D. Norbraten</a>
 */
public class CotFileReaderTest {
    
    Path control, test;
    CustomCoTparser ccp;
    CoTFileReader cfr;
    
    @Before
    public void before() {
        ccp = new CustomCoTparser();
        cfr = new CoTFileReader();
        cfr.addCoTparser(ccp);
    }
    
    @After
    public void after() {
        control = test = null;
        ccp = null;
        cfr = null;
    }
    
    @Test
    public void compare1() throws IOException {
        control = Path.of("src/test/resources/MT-5_Alert_Example_without_Alert_ObjectCutout_Image.cot");
        cfr.readAndProcessCoTFile(control);
        test = Path.of(ccp.getCotXmlFile().getAbsolutePath());

        Diff diff = DiffBuilder.compare(control)
        .withTest(test)
//        .checkForSimilar()
//        .checkForIdentical() // [1]
//        .ignoreComments() // [2]
        .ignoreWhitespace() // [3]
//        .normalizeWhitespace() // [4]
//        .withNodeMatcher(new DefaultNodeMatcher( ElementSelectors.byName)) // [10]
//        .ignoreElementContentWhitespace() // [15]
        .build();

        for (Difference d : diff.getDifferences())
            System.err.println(d);
        
        assertThat(IterableUtils.size(diff.getDifferences()), equalTo(0));
    }
    
    @Test
    public void compare2() throws IOException {
        control = Path.of("src/test/resources/MT-5_Alert_Example_with_Alert_ObjectCutout_Image.cot");
        cfr.readAndProcessCoTFile(control);
        test = Path.of(ccp.getCotXmlFile().getAbsolutePath());

        Diff diff = DiffBuilder.compare(control)
        .withTest(test)
//        .checkForSimilar()
//        .checkForIdentical() // [1]
//        .ignoreComments() // [2]
        .ignoreWhitespace() // [3]
//        .normalizeWhitespace() // [4]
//        .withNodeMatcher(new DefaultNodeMatcher( ElementSelectors.byName)) // [10]
//        .ignoreElementContentWhitespace() // [15]
        .build();

        for (Difference d : diff.getDifferences())
            System.err.println(d);
        
        assertThat(IterableUtils.size(diff.getDifferences()), equalTo(0));
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        Result result = JUnitCore.runClasses(CotFileReaderTest.class);
        for (Failure failure : result.getFailures())
            System.err.println(failure.toString());
        
        System.out.println(CotFileReaderTest.class.getName() + " success: " + result.wasSuccessful());
    }
}
