<?xml version="1.0" encoding="UTF-8"?>

<!--
    Document   : TransformXmlAttributesToSubElements.xsl
    Created    : 23 December 2020
    Author     : brutzman
    Description: Convert DFDL-required elements-only representation back to original elements-attributes representation.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
	<xsl:strip-space elements="*"/>
	<xsl:output encoding="UTF-8" media-type="text/xml" indent="yes" omit-xml-declaration="no" method="xml"/>
    <xsl:param name="noNamespaceSchemaLocation">
    <!--<xsl:text>CommonXmlElementsOnlySchemaManual.xsd</xsl:text> -->
        <xsl:text>CommonXmlElementsOnlySchemaTransformed.xsd</xsl:text>
    </xsl:param>

    <xsl:template match="/">
        
        <!-- wrap original tree within root element name to permit schema validation of intermediate instance document -->
        <!-- TODO improve this brute-force text output with xs:element/xs:attribute invocations instead -->
        <xsl:text disable-output-escaping="yes"><![CDATA[
<_root_ xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:noNamespaceSchemaLocation="]]></xsl:text>
        <xsl:value-of select="$noNamespaceSchemaLocation"/>
        <xsl:text disable-output-escaping="yes"><![CDATA[">
]]></xsl:text>

        <xsl:apply-templates select="*" /><!-- top of tree for full document converted here -->

        <xsl:text disable-output-escaping="yes"><![CDATA[
</_root_>]]></xsl:text>
		
	</xsl:template>

	<xsl:template match="*" >

            <xsl:element name="{local-name()}">
                <!-- TODO is nil element a singleton? -->

                <!-- first attribute values converted to immediate subelements -->
                <xsl:for-each select="@*[not(contains(local-name(),'xsi')) and not(contains(local-name(),'noNamespaceSchemaLocation'))]">
                    <xsl:sort select="lower-case(local-name())"/>

                    <xsl:variable name="intermediateAttributeElementName">
                        <xsl:text>_</xsl:text>
                        <xsl:value-of select="local-name(..)"/>
                        <xsl:text>_</xsl:text>
                        <xsl:value-of select="local-name()"/>
                    </xsl:variable>

                    <xsl:element name="{$intermediateAttributeElementName}">
                        <xsl:value-of select="."/>
                    </xsl:element>
                </xsl:for-each>

                <!-- contained value -->
                <xsl:if test="(string-length(normalize-space(child::text())) > 0)">
                    <xsl:value-of select="child::text()"/>
                </xsl:if>

                <!-- Warn if both attributes and contained content found -->
                <xsl:if test="(count(@*) > 0) and (string-length(normalize-space(child::text())) > 0)">
                    <xsl:message>
                        <xsl:text>*** illegal mixed content found for DFDL, element </xsl:text>
                        <xsl:value-of select="local-name()"/>
                        <xsl:text> contains both attributes and text </xsl:text>
                    </xsl:message>
                </xsl:if>

                <xsl:apply-templates select="*" />
            </xsl:element>

    </xsl:template>

</xsl:stylesheet>
