<?xml version="1.0" encoding="UTF-8"?>

<!--
    Document   : TransformXmlAttributesToSubElements.xsl
    Created    : 23 December 2020
    Author     : brutzman
    Description: Convert original elements-attributes representation to DFDL-required elements-only representation.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
	<xsl:strip-space elements="*"/>
	<xsl:output encoding="UTF-8" media-type="text/xml" indent="yes" omit-xml-declaration="no" method="xml"/>
    <xsl:param name="noNamespaceSchemaLocation">
        <xsl:text>CommonXmlElementsAttributesSchema.xsd</xsl:text>
    </xsl:param>

    <xsl:template match="/">

        <!-- remove _root_ element and restore original rootelement plus namespaced information -->
        <!-- TODO improve this brute-force text output with xs:element/xs:attribute invocations instead -->
        <xsl:text disable-output-escaping="yes"><![CDATA[
<]]></xsl:text><!-- line break and initial opening < character -->
        <xsl:value-of select="local-name(_root_/*[1])"/>
        <xsl:text disable-output-escaping="yes"><![CDATA[
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:noNamespaceSchemaLocation="]]></xsl:text>
        <xsl:value-of select="$noNamespaceSchemaLocation"/>
        <xsl:text disable-output-escaping="yes"><![CDATA[">
]]></xsl:text>

        <xsl:apply-templates select="_root_/*[1]/*" />

        <xsl:text disable-output-escaping="yes"><![CDATA[
</]]></xsl:text><!-- line break and start final closing tag -->
        <xsl:value-of select="local-name(_root_/*[1])"/>
        <xsl:text disable-output-escaping="yes"><![CDATA[>
]]></xsl:text>

	</xsl:template>

	<xsl:template match="*" >

        <xsl:variable name="elementName" select="local-name()"/>

        <xsl:element name="{$elementName}">

            <!-- first collect attribute values from subelements -->
            <xsl:variable name="intermediateElementPrefix">
                <xsl:text>_</xsl:text>
                <xsl:value-of select="local-name()"/>
                <xsl:text>_</xsl:text>
            </xsl:variable>
            <!-- debug diagnostic
            <xsl:message>
                <xsl:text>$elementName=</xsl:text>
                <xsl:value-of select="$elementName"/>
                <xsl:text>, </xsl:text>
                <xsl:text>$intermediateElementPrefix=</xsl:text>
                <xsl:value-of select="$intermediateElementPrefix"/>
            </xsl:message>
            -->

            <xsl:for-each select="*[starts-with(local-name(),$intermediateElementPrefix)]">

                <xsl:variable name="intermediateAttributeName">
                    <xsl:value-of select="substring-after(local-name(),$intermediateElementPrefix) "/>
                </xsl:variable>
                <!-- debug diagnostic
                <xsl:message>
                    <xsl:text>$intermediateAttributeName=</xsl:text>
                    <xsl:value-of select="$intermediateAttributeName"/>
                </xsl:message>
                -->

                <xsl:attribute name="{$intermediateAttributeName}">
                    <xsl:value-of select="."/>
                </xsl:attribute>
            </xsl:for-each>
            
            <xsl:choose>
                <xsl:when test="(string-length(normalize-space(child::text())) > 0)">
                    <!-- contained value -->
                    <xsl:value-of select="child::text()"/>
                </xsl:when>
                <xsl:otherwise>            
                    <!-- remaining elements -->
                    <xsl:apply-templates select="*[not(starts-with(local-name(),$intermediateElementPrefix))]" />
                </xsl:otherwise>
            </xsl:choose>
            
        </xsl:element>
    </xsl:template>

</xsl:stylesheet>
