<?xml version="1.0" encoding="UTF-8"?>

<!--
    Document   : TransformXsdSchemaToElementsOnly.xslt
    Created    : 24 December 2020
    Author     : brutzman
    Description: Convert original X3D Schema with regular elements-attributes representation
                 to modified schema for DFDL-required elements-only representation.
                 Preserve dfdl: markup, add default dfdl:format values if needed.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  version="2.0"
                 xmlns:xs="http://www.w3.org/2001/XMLSchema"
               xmlns:dfdl="http://www.ogf.org/dfdl/dfdl-1.0/">
	<xsl:strip-space elements="*"/>
	<xsl:output encoding="UTF-8" media-type="text/xml" indent="yes" omit-xml-declaration="no" method="xml"/>
    <xsl:param name="rootNodeName">
        <xsl:text></xsl:text>
    </xsl:param>

    <xsl:template match="/">
        
        <!-- TODO improve this brute-force text output with xs:element/xs:attribute invocations instead
        <xsl:text disable-output-escaping="yes"><![CDATA[
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
]]></xsl:text> -->
        <xsl:if test="(count(//xs:element[@name='_root_']) > 0)">
            <xsl:message>
                <xsl:text>*** Warning: this document already has a _root_ element present</xsl:text>
            </xsl:message>
        </xsl:if>
        <xs:schema xmlns:dfdl="http://www.ogf.org/dfdl/dfdl-1.0/">

            <!-- https://github.com/apache/incubator-daffodil/blob/master/daffodil-lib/src/main/resources/org/apache/daffodil/xsd/DFDLGeneralFormat.dfdl.xsd -->
            <xs:include schemaLocation="org/apache/daffodil/xsd/DFDLGeneralFormat.dfdl.xsd" />
            <!-- https://github.com/apache/incubator-daffodil/blob/master/daffodil-propgen/src/main/resources/org/apache/daffodil/xsd/dafext.xsd
            <xs:include schemaLocation="org/apache/daffodil/xsd/dafext.dfdl.xsd" /> -->
            <!-- https://github.com/DFDLSchemas/CSV/blob/master/src/main/resources/com/tresys/csv/xsd/csv.dfdl.xsd -->
            <xs:annotation>
                <xs:appinfo source="http://www.ogf.org/dfdl/">
                    <!-- ref="ex:GeneralFormatPortable" -->
                    <xsl:if test="not(xs:schema/xs:annotation/xs:appinfo/dfdl:format)">
                        <!-- insert default format if not already providedd -->
                        <dfdl:format alignment="1"
                                     alignmentUnits="bytes"
                                     binaryFloatRep="ieee"
                                     binaryNumberCheckPolicy="lax"
                                     binaryNumberRep="binary"
                                     binaryCalendarEpoch="1970-01-01T00:00:00"
                                     bitOrder="mostSignificantBitFirst"
                                     byteOrder="bigEndian"
                                     calendarCenturyStart="53"
                                     calendarCheckPolicy="strict"
                                     calendarDaysInFirstWeek="4"
                                     calendarFirstDayOfWeek="Sunday"
                                     calendarLanguage="en"
                                     calendarObserveDST="yes"
                                     calendarPatternKind="implicit"
                                     calendarTimeZone=""
                                     choiceLengthKind="implicit"
                                     decimalSigned="yes"
                                     documentFinalTerminatorCanBeMissing="no"
                                     emptyValueDelimiterPolicy="both"
                                     encoding="US-ASCII"
                                     encodingErrorPolicy="replace"
                                     escapeSchemeRef=""
                                     fillByte="%#r20;"
                                     floating="no"
                                     ignoreCase="no"
                                     initiatedContent="no"
                                     initiator=""
                                     leadingSkip="0"
                                     lengthKind="delimited"
                                     lengthUnits="bytes"
                                     occursCountKind="implicit"
                                     outputNewLine="%LF;"
                                     representation="text"
                                     separator=""
                                     separatorPosition="infix"
                                     separatorSuppressionPolicy="anyEmpty"
                                     sequenceKind="ordered"
                                     terminator=""
                                     textBidi="no"
                                     textBooleanPadCharacter="%SP;"
                                     textBooleanTrueRep="T"
                                     textBooleanFalseRep="F"
                                     textCalendarJustification="left"
                                     textCalendarPadCharacter="%SP;"
                                     textNumberCheckPolicy="lax"
                                     textNumberJustification="right"
                                     textNumberPadCharacter="%SP;"
                                     textNumberPattern="#,##0.###;-#,##0.###"
                                     textNumberRep="standard"
                                     textNumberRounding="explicit"
                                     textNumberRoundingIncrement="0"
                                     textNumberRoundingMode="roundHalfEven"
                                     textOutputMinLength="0"
                                     textPadKind="none"
                                     textStandardBase="10"
                                     textStandardDecimalSeparator="."
                                     textStandardExponentRep="E"
                                     textStandardGroupingSeparator=","
                                     textStandardInfinityRep="Inf"
                                     textStandardNaNRep="NaN"
                                     textStandardZeroRep=""
                                     textStringJustification="left"
                                     textStringPadCharacter="%SP;"
                                     textTrimKind="none"
                                     trailingSkip="0"
                                     truncateSpecifiedLengthString="no"
                                     utf16Width="fixed"
                        />
                    </xsl:if>
                </xs:appinfo>
            </xs:annotation>
            
            <xsl:variable name="actualRootNodeName">
                <xsl:choose>
                    <xsl:when test="(string-length($rootNodeName) > 0)">
                        <xsl:value-of select="$rootNodeName"/>
                    </xsl:when>
                    <xsl:when test="(string-length(xs:schema/xs:element[1]/@name) > 0)">
                        <xsl:value-of select="xs:schema/xs:element[1]/@name"/>
                        <xsl:message>
                            <xsl:text>*** using initial top-level element definition </xsl:text>
                            <xsl:value-of select="xs:schema/xs:element[1]/@name"/>
                            <xsl:text> as rootNodeName</xsl:text>
                        </xsl:message>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:message>
                            <xsl:text>*** missing parameter rootNodeName or initial top-level element definition</xsl:text>
                        </xsl:message>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <!-- define custom _root_ element -->
            <xs:element name="_root_">
                <xs:complexType>
                    <xs:sequence>
                        <xs:element ref="{$actualRootNodeName}"/>
                    </xs:sequence>
                </xs:complexType>
            </xs:element>

            <xsl:apply-templates select="xs:schema/*" /><!-- top of tree for full document converted here -->

            <!-- debug diagnostic
            <xsl:message>
                <xsl:text>* append elementized attribute definitions, referred to above</xsl:text>
            </xsl:message>
            <xsl:comment>
                <xsl:text> append elementized attribute definitions, referred to above</xsl:text>
            </xsl:comment>
            -->

        <!-- now add the elementized definitions for referenced attributes -->
        <xsl:for-each select="xs:schema/xs:element[string-length(@name) > 0][count(//xs:complexType//xs:attribute) > 0]">
            <xsl:variable name="elementName" select="@name"/>
            <!-- debug diagnostic
            <xsl:message>
                <xsl:text>* $elementName = </xsl:text>
                <xsl:value-of select="$elementName"/>
            </xsl:message>
            -->
            <xsl:for-each select="xs:complexType//xs:attribute">
                <xsl:variable name="prefixedAttributeName">
                    <xsl:text>_</xsl:text>
                    <xsl:value-of select="$elementName"/>
                    <xsl:text>_</xsl:text>
                    <xsl:value-of select="@name"/>
                </xsl:variable>
                <!-- debug
                <xsl:message>
                    <xsl:text>** $prefixedAttributeName = </xsl:text>
                    <xsl:value-of select="$prefixedAttributeName"/>
                </xsl:message>
                <xsl:comment>
                    <xsl:text>** $prefixedAttributeName=</xsl:text>
                    <xsl:value-of select="$prefixedAttributeName"/>
                    <xsl:text>, @type=</xsl:text>
                    <xsl:value-of select="@type"/>
                </xsl:comment>
                -->
                <xsl:element name="xs:element">
                    <xsl:attribute name="name">
                        <xsl:value-of select="$prefixedAttributeName"/>
                    </xsl:attribute>
                    <!-- must filter dfdl: attributes (not working), not clear if others needed...
                    <xsl:apply-templates select="@*[not(local-name() = 'name')][not(local-name() = 'type')][not(starts-with(local-name(),'dfdl:'))]" />
                    -->
                    <xs:simpleType>
                        <xs:restriction base="{@type}"/>
                    </xs:simpleType>
                    <xsl:apply-templates select="*" />
                </xsl:element>
            </xsl:for-each>
        </xsl:for-each>
<!--
        <xsl:text disable-output-escaping="yes"><![CDATA[
</xs:schema>]]></xsl:text>
-->

        </xs:schema><!-- xs:schema -->
		
	</xsl:template>

    <!-- ======================== -->

	<xsl:template match="*" >

        <xsl:variable name="namespacedElementName" select="concat('xs:',local-name())"/>

        <xsl:element name="{$namespacedElementName}">
            <xsl:apply-templates select="@*" />

            <xsl:choose>
                <xsl:when test="(count(xs:attribute) > 0)">
                    <!-- wrap sequence around elementized attributes followed by original elements -->
                    <xsl:element name="xs:sequence">
                        <!-- include any dfdl markup from subsequent elements so that element-ized attributes are handled similarly -->
                        <!-- debug diagnostics
                        <xsl:message>
                            <xsl:text>* count(xs:sequence[count(@dfdl:*) > 0])=</xsl:text>
                            <xsl:value-of select="count(xs:sequence[count(@dfdl:*) > 0])"/>
                        </xsl:message>
                            <xsl:message>
                                <xsl:text>* inside test...</xsl:text>
                            </xsl:message>
                                <xsl:message>
                                    <xsl:text>* inside loop...</xsl:text>
                                </xsl:message>
                        <xsl:if test="(count(xs:sequence[count(@dfdl:*) > 0]) > 0)">
                        </xsl:if>
                        -->
                        <xsl:for-each select="xs:sequence[1]/@dfdl:*">
                            <xsl:attribute name="{concat('dfdl:',local-name())}" select="."/>
                        </xsl:for-each>
                        <xs:annotation>
                            <xs:appinfo source="http://www.ogf.org/dfdl/">
                                <xsl:text>original attribute definitions are redefined as initial elements within this added sequence</xsl:text>
                            </xs:appinfo>
                        </xs:annotation>
                        <!-- convert each attribute definition into a specially named element -->
                        <xsl:for-each select="xs:attribute">
                            <xsl:variable name="prefixedAttributeName">
                                <xsl:text>_</xsl:text>
                                <xsl:value-of select="ancestor::xs:element/@name"/>
                                <xsl:text>_</xsl:text>
                                <xsl:value-of select="@name"/>
                            </xsl:variable>
                            <xs:element ref="{$prefixedAttributeName}"/>
                        </xsl:for-each>
                        <!-- process remaining remaining declarations, avoid xs:attribute definitions -->
                        <!-- the following element-related constructs must appear within the same xs:sequence that holds the element-ized attributes -->
                        <xsl:apply-templates select="*[not(contains(local-name(), 'attribute'))]" />
                    </xsl:element>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="*" />
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="(string-length(normalize-space(text())) > 0)">
                <xsl:value-of select="text()"/>
            </xsl:if>
        </xsl:element>

        <xsl:call-template name="dfdlDiagnostics"/>

    </xsl:template>

	<xsl:template match="@*" >

        <xsl:variable name="attributeName" select="local-name()"/><!-- TODO support dfdl: namespace -->

        <xsl:attribute name="{$attributeName}" select="."/>

    </xsl:template>

	<xsl:template match="@dfdl:*" >

        <xsl:variable name="dfdlAttributeName" select="concat('dfdl:',local-name())"/><!-- TODO support dfdl: namespace -->

        <xsl:attribute name="{$dfdlAttributeName}" select="."/>

    </xsl:template>

	<xsl:template match="dfdl:*" >

        <xsl:variable name="dfdlElementName" select="concat('dfdl:',local-name())"/><!-- TODO support dfdl: namespace -->

        <xsl:element name="{$dfdlElementName}">
            <xsl:apply-templates select="@*" />
            <xsl:apply-templates select="*" />
            <xsl:if test="(string-length(normalize-space(text())) > 0)">
                <xsl:value-of select="text()"/>
            </xsl:if>
        </xsl:element>

    </xsl:template>

	<xsl:template name="dfdlDiagnostics" >
        <!-- debug diagnostics
        <xsl:message>
            <xsl:text>*** dfdlDiagnostics local-name()=</xsl:text>
            <xsl:value-of select="local-name()"/>
        </xsl:message>
        -->
        <!-- local-name() not namespaced xs: -->
        <xsl:if test="(local-name() = 'sequence') and ((string-length(@minOccurs) > 0) or (string-length(@maxOccurs) > 0))">
            <xsl:message>
                <xsl:text>*** DFDL error: xs:sequence cannot contain </xsl:text>
                <xsl:if test="(string-length(@minOccurs) > 0)">
                    <xsl:text>minOccurs='</xsl:text>
                    <xsl:value-of select="@minOccurs"/>
                    <xsl:text>'</xsl:text>
                </xsl:if>
                <xsl:if test="(string-length(@maxOccurs) > 0)">
                    <xsl:text>maxOccurs='</xsl:text>
                    <xsl:value-of select="@maxOccurs"/>
                    <xsl:text>'</xsl:text>
                </xsl:if>
                <xsl:text> so place bounding constraints on contained individual element(s) instead</xsl:text>
            </xsl:message>
        </xsl:if>

    </xsl:template>

</xsl:stylesheet>
