/*
 * The MIT License
 *
 * Copyright 2018 Martin Dudel.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package mil.army.usareur.g3.mcsd.CoTUtils;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author martin.c.dudel.civ@mail.mil
 */
public class CoTtypes {

    Map<String, String> cotTypeMap = new HashMap<>();

    public CoTtypes() {
        cotTypeMap.put("a-.-A", "Air Track");
        cotTypeMap.put("a-.-A-C", "CIVIL AIRCRAFT");
        cotTypeMap.put("a-.-A-C-F", "FIXED WING");
        cotTypeMap.put("a-.-A-C-F-q", "FIXED WING RPV/Drone");
        cotTypeMap.put("a-.-A-C-H", "ROTARY WING");
        cotTypeMap.put("a-.-A-C-L", "LIGHTER THAN AIR");
        cotTypeMap.put("a-.-A-M-F", "FIXED WING");
        cotTypeMap.put("a-.-A-M-F-g", "FIXED WING GUNSHIP");
        cotTypeMap.put("a-.-A-M-F-A", "ATTACK/STRIKE");
        cotTypeMap.put("a-.-A-M-F-B", "BOMBER");
        cotTypeMap.put("a-.-A-M-F-C", "CARGO AIRLIFT (TRANSPORT)");
        cotTypeMap.put("a-.-A-M-F-C-H", "CARGO AIRLIFT (HEAVY)");
        cotTypeMap.put("a-.-A-M-F-C-L", "CARGO AIRLIFT (LIGHT)");
        cotTypeMap.put("a-.-A-M-F-C-M", "CARGO AIRLIFT (MEDIUM)");
        cotTypeMap.put("a-.-A-M-F-D", "AIRBORNE COMMAND POST (C2)");
        cotTypeMap.put("a-.-A-M-F-F", "FIGHTER");
        cotTypeMap.put("a-.-A-M-F-F-I", "INTERCEPTOR");
        cotTypeMap.put("a-.-A-M-F-H", "COMBAT SEARCH AND RESCUE (CSAR)");
        cotTypeMap.put("a-.-A-M-F-J", "ELECTRONIC COUNTERMEASURES (ECM/JAMMER)");
        cotTypeMap.put("a-.-A-M-F-K", "TANKER");
        cotTypeMap.put("a-.-A-M-F-L", "VSTOL");
        cotTypeMap.put("a-.-A-M-F-M", "SPECIAL OPERATIONS FORCES (SOF)");
        cotTypeMap.put("a-.-A-M-F-O", "MEDEVAC");
        cotTypeMap.put("a-.-A-M-F-P", "PATROL");
        cotTypeMap.put("a-.-A-M-F-P-M", "MINE COUNTERMEASURES");
        cotTypeMap.put("a-.-A-M-F-P-N", "ANTISURFACE WARFARE/ASUW");
        cotTypeMap.put("a-.-A-M-F-Q", "DRONE (RPV/UAV)");
        cotTypeMap.put("a-.-A-M-F-R", "RECONNAISSANCE");
        cotTypeMap.put("a-.-A-M-F-R-W", "AIRBORNE EARLY WARNING (AEW)");
        cotTypeMap.put("a-.-A-M-F-R-X", "PHOTOGRAPHIC");
        cotTypeMap.put("a-.-A-M-F-R-Z", "ELECTRONIC SURVEILLANCE MEASURES");
        cotTypeMap.put("a-.-A-M-F-S", "ANTISUBMARINE WARFARE (ASW) CARRIER BASED");
        cotTypeMap.put("a-.-A-M-F-T", "TRAINER");
        cotTypeMap.put("a-.-A-M-F-U", "UTILITY");
        cotTypeMap.put("a-.-A-M-F-U-H", "UTILITY (HEAVY)");
        cotTypeMap.put("a-.-A-M-F-U-L", "UTILITY (LIGHT)");
        cotTypeMap.put("a-.-A-M-F-U-M", "UTILITY (MEDIUM)");
        cotTypeMap.put("a-.-A-M-F-Y", "COMMUNICATIONS (C3I)");
        cotTypeMap.put("a-.-A-M-H", "ROTARY WING");
        cotTypeMap.put("a-.-A-M-H-A", "ATTACK");
        cotTypeMap.put("a-.-A-M-H-C", "CARGO AIRLIFT (TRANSPORT)");
        cotTypeMap.put("a-.-A-M-H-C-H", "CARGO AIRLIFT (HEAVY)");
        cotTypeMap.put("a-.-A-M-H-C-L", "CARGO AIRLIFT (LIGHT)");
        cotTypeMap.put("a-.-A-M-H-C-M", "CARGO AIRLIFT (MEDIUM)");
        cotTypeMap.put("a-.-A-M-H-D", "AIRBORNE COMMAND POST (C2)");
        cotTypeMap.put("a-.-A-M-H-H", "COMBAT SEARCH AND RESCUE (CSAR)");
        cotTypeMap.put("a-.-A-M-H-I", "MINE COUNTERMEASURES");
        cotTypeMap.put("a-.-A-M-H-J", "ELECTRONIC COUNTERMEASURES (ECM/JAMMER)");
        cotTypeMap.put("a-.-A-M-H-K", "TANKER");
        cotTypeMap.put("a-.-A-M-H-M", "SPECIAL OPERATIONS FORCES (SOF)");
        cotTypeMap.put("a-.-A-M-H-O", "MEDEVAC");
        cotTypeMap.put("a-.-A-M-H-Q", "DRONE (RPV/UAV)");
        cotTypeMap.put("a-.-A-M-H-R", "RECONNAISSANCE");
        cotTypeMap.put("a-.-A-M-H-S", "ANTISUBMARINE WARFARE/MPA");
        cotTypeMap.put("a-.-A-M-H-T", "TRAINER");
        cotTypeMap.put("a-.-A-M-H-U", "UTILITY");
        cotTypeMap.put("a-.-A-M-H-U-H", "UTILITY (HEAVY)");
        cotTypeMap.put("a-.-A-M-H-U-L", "UTILITY (LIGHT)");
        cotTypeMap.put("a-.-A-M-H-U-M", "UTILITY (MEDIUM)");
        cotTypeMap.put("a-.-A-M-L", "LIGHTER THAN AIR");
        cotTypeMap.put("a-.-A-W", "WEAPON");
        cotTypeMap.put("a-.-A-W-D", "DECOY");
        cotTypeMap.put("a-.-A-W-M", "MISSILE IN FLIGHT");
        cotTypeMap.put("a-.-A-W-M-A", "AIR LAUNCHED MISSILE");
        cotTypeMap.put("a-.-A-W-M-A-A", "AIR TO AIR MISSILE (AAM)");
        cotTypeMap.put("a-.-A-W-M-A-S", "AIR TO SURFACE MISSILE (ASM)");
        cotTypeMap.put("a-.-A-W-M-L", "LAND ATTACK MISSILE");
        cotTypeMap.put("a-.-A-W-M-S", "SURFACE/LAND LAUNCHED MISSILE");
        cotTypeMap.put("a-.-A-W-M-S-A", "SURFACE TO AIR MISSILE (SAM)");
        cotTypeMap.put("a-.-A-W-M-S-A-f", "SAM (FixedSite)");
        cotTypeMap.put("a-.-A-W-M-S-A-i", "SAM (Manpad)");
        cotTypeMap.put("a-.-A-W-M-S-A-m", "SAM (Mobile)");
        cotTypeMap.put("a-.-A-W-M-S-S", "SURFACE TO SURFACE MISSILE (SSM)");
        cotTypeMap.put("a-.-A-W-M-U", "SUBSURFACE TO SURFACE MISSILE (S/SSM)");
        cotTypeMap.put("a-.-G", "GROUND TRACK");
        cotTypeMap.put("a-.-G-E", "EQUIPMENT");
        cotTypeMap.put("a-.-G-E-S", "SENSOR");
        cotTypeMap.put("a-.-G-E-S-E", "EMPLACED SENSOR");
        cotTypeMap.put("a-.-G-E-S-R", "RADAR");
        cotTypeMap.put("a-.-G-E-V", "GROUND VEHICLE");
        cotTypeMap.put("a-.-G-E-V-A", "Armor (GUN)");
        cotTypeMap.put("a-.-G-E-V-A-A", "Armor (APC)");
        cotTypeMap.put("a-.-G-E-V-A-A-R", "ARMORED PERSONNEL CARRIER RECOVERY");
        cotTypeMap.put("a-.-G-E-V-A-C", "C2V/ACV");
        cotTypeMap.put("a-.-G-E-V-A-I", "ARMORED INFANTRY");
        cotTypeMap.put("a-.-G-E-V-A-L", "LIGHT ARMORED VEHICLE");
        cotTypeMap.put("a-.-G-E-V-A-S", "COMBAT SERVICE SUPPORT VEHICLE");
        cotTypeMap.put("a-.-G-E-V-A-T", "TANK");
        cotTypeMap.put("a-.-G-E-V-A-T-H", "TANK HEAVY");
        cotTypeMap.put("a-.-G-E-V-A-T-H-R", "TANK HEAVY RECOVERY");
        cotTypeMap.put("a-.-G-E-V-A-T-L", "TANK LIGHT");
        cotTypeMap.put("a-.-G-E-V-A-T-L-R", "TANK LIGHT RECOVERY");
        cotTypeMap.put("a-.-G-E-V-A-T-M", "TANK MEDIUM");
        cotTypeMap.put("a-.-G-E-V-A-T-M-R", "TANK MEDIUM RECOVERY");
        cotTypeMap.put("a-.-G-E-V-C", "CIVILIAN VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E", "ENGINEER VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-A", "MINE CLEARING VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-A-A", "ARMORED MOUNTED MINE CLEARING VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-A-T", "TRAILER MOUNTED MINE CLEARING VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-B", "BRIDGE");
        cotTypeMap.put("a-.-G-E-V-E-C", "CONSTRUCTION VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-D", "DOZER");
        cotTypeMap.put("a-.-G-E-V-E-E", "EARTHMOVER");
        cotTypeMap.put("a-.-G-E-V-E-M", "MINE LAYING VEHICLE");
        cotTypeMap.put("a-.-G-E-V-E-M-L", "TRUCK MOUNTED WITH VOLCANO");
        cotTypeMap.put("a-.-G-E-V-E-M-V", "ARMORED CARRIER WITH VOLCANO");
        cotTypeMap.put("a-.-G-E-V-T", "Locomotive");
        cotTypeMap.put("a-.-G-E-V-U", "UTILITY VEHICLE");
        cotTypeMap.put("a-.-G-E-V-U-B", "BUS");
        cotTypeMap.put("a-.-G-E-V-U-L", "LIMITED CROSS COUNTRY TRUCK");
        cotTypeMap.put("a-.-G-E-V-U-R", "WATER CRAFT");
        cotTypeMap.put("a-.-G-E-V-U-R-l", "Boat (Large)");
        cotTypeMap.put("a-.-G-E-V-U-R-m", "Boat (Med)");
        cotTypeMap.put("a-.-G-E-V-U-R-s", "Boat (Small)");
        cotTypeMap.put("a-.-G-E-V-U-S", "SEMI");
        cotTypeMap.put("a-.-G-E-V-U-X", "CROSS COUNTRY TRUCK");
        cotTypeMap.put("a-.-G-E-V-m", "Ambulance");
        cotTypeMap.put("a-.-G-E-W", "WEAPON");
        cotTypeMap.put("a-.-G-E-W-A", "AIR DEFENSE GUN");
        cotTypeMap.put("a-.-G-E-W-A-H", "AAA (Heavy)");
        cotTypeMap.put("a-.-G-E-W-A-L", "AAA (Light)");
        cotTypeMap.put("a-.-G-E-W-A-M", "AAA (Medium)");
        cotTypeMap.put("a-.-G-E-W-D", "DIRECT FIRE GUN");
        cotTypeMap.put("a-.-G-E-W-D-H", "DIRECT FIRE GUN HEAVY");
        cotTypeMap.put("a-.-G-E-W-D-H-S", "DIRECT FIRE GUN HEAVY SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-D-L", "DIRECT FIRE GUN LIGHT");
        cotTypeMap.put("a-.-G-E-W-D-L-S", "DIRECT FIRE GUN LIGHT SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-D-M", "DIRECT FIRE GUN MEDIUM");
        cotTypeMap.put("a-.-G-E-W-D-M-S", "DIRECT FIRE GUN MEDIUM SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-G", "ANTI TANK GUN");
        cotTypeMap.put("a-.-G-E-W-G-H", "ANTI TANK GUN HEAVY");
        cotTypeMap.put("a-.-G-E-W-G-L", "ANTI TANK GUN LIGHT");
        cotTypeMap.put("a-.-G-E-W-G-M", "ANTI TANK GUN MEDIUM");
        cotTypeMap.put("a-.-G-E-W-G-R", "ANTI TANK GUN RECOILLESS");
        cotTypeMap.put("a-.-G-E-W-H", "HOWITZER");
        cotTypeMap.put("a-.-G-E-W-H-H", "HOWITZER HEAVY");
        cotTypeMap.put("a-.-G-E-W-H-H-S", "HOWITZER HEAVY SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-H-L", "HOWITZER LIGHT");
        cotTypeMap.put("a-.-G-E-W-H-L-S", "HOWITZER LIGHT SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-H-M", "HOWITZER MEDIUM");
        cotTypeMap.put("a-.-G-E-W-H-M-S", "HOWITZER MEDIUM SELF PROPELLED");
        cotTypeMap.put("a-.-G-E-W-M", "MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-A", "AIR DEFENSE (AD) MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-A-I", "INTERMEDIATE RANGE AD MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-A-L", "LONG RANGE AD MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-A-S", "SHORT RANGE AD MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-A-T", "AD MISSILE LAUNCHER THEATER");
        cotTypeMap.put("a-.-G-E-W-M-S", "SURF SURF (SS) MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-S-I", "INTERMEDIATE RANGE SS MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-S-L", "LONG RANGE SS MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-S-S", "SHORT RANGE SS MISSILE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-M-T", "MISSILE LAUNCHER ANTITANK (AT)");
        cotTypeMap.put("a-.-G-E-W-M-T-H", "MISSILE LAUNCHER AT HEAVY");
        cotTypeMap.put("a-.-G-E-W-M-T-L", "MISSILE LAUNCHER AT LIGHT");
        cotTypeMap.put("a-.-G-E-W-M-T-M", "MISSILE LAUNCHER AT MEDIUM");
        cotTypeMap.put("a-.-G-E-W-O", "MORTAR");
        cotTypeMap.put("a-.-G-E-W-O-H", "MORTAR HEAVY");
        cotTypeMap.put("a-.-G-E-W-O-L", "MORTAR LIGHT");
        cotTypeMap.put("a-.-G-E-W-O-M", "MORTAR MEDIUM");
        cotTypeMap.put("a-.-G-E-W-R", "RIFLE/AUTOMATIC WEAPON");
        cotTypeMap.put("a-.-G-E-W-R-H", "HEAVY MACHINE GUN");
        cotTypeMap.put("a-.-G-E-W-R-L", "LIGHT MACHINE GUN");
        cotTypeMap.put("a-.-G-E-W-R-R", "RIFLE");
        cotTypeMap.put("a-.-G-E-W-S", "SINGLE ROCKET LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-S-H", "SINGLE ROCKET LAUNCHER HEAVY");
        cotTypeMap.put("a-.-G-E-W-S-L", "SINGLE ROCKET LAUNCHER LIGHT");
        cotTypeMap.put("a-.-G-E-W-S-M", "SINGLE ROCKET LAUNCHER MEDIUM");
        cotTypeMap.put("a-.-G-E-W-T", "ANTI TANK ROCKET LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-T-H", "ANTI TANK ROCKET LAUNCHER HEAVY");
        cotTypeMap.put("a-.-G-E-W-T-L", "ANTI TANK ROCKET LAUNCHER LIGHT");
        cotTypeMap.put("a-.-G-E-W-T-M", "ANTI TANK ROCKET LAUNCHER MEDIUM");
        cotTypeMap.put("a-.-G-E-W-X", "MULTIPLE ROCKET LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-X-H", "MULTIPLE ROCKET LAUNCHER HEAVY");
        cotTypeMap.put("a-.-G-E-W-X-L", "MULTIPLE ROCKET LAUNCHER LIGHT");
        cotTypeMap.put("a-.-G-E-W-X-M", "MULTIPLE ROCKET LAUNCHER MEDIUM");
        cotTypeMap.put("a-.-G-E-W-Z", "GRENADE LAUNCHER");
        cotTypeMap.put("a-.-G-E-W-Z-H", "GRENADE LAUNCHER HEAVY");
        cotTypeMap.put("a-.-G-E-W-Z-L", "GRENADE LAUNCHER LIGHT");
        cotTypeMap.put("a-.-G-E-W-Z-M", "GRENADE LAUNCHER MEDIUM");
        cotTypeMap.put("a-.-G-E-X", "SPECIAL EQUIPMENT");
        cotTypeMap.put("a-.-G-E-X-F", "FLAME THROWER");
        cotTypeMap.put("a-.-G-E-X-L", "LASER");
        cotTypeMap.put("a-.-G-E-X-M", "LAND MINES");
        cotTypeMap.put("a-.-G-E-X-M-C", "CLAYMORE");
        cotTypeMap.put("a-.-G-E-X-M-L", "LESS THAN LETHAL");
        cotTypeMap.put("a-.-G-E-X-N", "NBC EQUIPMENT");
        cotTypeMap.put("a-.-G-I", "Building");
        cotTypeMap.put("a-.-G-I-B", "MILITARY BASE/FACILITY");
        cotTypeMap.put("a-.-G-I-B-A", "AIRPORT/AIRBASE");
        cotTypeMap.put("a-.-G-I-B-N", "SEAPORT/NAVAL BASE");
        cotTypeMap.put("a-.-G-I-c", "Civilian");
        cotTypeMap.put("a-.-G-I-c-b", "Bridge");
        cotTypeMap.put("a-.-G-I-c-bar", "Fence/Wall/Barrier");
        cotTypeMap.put("a-.-G-I-c-can", "Canal");
        cotTypeMap.put("a-.-G-I-c-can-l", "Canal Lock");
        cotTypeMap.put("a-.-G-I-c-frm", "Farm");
        cotTypeMap.put("a-.-G-I-c-l", "Levy");
        cotTypeMap.put("a-.-G-I-c-mon", "Monument");
        cotTypeMap.put("a-.-G-I-c-o", "Offices");
        cotTypeMap.put("a-.-G-I-c-rah", "Residence/Apartment/Hotel");
        cotTypeMap.put("a-.-G-I-c-rel", "Religous");
        cotTypeMap.put("a-.-G-I-c-res", "Resevoir");
        cotTypeMap.put("a-.-G-I-c-ret", "Retail");
        cotTypeMap.put("a-.-G-I-c-sch", "School");
        cotTypeMap.put("a-.-G-I-c-vip", "VIP");
        cotTypeMap.put("a-.-G-I-c-whs", "Warehouse");
        cotTypeMap.put("a-.-G-I-E", "EQUIPMENT MANUFACTURE");
        cotTypeMap.put("a-.-G-I-E-h", "Heavy");
        cotTypeMap.put("a-.-G-I-E-l", "Light");
        cotTypeMap.put("a-.-G-I-E-o", "Plant Other");
        cotTypeMap.put("a-.-G-I-G", "GOVERNMENT LEADERSHIP");
        cotTypeMap.put("a-.-G-I-i", "IM Facilities");
        cotTypeMap.put("a-.-G-I-i-e", "Emergency Management");
        cotTypeMap.put("a-.-G-I-i-e-eoc", "Emergency Operations Center");
        cotTypeMap.put("a-.-G-I-i-e-ic", "Incident Commander");
        cotTypeMap.put("a-.-G-I-i-e-icc", "Incident Communications Center");
        cotTypeMap.put("a-.-G-I-i-e-icp", "Incident Command Post");
        cotTypeMap.put("a-.-G-I-i-e-wr", "War Room");
        cotTypeMap.put("a-.-G-I-i-f", "Fire/Hazmat");
        cotTypeMap.put("a-.-G-I-i-f-fd", "Fire Department");
        cotTypeMap.put("a-.-G-I-i-h", "Health and Medical");
        cotTypeMap.put("a-.-G-I-i-h-md", "Medical Dispatch");
        cotTypeMap.put("a-.-G-I-i-l", "Law Enforcement");
        cotTypeMap.put("a-.-G-I-i-l-cp", "Police Checkpoint");
        cotTypeMap.put("a-.-G-I-i-l-hp", "Highway Patrol");
        cotTypeMap.put("a-.-G-I-i-l-lp", "Local Police");
        cotTypeMap.put("a-.-G-I-i-l-ntz", "No Trespassing Zone");
        cotTypeMap.put("a-.-G-I-i-l-sd", "Sheriffs Department");
        cotTypeMap.put("a-.-G-I-i-l-sp", "State Police");
        cotTypeMap.put("a-.-G-I-i-m", "Emergency Medical Services");
        cotTypeMap.put("a-.-G-I-i-o", "Other");
        cotTypeMap.put("a-.-G-I-i-o-ad", "Agency Dispatch");
        cotTypeMap.put("a-.-G-I-i-o-ad-ems", "EMS");
        cotTypeMap.put("a-.-G-I-i-o-ad-f", "Fire");
        cotTypeMap.put("a-.-G-I-i-o-ad-hwy", "Highway");
        cotTypeMap.put("a-.-G-I-i-o-ad-mda", "Media");
        cotTypeMap.put("a-.-G-I-i-o-ad-pl", "Police Local");
        cotTypeMap.put("a-.-G-I-i-o-ad-ps", "Police Station");
        cotTypeMap.put("a-.-G-I-i-o-ad-tow", "Tow");
        cotTypeMap.put("a-.-G-I-i-o-ad-ts", "Transit");
        cotTypeMap.put("a-.-G-I-i-o-bcc", "Border Control Checkpoint");
        cotTypeMap.put("a-.-G-I-i-o-dmz", "DMZ");
        cotTypeMap.put("a-.-G-I-i-o-isp", "Independent Service Provider");
        cotTypeMap.put("a-.-G-I-i-o-mc", "Mobile Center");
        cotTypeMap.put("a-.-G-I-i-o-mc-f", "Fire");
        cotTypeMap.put("a-.-G-I-i-o-mc-h", "Medical");
        cotTypeMap.put("a-.-G-I-i-o-mc-l", "Law Enforcement");
        cotTypeMap.put("a-.-G-I-i-o-mes", "Message Center");
        cotTypeMap.put("a-.-G-I-i-o-mob", "Mobilization Center");
        cotTypeMap.put("a-.-G-I-i-o-mob-ems", "EMS");
        cotTypeMap.put("a-.-G-I-i-o-mob-es", "Transit");
        cotTypeMap.put("a-.-G-I-i-o-mob-f", "Fire");
        cotTypeMap.put("a-.-G-I-i-o-mob-hwy", "Highway");
        cotTypeMap.put("a-.-G-I-i-o-mob-mda", "Media");
        cotTypeMap.put("a-.-G-I-i-o-mob-pl", "Ploce Local");
        cotTypeMap.put("a-.-G-I-i-o-mob-ps", "Police Station");
        cotTypeMap.put("a-.-G-I-i-o-mob-tow", "Tow");
        cotTypeMap.put("a-.-G-I-i-o-ps", "Public Safety");
        cotTypeMap.put("a-.-G-I-i-o-rl", "Reporting Locations");
        cotTypeMap.put("a-.-G-I-i-o-sa", "Staging Area");
        cotTypeMap.put("a-.-G-I-i-o-sa-ems", "EMS");
        cotTypeMap.put("a-.-G-I-i-o-sa-f", "Fire");
        cotTypeMap.put("a-.-G-I-i-o-sa-hwy", "Highway");
        cotTypeMap.put("a-.-G-I-i-o-sa-mda", "Media");
        cotTypeMap.put("a-.-G-I-i-o-sa-pl", "Police Local");
        cotTypeMap.put("a-.-G-I-i-o-sa-ps", "Police Station");
        cotTypeMap.put("a-.-G-I-i-o-sa-tow", "Tow");
        cotTypeMap.put("a-.-G-I-i-o-sa-ts", "Transit");
        cotTypeMap.put("a-.-G-I-i-o-sc", "Security Checkpoint");
        cotTypeMap.put("a-.-G-I-i-o-seg", "Segment");
        cotTypeMap.put("a-.-G-I-i-o-t", "Transportation");
        cotTypeMap.put("a-.-G-I-i-o-t-im", "Transit Incident Management Center");
        cotTypeMap.put("a-.-G-I-i-o-t-tc", "Transit Management Center");
        cotTypeMap.put("a-.-G-I-i-o-t-td", "Traffic Inc Service Patrol Dispatch");
        cotTypeMap.put("a-.-G-I-i-o-t-tm", "Traffic Management Center");
        cotTypeMap.put("a-.-G-I-i-o-tmp", "Temporary");
        cotTypeMap.put("a-.-G-I-i-o-tmp-f", "Fire");
        cotTypeMap.put("a-.-G-I-i-o-tmp-h", "Medical");
        cotTypeMap.put("a-.-G-I-i-o-tmp-l", "Law Enforcement");
        cotTypeMap.put("a-.-G-I-i-p", "Public Works");
        cotTypeMap.put("a-.-G-I-M", "MILITARY MATERIEL FACILITY");
        cotTypeMap.put("a-.-G-I-M-A", "AIRCRAFT PRODUCTION/ASSEMBLY");
        cotTypeMap.put("a-.-G-I-M-C", "CHEMICAL/BIOLOGICAL WARFARE PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-E", "AMMUNITION AND EXPLOSIVES PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-F", "NUCLEAR ENERGY");
        cotTypeMap.put("a-.-G-I-M-F-A", "ATOMIC ENERGY REACTOR");
        cotTypeMap.put("a-.-G-I-M-F-P", "NUCLEAR MATERIAL PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-F-P-W", "WEAPONS GRADE");
        cotTypeMap.put("a-.-G-I-M-F-S", "NUCLEAR MATERIAL STORAGE");
        cotTypeMap.put("a-.-G-I-M-G", "ARMAMENT PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-M", "MISSILE/SPACE SYSTEM PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-N", "ENGINEERING EQUIPMENT PRODUCTION");
        cotTypeMap.put("a-.-G-I-M-N-B", "BRIDGE");
        cotTypeMap.put("a-.-G-I-M-N-B-l", "Bridge (Large)");
        cotTypeMap.put("a-.-G-I-M-N-B-m", "Bridge (Med)");
        cotTypeMap.put("a-.-G-I-M-N-B-s", "Bridge (Small)");
        cotTypeMap.put("a-.-G-I-M-N-c", "Canal");
        cotTypeMap.put("a-.-G-I-M-S", "SHIP CONSTRUCTION");
        cotTypeMap.put("a-.-G-I-M-V", "MILITARY VEHICLE PRODUCTION");
        cotTypeMap.put("a-.-G-I-P", "PROCESSING FACILITY");
        cotTypeMap.put("a-.-G-I-P-D", "DECON");
        cotTypeMap.put("a-.-G-I-R", "RAW MATERIAL PRODUCTION/STORAGE");
        cotTypeMap.put("a-.-G-I-R-M", "MINE");
        cotTypeMap.put("a-.-G-I-R-N", "NBC");
        cotTypeMap.put("a-.-G-I-R-N-B", "BIOLOGICAL");
        cotTypeMap.put("a-.-G-I-R-N-C", "CHEMICAL");
        cotTypeMap.put("a-.-G-I-R-N-N", "NUCLEAR");
        cotTypeMap.put("a-.-G-I-R-P", "PETROLEUM/GAS/OIL");
        cotTypeMap.put("a-.-G-I-R-P-r", "Refinery");
        cotTypeMap.put("a-.-G-I-T", "TRANSPORT FACILITY");
        cotTypeMap.put("a-.-G-I-T-a", "Airport");
        cotTypeMap.put("a-.-G-I-T-hb", "Helibase");
        cotTypeMap.put("a-.-G-I-T-hs", "Helispot");
        cotTypeMap.put("a-.-G-I-T-l", "Land Port");
        cotTypeMap.put("a-.-G-I-T-pg", "Parking Garage");
        cotTypeMap.put("a-.-G-I-T-r", "Railroad");
        cotTypeMap.put("a-.-G-I-T-s", "Seaport");
        cotTypeMap.put("a-.-G-I-U", "SERVICE, RESEARCH, UTILITY FACILITY");
        cotTypeMap.put("a-.-G-I-U-E", "ELECTRIC POWER FACILITY");
        cotTypeMap.put("a-.-G-I-U-E-c", "Coal Power Plant");
        cotTypeMap.put("a-.-G-I-U-E-h", "Hydroelectric Power Plant");
        cotTypeMap.put("a-.-G-I-U-E-o", "Other Power");
        cotTypeMap.put("a-.-G-I-U-E-ps", "Power Substation");
        cotTypeMap.put("a-.-G-I-U-E-D", "DAM");
        cotTypeMap.put("a-.-G-I-U-E-F", "FOSSIL FUEL");
        cotTypeMap.put("a-.-G-I-U-E-N", "NUCLEAR PLANT");
        cotTypeMap.put("a-.-G-I-U-P", "PUBLIC WATER SERVICES");
        cotTypeMap.put("a-.-G-I-U-R", "TECHNOLOGICAL RESEARCH FACILITY");
        cotTypeMap.put("a-.-G-I-U-T", "TELECOMMUNICATIONS FACILITY");
        cotTypeMap.put("a-.-G-I-U-T-com", "Communications");
        cotTypeMap.put("a-.-G-I-U-T-com-af", "Antenna Farm");
        cotTypeMap.put("a-.-G-I-U-T-com-sat", "Satellite Communications");
        cotTypeMap.put("a-.-G-I-U-T-com-tow", "Tower");
        cotTypeMap.put("a-.-G-I-U-T-r", "Radio");
        cotTypeMap.put("a-.-G-I-U-T-r-s", "Station");
        cotTypeMap.put("a-.-G-I-U-T-tp", "Telephone");
        cotTypeMap.put("a-.-G-I-U-T-tp-ts", "Telephone Switching");
        cotTypeMap.put("a-.-G-I-U-T-tv", "Television");
        cotTypeMap.put("a-.-G-I-U-T-tv-c", "Cable");
        cotTypeMap.put("a-.-G-I-U-T-tv-s", "Station");
        cotTypeMap.put("a-.-G-I-X", "MEDICAL FACILITY");
        cotTypeMap.put("a-.-G-I-X-H", "HOSPITAL");
        cotTypeMap.put("a-.-G-I-X-hcf", "Hospital Care Facility");
        cotTypeMap.put("a-.-G-I-r", "Road");
        cotTypeMap.put("a-.-G-I-r-h", "Road (Highway)");
        cotTypeMap.put("a-.-G-I-r-i", "Road (Improved)");
        cotTypeMap.put("a-.-G-I-r-ra", "Rest Area");
        cotTypeMap.put("a-.-G-I-r-u", "Road (Un-improved)");
        cotTypeMap.put("a-.-G-U", "UNIT");
        cotTypeMap.put("a-.-G-U-C", "COMBAT");
        cotTypeMap.put("a-.-G-U-C-A", "ARMOR");
        cotTypeMap.put("a-.-G-U-C-A-A", "ANTI ARMOR");
        cotTypeMap.put("a-.-G-U-C-A-A-A", "ANTI ARMOR ARMORED");
        cotTypeMap.put("a-.-G-U-C-A-A-A-S", "ANTI ARMOR ARMORED AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-A-A-A-T", "ANTI ARMOR ARMORED TRACKED");
        cotTypeMap.put("a-.-G-U-C-A-A-A-W", "ANTI ARMOR ARMORED WHEELED");
        cotTypeMap.put("a-.-G-U-C-A-A-C", "ANTI ARMOR ARCTIC");
        cotTypeMap.put("a-.-G-U-C-A-A-D", "ANTI ARMOR DISMOUNTED");
        cotTypeMap.put("a-.-G-U-C-A-A-L", "ANTI ARMOR LIGHT");
        cotTypeMap.put("a-.-G-U-C-A-A-M", "ANTI ARMOR AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-A-A-O", "ANTI ARMOR MOTORIZED");
        cotTypeMap.put("a-.-G-U-C-A-A-O-S", "ANTI ARMOR MOTORIZED AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-A-A-S", "ANTI ARMOR AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-A-A-U", "ANTI ARMOR MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-A-T", "ARMOR TRACK");
        cotTypeMap.put("a-.-G-U-C-A-T-A", "ARMOR TRACK AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-A-T-H", "ARMOR TRACK, HEAVY");
        cotTypeMap.put("a-.-G-U-C-A-T-L", "ARMOR TRACK, LIGHT");
        cotTypeMap.put("a-.-G-U-C-A-T-M", "ARMOR TRACK, MEDIUM");
        cotTypeMap.put("a-.-G-U-C-A-T-R", "ARMOR TRACK, RECOVERY");
        cotTypeMap.put("a-.-G-U-C-A-T-W", "ARMOR TRACK AMPHIBIOUS");
        cotTypeMap.put("a-.-G-U-C-A-T-W-R", "ARMOR TRACK AMPHIBIOUS RECOVERY");
        cotTypeMap.put("a-.-G-U-C-A-W", "ARMOR, WHEELED");
        cotTypeMap.put("a-.-G-U-C-A-W-A", "ARMOR, WHEELED AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-A-W-H", "ARMOR, WHEELED HEAVY");
        cotTypeMap.put("a-.-G-U-C-A-W-L", "ARMOR, WHEELED LIGHT");
        cotTypeMap.put("a-.-G-U-C-A-W-M", "ARMOR, WHEELED MEDIUM");
        cotTypeMap.put("a-.-G-U-C-A-W-R", "ARMOR, WHEELED RECOVERY");
        cotTypeMap.put("a-.-G-U-C-A-W-S", "ARMOR, WHEELED AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-A-W-W", "ARMOR, WHEELED AMPHIBIOUS");
        cotTypeMap.put("a-.-G-U-C-A-W-W-R", "ARMOR, WHEELED AMPHIBIOUS RECOVERY");
        cotTypeMap.put("a-.-G-U-C-D", "AIR DEFENSE");
        cotTypeMap.put("a-.-G-U-C-D-C", "COMPOSITE");
        cotTypeMap.put("a-.-G-U-C-D-G", "GUN UNIT");
        cotTypeMap.put("a-.-G-U-C-D-H", "H/MAD");
        cotTypeMap.put("a-.-G-U-C-D-H-H", "HAWK");
        cotTypeMap.put("a-.-G-U-C-D-H-P", "PATRIOT");
        cotTypeMap.put("a-.-G-U-C-D-M", "AIR DEFENSE MISSILE");
        cotTypeMap.put("a-.-G-U-C-D-M-H", "AIR DEFENSE MISSILE HEAVY");
        cotTypeMap.put("a-.-G-U-C-D-M-L", "AIR DEFENSE MISSILE LIGHT");
        cotTypeMap.put("a-.-G-U-C-D-M-L-A", "AIR DEFENSE MISSILE MOTORIZED (AVENGER)");
        cotTypeMap.put("a-.-G-U-C-D-M-M", "AIR DEFENSE MISSILE MEDIUM");
        cotTypeMap.put("a-.-G-U-C-D-O", "THEATER MISSILE DEFENSE UNIT");
        cotTypeMap.put("a-.-G-U-C-D-S", "SHORT RANGE");
        cotTypeMap.put("a-.-G-U-C-D-S-S", "STINGER");
        cotTypeMap.put("a-.-G-U-C-D-S-V", "VULCAN");
        cotTypeMap.put("a-.-G-U-C-D-T", "TARGETING UNIT");
        cotTypeMap.put("a-.-G-U-C-E", "ENGINEER");
        cotTypeMap.put("a-.-G-U-C-E-C", "ENGINEER COMBAT");
        cotTypeMap.put("a-.-G-U-C-E-C-A", "ENGINEER COMBAT AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-E-C-C", "ENGINEER COMBAT ARCTIC");
        cotTypeMap.put("a-.-G-U-C-E-C-H", "ENGINEER COMBAT HEAVY");
        cotTypeMap.put("a-.-G-U-C-E-C-L", "ENGINEER COMBAT LIGHT (SAPPER)");
        cotTypeMap.put("a-.-G-U-C-E-C-M", "ENGINEER COMBAT MEDIUM");
        cotTypeMap.put("a-.-G-U-C-E-C-O", "ENGINEER COMBAT MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-E-C-R", "ENGINEER COMBAT RECON");
        cotTypeMap.put("a-.-G-U-C-E-C-S", "ENGINEER COMBAT AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-E-C-T", "ENGINEER COMBAT MECHANIZED (TRACK)");
        cotTypeMap.put("a-.-G-U-C-E-C-W", "ENGINEER COMBAT MOTORIZED");
        cotTypeMap.put("a-.-G-U-C-E-N", "ENGINEER CONSTRUCTION");
        cotTypeMap.put("a-.-G-U-C-E-N-N", "ENGINEER NAVAL CONSTRUCTION");
        cotTypeMap.put("a-.-G-U-C-F", "Artillery (Fixed)");
        cotTypeMap.put("a-.-G-U-C-F-H", "HOWITZER/GUN");
        cotTypeMap.put("a-.-G-U-C-F-H-A", "AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-F-H-C", "ARCTIC");
        cotTypeMap.put("a-.-G-U-C-F-H-E", "Artillery (Mobile)");
        cotTypeMap.put("a-.-G-U-C-F-H-H", "HEAVY");
        cotTypeMap.put("a-.-G-U-C-F-H-L", "LIGHT");
        cotTypeMap.put("a-.-G-U-C-F-H-M", "MEDIUM");
        cotTypeMap.put("a-.-G-U-C-F-H-O", "MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-F-H-S", "AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-F-H-X", "AMPHIBIOUS");
        cotTypeMap.put("a-.-G-U-C-F-M", "MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-L", "AMPHIBIOUS MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-S", "SELF PROPELLED (SP) TRACKED MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-T", "TOWED MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-T-A", "TOWED AIRBORNE MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-T-C", "TOWED ARCTIC MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-T-O", "TOWED MOUNTAIN MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-T-S", "TOWED AIR ASSAULT MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-M-W", "SP WHEELED MORTAR");
        cotTypeMap.put("a-.-G-U-C-F-O", "METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-C-F-O-A", "AIRBORNE METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-C-F-O-L", "LIGHT METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-C-F-O-O", "MOUNTAIN METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-C-F-O-S", "AIR ASSAULT METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-C-F-R", "ROCKET");
        cotTypeMap.put("a-.-G-U-C-F-R-M", "Rockets (Fixed)");
        cotTypeMap.put("a-.-G-U-C-F-R-M-R", "Rockets (Mobile)");
        cotTypeMap.put("a-.-G-U-C-F-R-M-S", "MULTI ROCKET SELF PROPELLED");
        cotTypeMap.put("a-.-G-U-C-F-R-M-T", "MULTI ROCKET TOWED");
        cotTypeMap.put("a-.-G-U-C-F-R-S", "SINGLE ROCKET LAUNCHER");
        cotTypeMap.put("a-.-G-U-C-F-R-S-R", "SINGLE ROCKET TRUCK");
        cotTypeMap.put("a-.-G-U-C-F-R-S-S", "SINGLE ROCKET SELF PROPELLED");
        cotTypeMap.put("a-.-G-U-C-F-R-S-T", "SINGLE ROCKET TOWED");
        cotTypeMap.put("a-.-G-U-C-F-S", "ARTILLERY SURVEY");
        cotTypeMap.put("a-.-G-U-C-F-S-A", "AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-F-S-L", "LIGHT");
        cotTypeMap.put("a-.-G-U-C-F-S-O", "MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-F-S-S", "AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-F-T", "TARGET ACQUISITION");
        cotTypeMap.put("a-.-G-U-C-F-T-A", "ANGLICO");
        cotTypeMap.put("a-.-G-U-C-F-T-C", "COLT/FIST");
        cotTypeMap.put("a-.-G-U-C-F-T-C-D", "DISMOUNTED COLT/FIST");
        cotTypeMap.put("a-.-G-U-C-F-T-C-M", "TRACKED COLT/FIST");
        cotTypeMap.put("a-.-G-U-C-F-T-F", "FLASH (OPTICAL)");
        cotTypeMap.put("a-.-G-U-C-F-T-R", "RADAR");
        cotTypeMap.put("a-.-G-U-C-F-T-S", "SOUND");
        cotTypeMap.put("a-.-G-U-C-I", "Troops (Open)");
        cotTypeMap.put("a-.-G-U-C-I-A", "INFANTRY AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-I-C", "INFANTRY ARCTIC");
        cotTypeMap.put("a-.-G-U-C-I-I", "INFANTRY FIGHTING VEHICLE");
        cotTypeMap.put("a-.-G-U-C-I-L", "INFANTRY LIGHT");
        cotTypeMap.put("a-.-G-U-C-I-M", "INFANTRY MOTORIZED");
        cotTypeMap.put("a-.-G-U-C-I-N", "INFANTRY NAVAL");
        cotTypeMap.put("a-.-G-U-C-I-O", "INFANTRY MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-I-S", "INFANTRY AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-I-Z", "INFANTRY MECHANIZED");
        cotTypeMap.put("a-.-G-U-C-I-d", "Troops (DugIn)");
        cotTypeMap.put("a-.-G-U-C-M", "MISSILE (SURF SURF)");
        cotTypeMap.put("a-.-G-U-C-M-S", "MISSILE (SURF SURF) STRATEGIC");
        cotTypeMap.put("a-.-G-U-C-M-T", "MISSILE (SURF SURF) TACTICAL");
        cotTypeMap.put("a-.-G-U-C-R", "RECONNAISSANCE");
        cotTypeMap.put("a-.-G-U-C-R-A", "RECONNAISSANCE AIRBORNE");
        cotTypeMap.put("a-.-G-U-C-R-C", "RECONNAISSANCE ARCTIC");
        cotTypeMap.put("a-.-G-U-C-R-H", "RECONNAISSANCE HORSE");
        cotTypeMap.put("a-.-G-U-C-R-L", "RECONNAISSANCE LIGHT");
        cotTypeMap.put("a-.-G-U-C-R-O", "RECONNAISSANCE MOUNTAIN");
        cotTypeMap.put("a-.-G-U-C-R-R", "RECONNAISSANCE MARINE");
        cotTypeMap.put("a-.-G-U-C-R-R-D", "RECONNAISSANCE MARINE DIVISION");
        cotTypeMap.put("a-.-G-U-C-R-R-F", "RECONNAISSANCE MARINE FORCE");
        cotTypeMap.put("a-.-G-U-C-R-R-L", "RECONNAISSANCE MARINE LIGHT ARMORED");
        cotTypeMap.put("a-.-G-U-C-R-S", "RECONNAISSANCE AIR ASSAULT");
        cotTypeMap.put("a-.-G-U-C-R-V", "RECONNAISSANCE CAVALRY");
        cotTypeMap.put("a-.-G-U-C-R-V-A", "RECONNAISSANCE CAVALRY ARMORED");
        cotTypeMap.put("a-.-G-U-C-R-V-G", "RECONNAISSANCE CAVALRY GROUND");
        cotTypeMap.put("a-.-G-U-C-R-V-M", "RECONNAISSANCE CAVALRY MOTORIZED");
        cotTypeMap.put("a-.-G-U-C-R-V-O", "RECONNAISSANCE CAVALRY AIR");
        cotTypeMap.put("a-.-G-U-C-R-X", "RECONNAISSANCE LONG RANGE SURVEILLANCE");
        cotTypeMap.put("a-.-G-U-C-S", "INTERNAL SECURITY FORCES");
        cotTypeMap.put("a-.-G-U-C-S-A", "AVIATION");
        cotTypeMap.put("a-.-G-U-C-S-G", "GROUND");
        cotTypeMap.put("a-.-G-U-C-S-G-A", "MECHANIZED GROUND");
        cotTypeMap.put("a-.-G-U-C-S-G-D", "DISMOUNTED GROUND");
        cotTypeMap.put("a-.-G-U-C-S-G-M", "MOTORIZED GROUND");
        cotTypeMap.put("a-.-G-U-C-S-M", "WHEELED MECHANIZED");
        cotTypeMap.put("a-.-G-U-C-S-R", "Rail");
        cotTypeMap.put("a-.-G-U-C-S-W", "RIVERINE");
        cotTypeMap.put("a-.-G-U-C-V", "AVIATION");
        cotTypeMap.put("a-.-G-U-C-V-C", "COMPOSITE");
        cotTypeMap.put("a-.-G-U-C-V-F", "FIXED WING");
        cotTypeMap.put("a-.-G-U-C-V-F-A", "ATTACK FIXED WING");
        cotTypeMap.put("a-.-G-U-C-V-F-R", "RECON FIXED WING");
        cotTypeMap.put("a-.-G-U-C-V-F-U", "UTILITY FIXED WING");
        cotTypeMap.put("a-.-G-U-C-V-R", "ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-A", "ATTACK ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-M", "MINE COUNTERMEASURE ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-S", "SCOUT ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U", "UTILITY ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U-C", "C2 ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U-E", "MEDEVAC ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U-H", "HEAVY UTILITY ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U-L", "LIGHT UTILITY ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-U-M", "MEDIUM UTILITY ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-W", "ANTISUBMARINE WARFARE ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-R-d", "Helo ground unit (Dual)");
        cotTypeMap.put("a-.-G-U-C-V-R-s", "Helo ground unit (Single)");
        cotTypeMap.put("a-.-G-U-C-V-S", "SEARCH AND RESCUE");
        cotTypeMap.put("a-.-G-U-C-V-U", "UNMANNED AERIAL VEHICLE");
        cotTypeMap.put("a-.-G-U-C-V-U-F", "UNMANNED AERIAL VEHICLE FIXED WING");
        cotTypeMap.put("a-.-G-U-C-V-U-R", "UNMANNED AERIAL VEHICLE ROTARY WING");
        cotTypeMap.put("a-.-G-U-C-V-V", "VERTICAL/SHORT TAKEOFF AND LANDING");
        cotTypeMap.put("a-.-G-U-H", "SPECIAL C2 HEADQUARTERS COMPONENT");
        cotTypeMap.put("a-.-G-U-S", "COMBAT SERVICE SUPPORT");
        cotTypeMap.put("a-.-G-U-S-A", "ADMINISTRATIVE (ADMIN)");
        cotTypeMap.put("a-.-G-U-S-A-C", "ADMIN CORPS");
        cotTypeMap.put("a-.-G-U-S-A-F", "FINANCE");
        cotTypeMap.put("a-.-G-U-S-A-F-C", "FINANCE CORPS");
        cotTypeMap.put("a-.-G-U-S-A-F-T", "FINANCE THEATER");
        cotTypeMap.put("a-.-G-U-S-A-J", "JUDGE ADVOCATE GENERAL (JAG)");
        cotTypeMap.put("a-.-G-U-S-A-J-C", "JAG CORPS");
        cotTypeMap.put("a-.-G-U-S-A-J-T", "JAG THEATER");
        cotTypeMap.put("a-.-G-U-S-A-L", "LABOR");
        cotTypeMap.put("a-.-G-U-S-A-L-C", "LABOR CORPS");
        cotTypeMap.put("a-.-G-U-S-A-L-T", "LABOR THEATER");
        cotTypeMap.put("a-.-G-U-S-A-M", "MORTUARY/GRAVES REGISTRY");
        cotTypeMap.put("a-.-G-U-S-A-M-C", "MORTUARY/GRAVES REGISTRY CORPS");
        cotTypeMap.put("a-.-G-U-S-A-M-T", "MORTUARY/GRAVES REGISTRY THEATER");
        cotTypeMap.put("a-.-G-U-S-A-O", "POSTAL");
        cotTypeMap.put("a-.-G-U-S-A-O-C", "POSTAL CORPS");
        cotTypeMap.put("a-.-G-U-S-A-O-T", "POSTAL THEATER");
        cotTypeMap.put("a-.-G-U-S-A-P", "PUBLIC AFFAIRS");
        cotTypeMap.put("a-.-G-U-S-A-P-B", "PUBLIC AFFAIRS BROADCAST");
        cotTypeMap.put("a-.-G-U-S-A-P-B-C", "PUBLIC AFFAIRS BROADCAST CORPS");
        cotTypeMap.put("a-.-G-U-S-A-P-B-T", "PUBLIC AFFAIRS BROADCAST THEATER");
        cotTypeMap.put("a-.-G-U-S-A-P-C", "PUBLIC AFFAIRS CORPS");
        cotTypeMap.put("a-.-G-U-S-A-P-M", "PUBLIC AFFAIRS JOINT INFORMATION BUREAU");
        cotTypeMap.put("a-.-G-U-S-A-P-M-C", "PUBLIC AFFAIRS JIB CORPS");
        cotTypeMap.put("a-.-G-U-S-A-P-M-T", "PUBLIC AFFAIRS JIB THEATER");
        cotTypeMap.put("a-.-G-U-S-A-P-T", "PUBLIC AFFAIRS THEATER");
        cotTypeMap.put("a-.-G-U-S-A-Q", "QUARTERMASTER (SUPPLY)");
        cotTypeMap.put("a-.-G-U-S-A-Q-C", "QUARTERMASTER (SUPPLY) CORPS");
        cotTypeMap.put("a-.-G-U-S-A-Q-T", "QUARTERMASTER (SUPPLY) THEATER");
        cotTypeMap.put("a-.-G-U-S-A-R", "RELIGIOUS/CHAPLAIN");
        cotTypeMap.put("a-.-G-U-S-A-R-C", "RELIGIOUS/CHAPLAIN CORPS");
        cotTypeMap.put("a-.-G-U-S-A-R-T", "RELIGIOUS/CHAPLAIN THEATER");
        cotTypeMap.put("a-.-G-U-S-A-S", "PERSONNEL SERVICES");
        cotTypeMap.put("a-.-G-U-S-A-S-C", "PERSONNEL CORPS");
        cotTypeMap.put("a-.-G-U-S-A-S-T", "PERSONNEL THEATER");
        cotTypeMap.put("a-.-G-U-S-A-T", "ADMIN THEATER");
        cotTypeMap.put("a-.-G-U-S-A-W", "MORALE, WELFARE, RECREATION (MWR)");
        cotTypeMap.put("a-.-G-U-S-A-W-C", "MWR CORPS");
        cotTypeMap.put("a-.-G-U-S-A-W-T", "MWR THEATER");
        cotTypeMap.put("a-.-G-U-S-A-X", "REPLACEMENT HOLDING UNIT (RHU)");
        cotTypeMap.put("a-.-G-U-S-A-X-C", "RHU CORPS");
        cotTypeMap.put("a-.-G-U-S-A-X-T", "RHU THEATER");
        cotTypeMap.put("a-.-G-U-S-M", "MEDICAL");
        cotTypeMap.put("a-.-G-U-S-M-C", "MEDICAL CORPS");
        cotTypeMap.put("a-.-G-U-S-M-D", "MEDICAL DENTAL");
        cotTypeMap.put("a-.-G-U-S-M-D-C", "MEDICAL DENTAL CORPS");
        cotTypeMap.put("a-.-G-U-S-M-D-T", "MEDICAL DENTAL THEATER");
        cotTypeMap.put("a-.-G-U-S-M-M", "MEDICAL TREATMENT FACILITY");
        cotTypeMap.put("a-.-G-U-S-M-M-C", "MEDICAL TREATMENT FACILITY CORPS");
        cotTypeMap.put("a-.-G-U-S-M-M-T", "MEDICAL TREATMENT FACILITY THEATER");
        cotTypeMap.put("a-.-G-U-S-M-P", "MEDICAL PSYCHOLOGICAL");
        cotTypeMap.put("a-.-G-U-S-M-P-C", "MEDICAL PSYCHOLOGICAL CORPS");
        cotTypeMap.put("a-.-G-U-S-M-P-T", "MEDICAL PSYCHOLOGICAL THEATER");
        cotTypeMap.put("a-.-G-U-S-M-T", "MEDICAL THEATER");
        cotTypeMap.put("a-.-G-U-S-M-V", "MEDICAL VETERINARY");
        cotTypeMap.put("a-.-G-U-S-M-V-C", "MEDICAL VETERINARY CORPS");
        cotTypeMap.put("a-.-G-U-S-M-V-T", "MEDICAL VETERINARY THEATER");
        cotTypeMap.put("a-.-G-U-S-S", "SUPPLY");
        cotTypeMap.put("a-.-G-U-S-S-1", "SUPPLY CLASS I");
        cotTypeMap.put("a-.-G-U-S-S-1-C", "SUPPLY CLASS I CORPS");
        cotTypeMap.put("a-.-G-U-S-S-1-T", "SUPPLY CLASS I THEATER");
        cotTypeMap.put("a-.-G-U-S-S-2", "SUPPLY CLASS II");
        cotTypeMap.put("a-.-G-U-S-S-2-C", "SUPPLY CLASS II CORPS");
        cotTypeMap.put("a-.-G-U-S-S-2-T", "SUPPLY CLASS II THEATER");
        cotTypeMap.put("a-.-G-U-S-S-3", "SUPPLY CLASS III");
        cotTypeMap.put("a-.-G-U-S-S-3-A", "SUPPLY CLASS III AVIATION");
        cotTypeMap.put("a-.-G-U-S-S-3-A-C", "SUPPLY CLASS III AVIATION CORPS");
        cotTypeMap.put("a-.-G-U-S-S-3-A-T", "SUPPLY CLASS III AVIATION THEATER");
        cotTypeMap.put("a-.-G-U-S-S-3-C", "SUPPLY CLASS III CORPS");
        cotTypeMap.put("a-.-G-U-S-S-3-T", "SUPPLY CLASS III THEATER");
        cotTypeMap.put("a-.-G-U-S-S-4", "SUPPLY CLASS IV");
        cotTypeMap.put("a-.-G-U-S-S-4-C", "SUPPLY CLASS IV CORPS");
        cotTypeMap.put("a-.-G-U-S-S-4-T", "SUPPLY CLASS IV THEATER");
        cotTypeMap.put("a-.-G-U-S-S-5", "SUPPLY CLASS V");
        cotTypeMap.put("a-.-G-U-S-S-5-C", "SUPPLY CLASS V CORPS");
        cotTypeMap.put("a-.-G-U-S-S-5-T", "SUPPLY CLASS V THEATER");
        cotTypeMap.put("a-.-G-U-S-S-6", "SUPPLY CLASS VI");
        cotTypeMap.put("a-.-G-U-S-S-6-C", "SUPPLY CLASS VI CORPS");
        cotTypeMap.put("a-.-G-U-S-S-6-T", "SUPPLY CLASS VI THEATER");
        cotTypeMap.put("a-.-G-U-S-S-7", "SUPPLY CLASS VII");
        cotTypeMap.put("a-.-G-U-S-S-7-C", "SUPPLY CLASS VII CORPS");
        cotTypeMap.put("a-.-G-U-S-S-7-T", "SUPPLY CLASS VII THEATER");
        cotTypeMap.put("a-.-G-U-S-S-8", "SUPPLY CLASS VIII");
        cotTypeMap.put("a-.-G-U-S-S-8-C", "SUPPLY CLASS VIII CORPS");
        cotTypeMap.put("a-.-G-U-S-S-8-T", "SUPPLY CLASS VIII THEATER");
        cotTypeMap.put("a-.-G-U-S-S-9", "SUPPLY CLASS IX");
        cotTypeMap.put("a-.-G-U-S-S-9-C", "SUPPLY CLASS IX CORPS");
        cotTypeMap.put("a-.-G-U-S-S-9-T", "SUPPLY CLASS IX THEATER");
        cotTypeMap.put("a-.-G-U-S-S-C", "SUPPLY CORPS");
        cotTypeMap.put("a-.-G-U-S-S-L", "SUPPLY LAUNDRY/BATH");
        cotTypeMap.put("a-.-G-U-S-S-L-C", "SUPPLY LAUNDRY/BATH CORPS");
        cotTypeMap.put("a-.-G-U-S-S-L-T", "SUPPLY LAUNDRY/BATH THEATER");
        cotTypeMap.put("a-.-G-U-S-S-T", "SUPPLY THEATER");
        cotTypeMap.put("a-.-G-U-S-S-W", "SUPPLY WATER");
        cotTypeMap.put("a-.-G-U-S-S-W-C", "SUPPLY WATER CORPS");
        cotTypeMap.put("a-.-G-U-S-S-W-P", "SUPPLY WATER PURIFICATION");
        cotTypeMap.put("a-.-G-U-S-S-W-P-C", "SUPPLY WATER PURIFICATION CORPS");
        cotTypeMap.put("a-.-G-U-S-S-W-P-T", "SUPPLY WATER PURIFICATION THEATER");
        cotTypeMap.put("a-.-G-U-S-S-W-T", "SUPPLY WATER THEATER");
        cotTypeMap.put("a-.-G-U-S-S-X", "SUPPLY CLASS X");
        cotTypeMap.put("a-.-G-U-S-S-X-C", "SUPPLY CLASS X CORPS");
        cotTypeMap.put("a-.-G-U-S-S-X-T", "SUPPLY CLASS X THEATER");
        cotTypeMap.put("a-.-G-U-S-S-d", "SupplyDepot");
        cotTypeMap.put("a-.-G-U-S-T", "TRANSPORTATION");
        cotTypeMap.put("a-.-G-U-S-T-A", "APOD/APOE");
        cotTypeMap.put("a-.-G-U-S-T-A-C", "APOD/APOE CORPS");
        cotTypeMap.put("a-.-G-U-S-T-A-T", "APOD/APOE THEATER");
        cotTypeMap.put("a-.-G-U-S-T-C", "TRANSPORTATION CORPS");
        cotTypeMap.put("a-.-G-U-S-T-I", "MISSILE");
        cotTypeMap.put("a-.-G-U-S-T-I-C", "MISSILE CORPS");
        cotTypeMap.put("a-.-G-U-S-T-I-T", "MISSILE THEATER");
        cotTypeMap.put("a-.-G-U-S-T-M", "MOVEMENT CONTROL CENTER(MCC)");
        cotTypeMap.put("a-.-G-U-S-T-M-C", "MCC CORPS");
        cotTypeMap.put("a-.-G-U-S-T-M-T", "MCC THEATER");
        cotTypeMap.put("a-.-G-U-S-T-R", "RAILHEAD");
        cotTypeMap.put("a-.-G-U-S-T-R-C", "RAILHEAD CORPS");
        cotTypeMap.put("a-.-G-U-S-T-R-T", "RAILHEAD THEATER");
        cotTypeMap.put("a-.-G-U-S-T-S", "SPOD/SPOE");
        cotTypeMap.put("a-.-G-U-S-T-S-C", "SPOD/SPOE CORPS");
        cotTypeMap.put("a-.-G-U-S-T-S-T", "SPOD/SPOE THEATER");
        cotTypeMap.put("a-.-G-U-S-T-T", "TRANSPORTATION THEATER");
        cotTypeMap.put("a-.-G-U-S-X", "MAINTENANCE");
        cotTypeMap.put("a-.-G-U-S-X-C", "MAINTENANCE CORPS");
        cotTypeMap.put("a-.-G-U-S-X-E", "ELECTRO OPTICAL");
        cotTypeMap.put("a-.-G-U-S-X-E-C", "ELECTRO OPTICAL CORPS");
        cotTypeMap.put("a-.-G-U-S-X-E-T", "ELECTRO OPTICAL THEATER");
        cotTypeMap.put("a-.-G-U-S-X-H", "MAINTENANCE HEAVY");
        cotTypeMap.put("a-.-G-U-S-X-H-C", "MAINTENANCE HEAVY CORPS");
        cotTypeMap.put("a-.-G-U-S-X-H-T", "MAINTENANCE HEAVY THEATER");
        cotTypeMap.put("a-.-G-U-S-X-O", "ORDNANCE");
        cotTypeMap.put("a-.-G-U-S-X-O-C", "ORDNANCE CORPS");
        cotTypeMap.put("a-.-G-U-S-X-O-M", "ORDNANCE MISSILE");
        cotTypeMap.put("a-.-G-U-S-X-O-M-C", "ORDNANCE MISSILE CORPS");
        cotTypeMap.put("a-.-G-U-S-X-O-M-T", "ORDNANCE MISSILE THEATER");
        cotTypeMap.put("a-.-G-U-S-X-O-T", "ORDNANCE THEATER");
        cotTypeMap.put("a-.-G-U-S-X-R", "MAINTENANCE RECOVERY");
        cotTypeMap.put("a-.-G-U-S-X-R-C", "MAINTENANCE RECOVERY CORPS");
        cotTypeMap.put("a-.-G-U-S-X-R-T", "MAINTENANCE RECOVERY THEATER");
        cotTypeMap.put("a-.-G-U-S-X-T", "MAINTENANCE THEATER");
        cotTypeMap.put("a-.-G-U-U", "COMBAT SUPPORT");
        cotTypeMap.put("a-.-G-U-U-A", "COMBAT SUPPORT NBC");
        cotTypeMap.put("a-.-G-U-U-A-B", "BIOLOGICAL");
        cotTypeMap.put("a-.-G-U-U-A-B-R", "RECON EQUIPPED");
        cotTypeMap.put("a-.-G-U-U-A-C", "CHEMICAL");
        cotTypeMap.put("a-.-G-U-U-A-C-C", "SMOKE/DECON");
        cotTypeMap.put("a-.-G-U-U-A-C-C-K", "MECHANIZED SMOKE/DECON");
        cotTypeMap.put("a-.-G-U-U-A-C-C-M", "MOTORIZED SMOKE/DECON");
        cotTypeMap.put("a-.-G-U-U-A-C-R", "CHEMICAL RECON");
        cotTypeMap.put("a-.-G-U-U-A-C-R-S", "CHEMICAL WHEELED ARMORED VEHICLE");
        cotTypeMap.put("a-.-G-U-U-A-C-R-W", "CHEMICAL WHEELED ARMORED VEHICLE");
        cotTypeMap.put("a-.-G-U-U-A-C-S", "SMOKE");
        cotTypeMap.put("a-.-G-U-U-A-C-S-A", "ARMOR SMOKE");
        cotTypeMap.put("a-.-G-U-U-A-C-S-M", "MOTORIZED SMOKE");
        cotTypeMap.put("a-.-G-U-U-A-D", "DECONTAMINATION");
        cotTypeMap.put("a-.-G-U-U-A-N", "NUCLEAR");
        cotTypeMap.put("a-.-G-U-U-E", "EXPLOSIVE ORDINANCE DISPOSAL");
        cotTypeMap.put("a-.-G-U-U-I", "INFORMATION WARFARE UNIT");
        cotTypeMap.put("a-.-G-U-U-L", "LAW ENFORCEMENT UNIT");
        cotTypeMap.put("a-.-G-U-U-L-C", "CIVILIAN LAW ENFORCEMENT");
        cotTypeMap.put("a-.-G-U-U-L-D", "CENTRAL INTELLIGENCE DIVISION (CID)");
        cotTypeMap.put("a-.-G-U-U-L-F", "SECURITY POLICE (AIR)");
        cotTypeMap.put("a-.-G-U-U-L-M", "MILITARY POLICE");
        cotTypeMap.put("a-.-G-U-U-L-S", "SHORE PATROL");
        cotTypeMap.put("a-.-G-U-U-M", "MILITARY INTELLIGENCE");
        cotTypeMap.put("a-.-G-U-U-M-A", "AERIAL EXPLOITATION");
        cotTypeMap.put("a-.-G-U-U-M-C", "COUNTER INTELLIGENCE");
        cotTypeMap.put("a-.-G-U-U-M-J", "JOINT INTELLIGENCE CENTER");
        cotTypeMap.put("a-.-G-U-U-M-M-O", "METEOROLOGICAL");
        cotTypeMap.put("a-.-G-U-U-M-O", "OPERATIONS");
        cotTypeMap.put("a-.-G-U-U-M-Q", "INTERROGATION");
        cotTypeMap.put("a-.-G-U-U-M-R", "SURVEILLANCE");
        cotTypeMap.put("a-.-G-U-U-M-R-G", "GROUND SURVEILLANCE RADAR");
        cotTypeMap.put("a-.-G-U-U-M-R-S", "SENSOR");
        cotTypeMap.put("a-.-G-U-U-M-R-S-S", "SENSOR SCM");
        cotTypeMap.put("a-.-G-U-U-M-R-X", "GROUND STATION MODULE");
        cotTypeMap.put("a-.-G-U-U-M-S", "SIGNAL INTELLIGENCE (SIGINT)");
        cotTypeMap.put("a-.-G-U-U-M-S-E", "ELECTRONIC WARFARE");
        cotTypeMap.put("a-.-G-U-U-M-S-E-A", "ARMORED WHEELED VEHICLE");
        cotTypeMap.put("a-.-G-U-U-M-S-E-C", "CORPS");
        cotTypeMap.put("a-.-G-U-U-M-S-E-D", "DIRECTION FINDING");
        cotTypeMap.put("a-.-G-U-U-M-S-E-I", "INTERCEPT");
        cotTypeMap.put("a-.-G-U-U-M-S-E-J", "JAMMING");
        cotTypeMap.put("a-.-G-U-U-M-S-E-T", "THEATER");
        cotTypeMap.put("a-.-G-U-U-M-T", "TACTICAL EXPLOIT");
        cotTypeMap.put("a-.-G-U-U-P", "LANDING SUPPORT");
        cotTypeMap.put("a-.-G-U-U-S", "SIGNAL UNIT");
        cotTypeMap.put("a-.-G-U-U-S-A", "AREA");
        cotTypeMap.put("a-.-G-U-U-S-C", "COMMUNICATION CONFIGURED PACKAGE");
        cotTypeMap.put("a-.-G-U-U-S-C-L", "LARGE COMMUNICATION CONFIGURED PACKAGE");
        cotTypeMap.put("a-.-G-U-U-S-F", "FORWARD COMMUNICATIONS");
        cotTypeMap.put("a-.-G-U-U-S-M", "MULTIPLE SUBSCRIBER ELEMENT");
        cotTypeMap.put("a-.-G-U-U-S-M-L", "LARGE EXTENSION NODE");
        cotTypeMap.put("a-.-G-U-U-S-M-N", "NODE CENTER");
        cotTypeMap.put("a-.-G-U-U-S-M-S", "SMALL EXTENSION NODE");
        cotTypeMap.put("a-.-G-U-U-S-O", "COMMAND OPERATIONS");
        cotTypeMap.put("a-.-G-U-U-S-R", "RADIO UNIT");
        cotTypeMap.put("a-.-G-U-U-S-R-S", "TACTICAL SATELLITE");
        cotTypeMap.put("a-.-G-U-U-S-R-T", "TELETYPE CENTER");
        cotTypeMap.put("a-.-G-U-U-S-R-W", "RELAY");
        cotTypeMap.put("a-.-G-U-U-S-S", "SIGNAL SUPPORT");
        cotTypeMap.put("a-.-G-U-U-S-W", "TELEPHONE SWITCH");
        cotTypeMap.put("a-.-G-U-U-S-X", "ELECTRONIC RANGING");
        cotTypeMap.put("a-.-G-U-i", "Incident Management Resources");
        cotTypeMap.put("a-.-G-U-i-a", "Animal Health");
        cotTypeMap.put("a-.-G-U-i-a-lar", "Large Animal Rescue Strike Team");
        cotTypeMap.put("a-.-G-U-i-a-las", "Large Animal Sheltering Team");
        cotTypeMap.put("a-.-G-U-i-a-lat", "Large Animal Transport Team");
        cotTypeMap.put("a-.-G-U-i-a-sar", "Small Animal Rescue Strike Team");
        cotTypeMap.put("a-.-G-U-i-a-sas", "Small Animal Sheltering Team");
        cotTypeMap.put("a-.-G-U-i-a-sat", "Small Animal Transport Team");
        cotTypeMap.put("a-.-G-U-i-a-imt", "Incident Management Team Animal Protection");
        cotTypeMap.put("a-.-G-U-i-e", "Emergency Management");
        cotTypeMap.put("a-.-G-U-i-e-acrc", "Airborne Communications Relay(CAP)");
        cotTypeMap.put("a-.-G-U-i-e-acrt", "Airborne Communications Relay Team");
        cotTypeMap.put("a-.-G-U-i-e-att", "Airborne Transport Team");
        cotTypeMap.put("a-.-G-U-i-e-cis", "Critical Incident Stress Management Team");
        cotTypeMap.put("a-.-G-U-i-e-cst", "Communications Support Team");
        cotTypeMap.put("a-.-G-U-i-e-dc", "Donations Coordinator");
        cotTypeMap.put("a-.-G-U-i-e-dmt", "Donations Management Personnel Team");
        cotTypeMap.put("a-.-G-U-i-e-ect", "Evacuation Coordination Team");
        cotTypeMap.put("a-.-G-U-i-e-efa", "EOC Finance Administration");
        cotTypeMap.put("a-.-G-U-i-e-elt", "Evacuation Liaison Team");
        cotTypeMap.put("a-.-G-U-i-e-ems", "EOC Management Support Team");
        cotTypeMap.put("a-.-G-U-i-e-eos", "EOC Operations Section Chief");
        cotTypeMap.put("a-.-G-U-i-e-eps", "EOC Planning Section Chief");
        cotTypeMap.put("a-.-G-U-i-e-ial", "Individual Assistance Disaster Assessment Team Leader");
        cotTypeMap.put("a-.-G-U-i-e-iat", "Individual Assistance Disaster Assessment Team");
        cotTypeMap.put("a-.-G-U-i-e-imt", "Incident Management Team");
        cotTypeMap.put("a-.-G-U-i-e-mcc", "Mobile Communications Center");
        cotTypeMap.put("a-.-G-U-i-e-mfk", "Mobile Feeding Kitchen");
        cotTypeMap.put("a-.-G-U-i-e-pac", "Public Assistance Coordinator");
        cotTypeMap.put("a-.-G-U-i-e-rna", "Rapid Needs Assessment Team");
        cotTypeMap.put("a-.-G-U-i-e-smt", "Shelter Management Team");
        cotTypeMap.put("a-.-G-U-i-e-val", "Volunteer Agency Liaison");
        cotTypeMap.put("a-.-G-U-i-m", "Emergency Medical Services");
        cotTypeMap.put("a-.-G-U-i-m-aaf", "Air Ambulance (Fixed-Wing)");
        cotTypeMap.put("a-.-G-U-i-m-aar", "Air Ambulance (Rotary-Wing)");
        cotTypeMap.put("a-.-G-U-i-m-ag", "Ambulances (Ground)");
        cotTypeMap.put("a-.-G-U-i-m-ast", "Ambulance Strike Team");
        cotTypeMap.put("a-.-G-U-i-m-atf", "Ambulance Task Force");
        cotTypeMap.put("a-.-G-U-i-m-etf", "Emergency Medical Task Force");
        cotTypeMap.put("a-.-G-U-i-f", "Fire HAZMAT");
        cotTypeMap.put("a-.-G-U-i-f-act", "Area Command Team, Firefighting");
        cotTypeMap.put("a-.-G-U-i-f-bp", "Brush Patrol");
        cotTypeMap.put("a-.-G-U-i-f-ct", "Crew Transport");
        cotTypeMap.put("a-.-G-U-i-f-efp", "Engine, Fire (Pumper)");
        cotTypeMap.put("a-.-G-U-i-f-fb", "Fire Boat");
        cotTypeMap.put("a-.-G-U-i-f-ft", "Fuel Tender");
        cotTypeMap.put("a-.-G-U-i-f-fta", "Fire Truck");
        cotTypeMap.put("a-.-G-U-i-f-ftf", "Foam Tender");
        cotTypeMap.put("a-.-G-U-i-f-hc", "Hand Crew");
        cotTypeMap.put("a-.-G-U-i-f-het", "HazMat Entry Team");
        cotTypeMap.put("a-.-G-U-i-f-hf", "Helicopters, Firefighting");
        cotTypeMap.put("a-.-G-U-i-f-ht", "Helitanker");
        cotTypeMap.put("a-.-G-U-i-f-imt", "Incident Management Team Firefighting");
        cotTypeMap.put("a-.-G-U-i-f-ibt", "Interagency Buying Team Firefighting");
        cotTypeMap.put("a-.-G-U-i-f-mcu", "Mobile Communications Unit");
        cotTypeMap.put("a-.-G-U-i-f-nsf", "USCG National Strike Force");
        cotTypeMap.put("a-.-G-U-i-f-pp", "Portable Pump");
        cotTypeMap.put("a-.-G-U-i-f-st", "Strike Team, Engine (Fire)");
        cotTypeMap.put("a-.-G-U-i-f-wt", "Water Tender, Firefighting (Tanker)");
        cotTypeMap.put("a-.-G-U-i-h", "Health and Medical");
        cotTypeMap.put("a-.-G-U-i-h-dmb", "Disaster Medical Assistance Team - Basic");
        cotTypeMap.put("a-.-G-U-i-h-dmc", "Disaster Medical Assistance Team - Crush Injury Specialty");
        cotTypeMap.put("a-.-G-U-i-h-dmm", "Disaster Medical Assistance Team - Mental Health");
        cotTypeMap.put("a-.-G-U-i-h-dmp", "Disaster Medical Assistance Team - Pediatric");
        cotTypeMap.put("a-.-G-U-i-h-dms", "Disaster Medical Assistance Team - Burn");
        cotTypeMap.put("a-.-G-U-i-h-mor", "Disaster Mortuary Operational Response Team");
        cotTypeMap.put("a-.-G-U-i-h-msr", "International Medical Surgical Response Team");
        cotTypeMap.put("a-.-G-U-i-h-mst", "NDMS Management Support Team");
        cotTypeMap.put("a-.-G-U-i-h-vma", "Veterinary Medical Assistance Team");
        cotTypeMap.put("a-.-G-U-i-l", "Law Enforcement");
        cotTypeMap.put("a-.-G-U-i-l-bs", "Bomb Squad Explosives Team");
        cotTypeMap.put("a-.-G-U-i-l-cct", "Crowd Control Team");
        cotTypeMap.put("a-.-G-U-i-l-dt", "Public Safety Dive Team");
        cotTypeMap.put("a-.-G-U-i-l-hps", "Aviation-Helicopters Patrol Surveillance");
        cotTypeMap.put("a-.-G-U-i-l-oa", "Observation Aircraft");
        cotTypeMap.put("a-.-G-U-i-l-tt", "SWAT Tactical Team");
        cotTypeMap.put("a-.-G-U-i-p", "Public Works");
        cotTypeMap.put("a-.-G-U-i-p-acf", "Air Curtain Burners (Above Ground)");
        cotTypeMap.put("a-.-G-U-i-p-ach", "Air Conditioner Heater");
        cotTypeMap.put("a-.-G-U-i-p-act", "Air Curtain Burners (In Ground)");
        cotTypeMap.put("a-.-G-U-i-p-atc", "All Terrain Cranes");
        cotTypeMap.put("a-.-G-U-i-p-bh", "Backhoe Loader");
        cotTypeMap.put("a-.-G-U-i-p-cah", "Chillers Air Handlers");
        cotTypeMap.put("a-.-G-U-i-p-cc", "Crawler Cranes");
        cotTypeMap.put("a-.-G-U-i-p-cce", "Concrete Cutter Multi-Processor for Hydraulic Excavator");
        cotTypeMap.put("a-.-G-U-i-p-dat", "Disaster Assessment Team");
        cotTypeMap.put("a-.-G-U-i-p-dmm", "Debris Management Monitoring Team");
        cotTypeMap.put("a-.-G-U-i-p-dmr", "Debris Management Site Reduction Team");
        cotTypeMap.put("a-.-G-U-i-p-dmt", "Debris Management Team");
        cotTypeMap.put("a-.-G-U-i-p-drt", "Disaster Recovery Team");
        cotTypeMap.put("a-.-G-U-i-p-dt", "Dump Trailer");
        cotTypeMap.put("a-.-G-U-i-p-dtf", "Dump Truck Off Road");
        cotTypeMap.put("a-.-G-U-i-p-dtn", "Dump Truck On Road");
        cotTypeMap.put("a-.-G-U-i-p-epr", "Electrical Power Restoration Team");
        cotTypeMap.put("a-.-G-U-i-p-es", "Engineering Services");
        cotTypeMap.put("a-.-G-U-i-p-fbt", "Flat Bed Trailer Truck");
        cotTypeMap.put("a-.-G-U-i-p-gen", "Generators");
        cotTypeMap.put("a-.-G-U-i-p-hel", "Hydraulic Excavator (Large)");
        cotTypeMap.put("a-.-G-U-i-p-hem", "Hydraulic Excavator (Medium)");
        cotTypeMap.put("a-.-G-U-i-p-htc", "Hydraulic Truck Cranes");
        cotTypeMap.put("a-.-G-U-i-p-ltc", "Lattice Truck Cranes");
        cotTypeMap.put("a-.-G-U-i-p-tb", "Tug Boat");
        cotTypeMap.put("a-.-G-U-i-p-td", "Track Dozer");
        cotTypeMap.put("a-.-G-U-i-p-tg", "Tub Grinder");
        cotTypeMap.put("a-.-G-U-i-p-tt", "Tractor Trailer");
        cotTypeMap.put("a-.-G-U-i-p-wd", "Wheel Dozer");
        cotTypeMap.put("a-.-G-U-i-p-wll", "Wheel Loaders (Large)");
        cotTypeMap.put("a-.-G-U-i-p-wlm", "Wheel Loaders (Medium)");
        cotTypeMap.put("a-.-G-U-i-p-wls", "Wheel Loaders (Small)");
        cotTypeMap.put("a-.-G-U-i-p-wpt", "Water Purification Team");
        cotTypeMap.put("a-.-G-U-i-p-wt", "Water Truck");
        cotTypeMap.put("a-.-G-U-i-s", "Search and Rescue");
        cotTypeMap.put("a-.-G-U-i-s-ar", "Airborne Reconnaissance");
        cotTypeMap.put("a-.-G-U-i-s-ast", "Air Search Team");
        cotTypeMap.put("a-.-G-U-i-s-cas", "Canine SAR Team Avalanche Snow");
        cotTypeMap.put("a-.-G-U-i-s-cav", "Cave SAR Team");
        cotTypeMap.put("a-.-G-U-i-s-cdr", "Canine SAR Team Disaster Response");
        cotTypeMap.put("a-.-G-U-i-s-clc", "Canine SAR Team Land Cadaver");
        cotTypeMap.put("a-.-G-U-i-s-col", "Collapse SAR Team");
        cotTypeMap.put("a-.-G-U-i-s-cwa", "Canine SAR Team Water");
        cotTypeMap.put("a-.-G-U-i-s-cwi", "Canine SAR Team Wilderness");
        cotTypeMap.put("a-.-G-U-i-s-cwt", "Canine SAR Team Wilderness Tracking");
        cotTypeMap.put("a-.-G-U-i-s-mt", "Mine Tunnel SAR Team");
        cotTypeMap.put("a-.-G-U-i-s-mnt", "Mountain SAR Team");
        cotTypeMap.put("a-.-G-U-i-s-rdf", "Radio Direction Finding Team");
        cotTypeMap.put("a-.-G-U-i-s-sfd", "Swiftwater Flood Search and Dive Rescue Team");
        cotTypeMap.put("a-.-G-U-i-s-uis", "USAR Incident Support Team");
        cotTypeMap.put("a-.-G-U-i-s-utf", "USAR Task Force");
        cotTypeMap.put("a-.-G-U-i-s-wi", "Wilderness SAR Team");
        cotTypeMap.put("a-.-G-U-i-o", "Other");
        cotTypeMap.put("a-.-S", "SEA SURFACE TRACK");
        cotTypeMap.put("a-.-S-C", "COMBATANT");
        cotTypeMap.put("a-.-S-C-A", "AMPHIBIOUS WARFARE SHIP");
        cotTypeMap.put("a-.-S-C-A-L-A", "ASSAULT VESSEL");
        cotTypeMap.put("a-.-S-C-A-L-C", "LANDING CRAFT");
        cotTypeMap.put("a-.-S-C-A-L-S", "LANDING SHIP");
        cotTypeMap.put("a-.-S-C-H", "HOVERCRAFT");
        cotTypeMap.put("a-.-S-C-L", "LINE");
        cotTypeMap.put("a-.-S-C-L-B-B", "BATTLESHIP");
        cotTypeMap.put("a-.-S-C-L-C-C", "CRUISER");
        cotTypeMap.put("a-.-S-C-L-C-V", "CARRIER");
        cotTypeMap.put("a-.-S-C-L-D-D", "DESTROYER");
        cotTypeMap.put("a-.-S-C-L-F-F", "FRIGATE/CORVETTE");
        cotTypeMap.put("a-.-S-C-M", "MINE WARFARE VESSEL");
        cotTypeMap.put("a-.-S-C-M-M-A", "MCM SUPPORT");
        cotTypeMap.put("a-.-S-C-M-M-D", "MCM DRONE");
        cotTypeMap.put("a-.-S-C-M-M-H", "MINEHUNTER");
        cotTypeMap.put("a-.-S-C-M-M-L", "MINELAYER");
        cotTypeMap.put("a-.-S-C-M-M-S", "MINESWEEPER");
        cotTypeMap.put("a-.-S-C-P", "PATROL");
        cotTypeMap.put("a-.-S-C-P-S-B", "ANTISUBMARINE WARFARE");
        cotTypeMap.put("a-.-S-C-P-S-U", "ANTISURFACE WARFARE");
        cotTypeMap.put("a-.-S-G", "NAVY GROUP");
        cotTypeMap.put("a-.-S-G-C", "CONVOY");
        cotTypeMap.put("a-.-S-G-G", "NAVY TASK GROUP");
        cotTypeMap.put("a-.-S-G-T", "NAVY TASK FORCE");
        cotTypeMap.put("a-.-S-G-U", "NAVY TASK UNIT");
        cotTypeMap.put("a-.-S-N", "NONCOMBATANT");
        cotTypeMap.put("a-.-S-N-F", "FLEET SUPPORT");
        cotTypeMap.put("a-.-S-N-H", "HOVERCRAFT");
        cotTypeMap.put("a-.-S-N-I", "INTELLIGENCE");
        cotTypeMap.put("a-.-S-N-M", "HOSPITAL SHIP");
        cotTypeMap.put("a-.-S-N-N", "STATION");
        cotTypeMap.put("a-.-S-N-N-R", "RESCUE");
        cotTypeMap.put("a-.-S-N-R", "UNDERWAY REPLENISHMENT");
        cotTypeMap.put("a-.-S-N-S", "SERVICE/SUPPORT HARBOR");
        cotTypeMap.put("a-.-S-O", "OWN TRACK");
        cotTypeMap.put("a-.-S-S", "STATION");
        cotTypeMap.put("a-.-S-S-A", "ASW SHIP");
        cotTypeMap.put("a-.-S-S-P", "PICKET");
        cotTypeMap.put("a-.-S-X", "NON MILITARY");
        cotTypeMap.put("a-.-S-X-F", "FISHING");
        cotTypeMap.put("a-.-S-X-F-D-F", "DRIFTER");
        cotTypeMap.put("a-.-S-X-F-D-R", "DREDGE");
        cotTypeMap.put("a-.-S-X-F-T-R", "TRAWLER");
        cotTypeMap.put("a-.-S-X-H", "HOVERCRAFT");
        cotTypeMap.put("a-.-S-X-L", "LAW ENFORCEMENT VESSEL");
        cotTypeMap.put("a-.-S-X-M", "MERCHANT");
        cotTypeMap.put("a-.-S-X-M-C", "CARGO");
        cotTypeMap.put("a-.-S-X-M-F", "FERRY");
        cotTypeMap.put("a-.-S-X-M-H", "HAZARDOUS MATERIALS (HAZMAT)");
        cotTypeMap.put("a-.-S-X-M-O", "OILER/TANKER");
        cotTypeMap.put("a-.-S-X-M-P", "PASSENGER");
        cotTypeMap.put("a-.-S-X-M-R", "ROLL ON/ROLL OFF");
        cotTypeMap.put("a-.-S-X-M-T-O", "TOWING VESSEL");
        cotTypeMap.put("a-.-S-X-M-T-U", "TUG");
        cotTypeMap.put("a-.-S-X-R", "LEISURE CRAFT");
        cotTypeMap.put("a-.-U", "SUBSURFACE TRACK");
        cotTypeMap.put("a-.-U-N", "NON SUBMARINE");
        cotTypeMap.put("a-.-U-N-D", "DIVER");
        cotTypeMap.put("a-.-U-S", "SUBMARINE");
        cotTypeMap.put("a-.-U-S-C", "SUBMARINE CONVENTIONAL PROPULSION");
        cotTypeMap.put("a-.-U-S-N", "SUBMARINE NUCLEAR PROPULSION");
        cotTypeMap.put("a-.-U-S-O", "OTHER SUBMERSIBLE");
        cotTypeMap.put("a-.-U-S-S", "STATION");
        cotTypeMap.put("a-.-U-S-S-A", "ASW SUBMARINE");
        cotTypeMap.put("a-.-U-S-U", "UNMANNED UNDERWATER VEHICLE (UUV)");
        cotTypeMap.put("a-.-U-W", "UNDERWATER WEAPON");
        cotTypeMap.put("a-.-U-W-D", "UNDERWATER DECOY");
        cotTypeMap.put("a-.-U-W-D-M", "SEA MINE DECOY");
        cotTypeMap.put("a-.-U-W-M", "SEA MINE");
        cotTypeMap.put("a-.-U-W-M-D", "SEA MINE DEALT");
        cotTypeMap.put("a-.-U-W-M-F", "SEA MINE (FLOATING)");
        cotTypeMap.put("a-.-U-W-M-F-D", "SEA MINE (FLOATING) DEALT");
        cotTypeMap.put("a-.-U-W-M-G", "SEA MINE (GROUND)");
        cotTypeMap.put("a-.-U-W-M-G-D", "SEA MINE (GROUND) DEALT");
        cotTypeMap.put("a-.-U-W-M-M", "SEA MINE (MOORED)");
        cotTypeMap.put("a-.-U-W-M-M-D", "SEA MINE (MOORED) DEALT");
        cotTypeMap.put("a-.-U-W-M-O", "SEA MINE (IN OTHER POSITION)");
        cotTypeMap.put("a-.-U-W-M-O-D", "SEA MINE (IN OTHER POSITION) DEALT");
        cotTypeMap.put("a-.-U-W-T", "TORPEDO");
        cotTypeMap.put("a-.-X", "Other");
        cotTypeMap.put("a-.-X-i", "Incident");
        cotTypeMap.put("a-.-X-i-g", "Geophysical");
        cotTypeMap.put("a-.-X-i-g-a", "Geophysical Avalanche");
        cotTypeMap.put("a-.-X-i-g-e", "Geophysical Earthquake");
        cotTypeMap.put("a-.-X-i-g-e-a", "Geophysical Earthquake Aftershock");
        cotTypeMap.put("a-.-X-i-g-e-e", "Geophysical Earthquake Epicenter");
        cotTypeMap.put("a-.-X-i-g-l", "Geophysical Landslide");
        cotTypeMap.put("a-.-X-i-g-s", "Geophysical Subsistance");
        cotTypeMap.put("a-.-X-i-g-v", "Geophysical Volcano");
        cotTypeMap.put("a-.-X-i-g-v-e", "Geophysical Volcano Eruption");
        cotTypeMap.put("a-.-X-i-g-v-t", "Geophysical Volcano Threat");
        cotTypeMap.put("a-.-X-i-m", "Meteorological");
        cotTypeMap.put("a-.-X-i-m-z", "Meteorological Drizzle");
        cotTypeMap.put("a-.-X-i-m-d", "Meteorological Drought");
        cotTypeMap.put("a-.-X-i-m-f", "Meteorological Flood");
        cotTypeMap.put("a-.-X-i-m-g", "Meteorological Fog");
        cotTypeMap.put("a-.-X-i-m-h", "Meteorological Hail");
        cotTypeMap.put("a-.-X-i-m-i", "Meteorological Inversion");
        cotTypeMap.put("a-.-X-i-m-r", "Meteorological Rain");
        cotTypeMap.put("a-.-X-i-m-a", "Meteorological Sandstorm");
        cotTypeMap.put("a-.-X-i-m-s", "Meteorological Snow");
        cotTypeMap.put("a-.-X-i-m-u", "Meteorological Thunderstorm");
        cotTypeMap.put("a-.-X-i-m-t", "Meteorological Tornado");
        cotTypeMap.put("a-.-X-i-m-c", "Meteorological Cyclone");
        cotTypeMap.put("a-.-X-i-m-n", "Meteorological Tsunami");
        cotTypeMap.put("a-.-X-i-s", "Safety");
        cotTypeMap.put("a-.-X-i-l", "Security");
        cotTypeMap.put("a-.-X-i-l-c", "Security Civil Disturbance");
        cotTypeMap.put("a-.-X-i-l-c-d", "Security Civil Disturbance Demonstration");
        cotTypeMap.put("a-.-X-i-l-c-p", "Security Civil Disturbance Displaced Population");
        cotTypeMap.put("a-.-X-i-l-c-r", "Security Civil Disturbance Rioting");
        cotTypeMap.put("a-.-X-i-l-l", "Security Law Enforcement Activity");
        cotTypeMap.put("a-.-X-i-l-l-b", "Security Law Enforcement Activity Bombing");
        cotTypeMap.put("a-.-X-i-l-l-b-e", "Security Law Enforcement Activity Bombing Explosion");
        cotTypeMap.put("a-.-X-i-l-l-b-t", "Security Law Enforcement Activity Bombing Threat");
        cotTypeMap.put("a-.-X-i-l-l-l", "Security Law Enforcement Activity Looting");
        cotTypeMap.put("a-.-X-i-l-l-p", "Security Law Enforcement Activity Poisoning");
        cotTypeMap.put("a-.-X-i-l-l-s", "Security Law Enforcement Activity Shooting");
        cotTypeMap.put("a-.-X-i-r", "Rescue");
        cotTypeMap.put("a-.-X-i-f", "Incident Fire");
        cotTypeMap.put("a-.-X-i-f-w", "Incident Fire Wild");
        cotTypeMap.put("a-.-X-i-f-n", "Incident Fire Non-residential");
        cotTypeMap.put("a-.-X-i-f-n-h", "Incident Fire Non-residential Special Need");
        cotTypeMap.put("a-.-X-i-f-n-s", "Incident Fire Non-residential School");
        cotTypeMap.put("a-.-X-i-f-r", "Incident Fire Residential");
        cotTypeMap.put("a-.-X-i-f-h", "Incident Fire Hot Spot");
        cotTypeMap.put("a-.-X-i-f-o", "Incident Fire Origin");
        cotTypeMap.put("a-.-X-i-f-s", "Incident Fire Smoke");
        cotTypeMap.put("a-.-X-i-f-p", "Incident Fire Special Needs");
        cotTypeMap.put("a-.-X-i-h", "Medical and Public Health");
        cotTypeMap.put("a-.-X-i-e", "Pollution and other Environmental");
        cotTypeMap.put("a-.-X-i-t", "Public and Private transportation");
        cotTypeMap.put("a-.-X-i-t-a", "Public and Private transportation Air Incident");
        cotTypeMap.put("a-.-X-i-t-a-a", "Public and Private transportation Air Accident");
        cotTypeMap.put("a-.-X-i-t-a-h", "Public and Private transportation Air Hijacking");
        cotTypeMap.put("a-.-X-i-t-m", "Public and Private transportation Maritime Incident");
        cotTypeMap.put("a-.-X-i-t-m-a", "Public and Private transportation Maritime Accident");
        cotTypeMap.put("a-.-X-i-t-m-h", "Public and Private transportation Maritime Hijacking");
        cotTypeMap.put("a-.-X-i-t-r", "Public and Private transportation Rail Incident");
        cotTypeMap.put("a-.-X-i-t-r-a", "Public and Private transportation Rail Accident");
        cotTypeMap.put("a-.-X-i-t-r-h", "Public and Private transportation Rail Hijacking");
        cotTypeMap.put("a-.-X-i-t-v", "Public and Private transportation Vehicle Incident");
        cotTypeMap.put("a-.-X-i-t-v-a", "Public and Private transportation Vehicle Accident");
        cotTypeMap.put("a-.-X-i-t-v-h", "Public and Private transportation Vehicle Hijacking");
        cotTypeMap.put("a-.-X-i-i", "Infrastructure");
        cotTypeMap.put("a-.-X-i-c", "CBRNE");
        cotTypeMap.put("a-.-X-i-o", "Other");
        cotTypeMap.put("a-.-.*-9-1-1", "MAYDAY");
        cotTypeMap.put("b", "Bits");
        cotTypeMap.put("b-i", "Image");
        cotTypeMap.put("b-m-r", "route");
        cotTypeMap.put("b-m-p", "map point");
        cotTypeMap.put("b-m-p-t", "targeted");
        cotTypeMap.put("b-m-p-w", "waypoint");
        cotTypeMap.put("b-m-p-m-c", "click");
        cotTypeMap.put("b-m-p-s-p-i", "spi");
        cotTypeMap.put("b-m-p-v-p-i", "vpoi");
        cotTypeMap.put("b-m-g-o", "grid");
        cotTypeMap.put("b-d", "Detection");
        cotTypeMap.put("b-d-a", "Detection/Acoustic");
        cotTypeMap.put("b-d-a-i", "Detection/Acoustic/Impulsive");
        cotTypeMap.put("b-d-a-v", "Detection/Acoustic/Voice");
        cotTypeMap.put("b-d-a-c", "Detection/Acoustic/Cyclostationary");
        cotTypeMap.put("b-d-m", "Detection/Motion");
        cotTypeMap.put("b-d-s", "Detection/Seismic");
        cotTypeMap.put("b-d-r", "Detection/Radiaition");
        cotTypeMap.put("b-d-n", "Detection/Nuclear");
        cotTypeMap.put("b-d-c", "Detection/CBRNE");
        cotTypeMap.put("b-d-c-b", "Detection/CBRNE/BioChem");
        cotTypeMap.put("b-d-c-b-b", "Detection/CBRNE/BioChem/Biological");
        cotTypeMap.put("b-d-c-b-c", "Detection/CBRNE/BioChem/Chemical");
        cotTypeMap.put("b-d-c-e", "Detection/CBRNE/Explosive");
        cotTypeMap.put("b-d-c-e-d", "Detection/CBRNE/Explosive/Device");
        cotTypeMap.put("b-d-c-n", "Detection/CBRNE/NuclearRadiological");
        cotTypeMap.put("b-d-c-n-n", "Detection/CBRNE/NuclearRadiological/Nuclear");
        cotTypeMap.put("b-d-c-n-n-b", "Detection/CBRNE/NuclearRadiological/Nuclear/Bomb");
        cotTypeMap.put("b-d-c-n-n-sm", "Detection/CBRNE/NuclearRadiological/Nuclear/Special Nuclear Material");
        cotTypeMap.put("b-d-c-n-r", "Detection/CBRNE/NuclearRadiological/Radiation");
        cotTypeMap.put("b-d-c-n-r-dd", "Detection/CBRNE/NuclearRadiological/Radiological/Dispersal Device (Dirty Bomb)");
        cotTypeMap.put("b-d-l", "Detection/Launch");
        cotTypeMap.put("b-d-l-b", "Detection/Launch/Bullet");
        cotTypeMap.put("b-d-l-m", "Detection/Launch/Mortar");
        cotTypeMap.put("b-d-i", "Detection/Impact");
        cotTypeMap.put("b-d-i-m", "Detection/Impact/Mortar");
        cotTypeMap.put("b-l", "Alarm");
        cotTypeMap.put("b-l-c", "Alarm/CBRNE");
        cotTypeMap.put("b-l-c-b", "Alarm/CBRNE/BioChem");
        cotTypeMap.put("b-l-c-b-b", "Alarm/CBRNE/BioChem/Biological");
        cotTypeMap.put("b-l-c-b-c", "Alarm/CBRNE/BioChem/Chemical");
        cotTypeMap.put("b-l-c-e", "Alarm/CBRNE/Explosive");
        cotTypeMap.put("b-l-c-e-d", "Alarm/CBRNE/Explosive/Device");
        cotTypeMap.put("b-l-c-n", "Alarm/CBRNE/NuclearRadiological");
        cotTypeMap.put("b-l-c-n-n", "Alarm/CBRNE/NuclearRadiological/Nuclear");
        cotTypeMap.put("b-l-c-n-n-b", "Alarm/CBRNE/NuclearRadiological/Nuclear/Bomb");
        cotTypeMap.put("b-l-c-n-n-sm", "Alarm/CBRNE/NuclearRadiological/Nuclear/Special Nuclear Material");
        cotTypeMap.put("b-l-c-n-r", "Alarm/CBRNE/NuclearRadiological/Radiological");
        cotTypeMap.put("b-l-c-n-r-dd", "Alarm/CBRNE/Radiological/Dispersal Device (Dirty Bomb)");
        cotTypeMap.put("b-l-e", "Alarm/Environmental");
        cotTypeMap.put("b-l-e-h", "Alarm/Environmental/Hazmat");
        cotTypeMap.put("b-l-f", "Alarm/Fire");
        cotTypeMap.put("b-l-f-a", "Alarm/Fire/Audible");
        cotTypeMap.put("b-l-f-a-a", "Alarm/Fire/Audible/Pump Activated");
        cotTypeMap.put("b-l-f-a-c", "Alarm/Fire/Audible/Combustion");
        cotTypeMap.put("b-l-f-a-d", "Alarm/Fire/Audible/Duct Detector");
        cotTypeMap.put("b-l-f-a-f", "Alarm/Fire/Audible/Flame Detector");
        cotTypeMap.put("b-l-f-a-h", "Alarm/Fire/Audible/Heat");
        cotTypeMap.put("b-l-f-a-p", "Alarm/Fire/Audible/Pull Station");
        cotTypeMap.put("b-l-f-a-s", "Alarm/Fire/Audible/Smoke");
        cotTypeMap.put("b-l-f-a-w", "Alarm/Fire/Audible/Waterflow");
        cotTypeMap.put("b-l-g", "Alarm/Geophysical");
        cotTypeMap.put("b-l-h", "Alarm/Medical and Public Health");
        cotTypeMap.put("b-l-h-a", "Alarm/Medical and Public Health/Audible");
        cotTypeMap.put("b-l-h-am", "Alarm/Medical and Public Health/Ambulance");
        cotTypeMap.put("b-l-h-am-s", "Alarm/Medical and Public Health/Ambulance/Silent");
        cotTypeMap.put("b-l-i", "Alarm/Infrastructure");
        cotTypeMap.put("b-l-l", "Alarm/Security");
        cotTypeMap.put("b-l-l-l", "Alarm/Security/Law Enforcement");
        cotTypeMap.put("b-l-l-l-ad", "Alarm/Security/Law Enforcement/Armed and Dangerous");
        cotTypeMap.put("b-l-l-l-an", "Alarm/Security/Law Enforcement/Animal");
        cotTypeMap.put("b-l-l-l-an-or", "Alarm/Security/Law Enforcement/Animal/On Roadway");
        cotTypeMap.put("b-l-l-l-as", "Alarm/Security/Law Enforcement/Assualt");
        cotTypeMap.put("b-l-l-l-av", "Alarm/Security/Law Enforcement/Abandoned Vehicle");
        cotTypeMap.put("b-l-l-l-ba", "Alarm/Security/Law Enforcement/Battery");
        cotTypeMap.put("b-l-l-l-bt", "Alarm/Security/Law Enforcement/Bomb Threat");
        cotTypeMap.put("b-l-l-l-bur", "Alarm/Security/Law Enforcement/Burglary");
        cotTypeMap.put("b-l-l-l-bur-a", "Alarm/Security/Law Enforcement/Burglary/Audible");
        cotTypeMap.put("b-l-l-l-bur-a-d", "Alarm/Security/Law Enforcement/Burglary/Audible/Day/Night Zone");
        cotTypeMap.put("b-l-l-l-bur-a-e", "Alarm/Security/Law Enforcement/Burglary/Audible/Entry/Exit");
        cotTypeMap.put("b-l-l-l-bur-a-i", "Alarm/Security/Law Enforcement/Burglary/Audible/Interior");
        cotTypeMap.put("b-l-l-l-bur-a-o", "Alarm/Security/Law Enforcement/Burglary/Audible/Outdoor");
        cotTypeMap.put("b-l-l-l-bur-a-p", "Alarm/Security/Law Enforcement/Burglary/Audible/Perimeter");
        cotTypeMap.put("b-l-l-l-bur-a-t", "Alarm/Security/Law Enforcement/Burglary/Audible/24 Hour Zone");
        cotTypeMap.put("b-l-l-l-bur-a-x", "Alarm/Security/Law Enforcement/Burglary/Audible/Exit Error");
        cotTypeMap.put("b-l-l-l-bur-s", "Alarm/Security/Law Enforcement/Burglary/Silent");
        cotTypeMap.put("b-l-l-l-bur-s-c", "Alarm/Security/Law Enforcement/Burglary/Silent/Recent Close");
        cotTypeMap.put("b-l-l-l-c", "Alarm/Security/Law Enforcement/Crash");
        cotTypeMap.put("b-l-l-l-c-a", "Alarm/Security/Law Enforcement/Crash/Aircraft");
        cotTypeMap.put("b-l-l-l-c-wf", "Alarm/Security/Law Enforcement/Crash/With Fatalities");
        cotTypeMap.put("b-l-l-l-c-wi", "Alarm/Security/Law Enforcement/Crash/With injuries");
        cotTypeMap.put("b-l-l-l-c-wr", "Alarm/Security/Law Enforcement/Crash/With Roadblock");
        cotTypeMap.put("b-l-l-l-c-pc", "Alarm/Security/Law Enforcement/Crash/Patrol Car");
        cotTypeMap.put("b-l-l-l-ca", "Alarm/Security/Law Enforcement/Child Abuse");
        cotTypeMap.put("b-l-l-l-cd", "Alarm/Security/Law Enforcement/Civil Disturbance or Disorder");
        cotTypeMap.put("b-l-l-l-cr", "Alarm/Security/Law Enforcement/Callbox Request");
        cotTypeMap.put("b-l-l-l-dc", "Alarm/Security/Law Enforcement/Drug or Contraband");
        cotTypeMap.put("b-l-l-l-dov", "Alarm/Security/Law Enforcement/Domestic Violence");
        cotTypeMap.put("b-l-l-l-dp", "Alarm/Security/Law Enforcement/Dead Person");
        cotTypeMap.put("b-l-l-l-drp", "Alarm/Security/Law Enforcement/Drunk Pedestrian");
        cotTypeMap.put("b-l-l-l-dui", "Alarm/Security/Law Enforcement/DUI");
        cotTypeMap.put("b-l-l-l-dui-j", "Alarm/Security/Law Enforcement/DUI/Juvenile");
        cotTypeMap.put("b-l-l-l-dv", "Alarm/Security/Law Enforcement/Disabled Vehicle");
        cotTypeMap.put("b-l-l-l-ep", "Alarm/Security/Law Enforcement/Escaped Prisoner");
        cotTypeMap.put("b-l-l-l-hr", "Alarm/Security/Law Enforcement/Hit and Run");
        cotTypeMap.put("b-l-l-l-hr-wi", "Alarm/Security/Law Enforcement/Hit and Run/With Injuries");
        cotTypeMap.put("b-l-l-l-hr-wr", "Alarm/Security/Law Enforcement/Hit and Run/With Roadblock");
        cotTypeMap.put("b-l-l-l-hup", "Alarm/Security/Law Enforcement/Hold Up");
        cotTypeMap.put("b-l-l-l-hup-a", "Alarm/Security/Law Enforcement/Hold Up/Audible");
        cotTypeMap.put("b-l-l-l-hup-s", "Alarm/Security/Law Enforcement/Hold Up/Silent");
        cotTypeMap.put("b-l-l-l-ii", "Alarm/Security/Law Enforcement/Information or Intelligence - calls to take witness info");
        cotTypeMap.put("b-l-l-l-io", "Alarm/Security/Law Enforcement/Impersonating an Officer");
        cotTypeMap.put("b-l-l-l-jo", "Alarm/Security/Law Enforcement/Juvenile Offense");
        cotTypeMap.put("b-l-l-l-k", "Alarm/Security/Law Enforcement/Kidnapping");
        cotTypeMap.put("b-l-l-l-l", "Alarm/Security/Law Enforcement/Larceny");
        cotTypeMap.put("b-l-l-l-m", "Alarm/Security/Law Enforcement/Murder");
        cotTypeMap.put("b-l-l-l-mi", "Alarm/Security/Law Enforcement/Mentally Ill Person");
        cotTypeMap.put("b-l-l-l-mm", "Alarm/Security/Law Enforcement/Malacious Mischief");
        cotTypeMap.put("b-l-l-l-mp", "Alarm/Security/Law Enforcement/Missing Person");
        cotTypeMap.put("b-l-l-l-od", "Alarm/Security/Law Enforcement/Officer Down");
        cotTypeMap.put("b-l-l-l-ph", "Alarm/Security/Law Enforcement/Pedestrian or Hitchhiker");
        cotTypeMap.put("b-l-l-l-r", "Alarm/Security/Law Enforcement/Robbery");
        cotTypeMap.put("b-l-l-l-r-sa", "Alarm/Security/Law Enforcement/Robbery/Strongarm");
        cotTypeMap.put("b-l-l-l-rd", "Alarm/Security/Law Enforcement/Reckless Driving");
        cotTypeMap.put("b-l-l-l-rt", "Alarm/Security/Law Enforcement/Rock Throwing");
        cotTypeMap.put("b-l-l-l-s", "Alarm/Security/Law Enforcement/Shooting");
        cotTypeMap.put("b-l-l-l-sc", "Alarm/Security/Law Enforcement/Suspicious Circumstances");
        cotTypeMap.put("b-l-l-l-sd", "Alarm/Security/Law Enforcement/Special Detail");
        cotTypeMap.put("b-l-l-l-sg", "Alarm/Security/Law Enforcement/Smuggling");
        cotTypeMap.put("b-l-l-l-si", "Alarm/Security/Law Enforcement/Sick or Injured Person");
        cotTypeMap.put("b-l-l-l-so", "Alarm/Security/Law Enforcement/Sex Offense");
        cotTypeMap.put("b-l-l-l-sp", "Alarm/Security/Law Enforcement/Suspicious Person");
        cotTypeMap.put("b-l-l-l-su", "Alarm/Security/Law Enforcement/Suicide");
        cotTypeMap.put("b-l-l-l-sv", "Alarm/Security/Law Enforcement/Suspicious Vehcle");
        cotTypeMap.put("b-l-l-l-te", "Alarm/Security/Law Enforcement/Toll Evasion");
        cotTypeMap.put("b-l-l-l-vt", "Alarm/Security/Law Enforcement/Lost or Stolen Vehicle Tag");
        cotTypeMap.put("b-l-m", "Alarm/Meteorological");
        cotTypeMap.put("b-l-o", "Alarm/Other");
        cotTypeMap.put("b-l-o-byp", "Alarm/Other/Bypass");
        cotTypeMap.put("b-l-o-byp-s", "Alarm/Other/Bypass/Silent");
        cotTypeMap.put("b-l-o-byp-s-24", "Alarm/Other/Bypass/Silent/24 Hour zone");
        cotTypeMap.put("b-l-o-byp-s-b", "Alarm/Other/Bypass/Silent/Burglary");
        cotTypeMap.put("b-l-o-byp-s-f", "Alarm/Other/Bypass/Silent/Fire");
        cotTypeMap.put("b-l-o-byp-s-g", "Alarm/Other/Bypass/Silent/Group");
        cotTypeMap.put("b-l-o-byp-s-sb", "Alarm/Other/Bypass/Silent/Swinger Bypassed");
        cotTypeMap.put("b-l-o-byp-s-zs", "Alarm/Other/Bypass/Silent/Zone or Sensor");
        cotTypeMap.put("b-l-o-can", "Alarm/Other/Cancel");
        cotTypeMap.put("b-l-o-can-s", "Alarm/Other/Cancel/Silent");
        cotTypeMap.put("b-l-o-clo", "Alarm/Other/Close");
        cotTypeMap.put("b-l-o-clo-s", "Alarm/Other/Close/Silent");
        cotTypeMap.put("b-l-o-clo-s-sm", "Alarm/Other/Close/Silent/Stay Mode");
        cotTypeMap.put("b-l-o-far", "Alarm/Other/Forced Arming");
        cotTypeMap.put("b-l-o-far-s", "Alarm/Other/Forced Arming/Silent");
        cotTypeMap.put("b-l-o-far-s-pa", "Alarm/Other/Forced Arming/Silent/Partial Arm");
        cotTypeMap.put("b-l-o-ftt", "Alarm/Other/Failure to Test");
        cotTypeMap.put("b-l-o-ftt-s", "Alarm/Other/Failure to Test/Silent");
        cotTypeMap.put("b-l-o-ftt-s-pn", "Alarm/Other/Failure to Test/Silent/Point not Tested");
        cotTypeMap.put("b-l-o-log", "Alarm/Other/Log");
        cotTypeMap.put("b-l-o-log-s", "Alarm/Other/Log/Silent");
        cotTypeMap.put("b-l-o-log-s-ad", "Alarm/Other/Log/Silent/Access Denied");
        cotTypeMap.put("b-l-o-log-s-ap", "Alarm/Other/Log/Silent/Access Permitted");
        cotTypeMap.put("b-l-o-log-s-as", "Alarm/Other/Log/Silent/Access Schedule Change");
        cotTypeMap.put("b-l-o-log-s-cr", "Alarm/Other/Log/Silent/Callback Requested");
        cotTypeMap.put("b-l-o-log-s-ds", "Alarm/Other/Log/Silent/Dialer Shutdown");
        cotTypeMap.put("b-l-o-log-s-er", "Alarm/Other/Log/Silent/Event Log Reset");
        cotTypeMap.put("b-l-o-log-s-es", "Alarm/Other/Log/Silent/Exception Schedule Change");
        cotTypeMap.put("b-l-o-log-s-pe", "Alarm/Other/Log/Silent/Program Mode Entry");
        cotTypeMap.put("b-l-o-log-s-px", "Alarm/Other/Log/Silent/Program Mode Exit");
        cotTypeMap.put("b-l-o-log-s-sa", "Alarm/Other/Log/Silent/Successful Access");
        cotTypeMap.put("b-l-o-log-s-sr", "Alarm/Other/Log/Silent/Status Report to Follow");
        cotTypeMap.put("b-l-o-log-s-ss", "Alarm/Other/Log/Silent/System shutdown");
        cotTypeMap.put("b-l-o-log-s-tr", "Alarm/Other/Log/Silent/Time/Date Reset");
        cotTypeMap.put("b-l-o-log-s-ua", "Alarm/Other/Log/Silent/Unsuccessful Access");
        cotTypeMap.put("b-l-o-log-s-vx", "Alarm/Other/Log/Silent/Video Transmitter Active");
        cotTypeMap.put("b-l-o-ltc", "Alarm/Other/Late to Close");
        cotTypeMap.put("b-l-o-ltc-s", "Alarm/Other/Late to Close/Silent");
        cotTypeMap.put("b-l-o-ltc-s-af", "Alarm/Other/Late to Close/Silent/Auto Armed Failed");
        cotTypeMap.put("b-l-o-lto", "Alarm/Other/Late to Open");
        cotTypeMap.put("b-l-o-lto-s", "Alarm/Other/Late to Open/Silent");
        cotTypeMap.put("b-l-o-nul", "Alarm/Other/Null");
        cotTypeMap.put("b-l-o-nul-s", "Alarm/Other/Null/Silent");
        cotTypeMap.put("b-l-o-nul-s-p", "Alarm/Other/Null/Silent/Periodic");
        cotTypeMap.put("b-l-o-nul-s-pr", "Alarm/Other/Null/Silent/Periodic Radio");
        cotTypeMap.put("b-l-o-opn", "Alarm/Other/Open");
        cotTypeMap.put("b-l-o-opn-s", "Alarm/Other/Open/Silent");
        cotTypeMap.put("b-l-o-opn-s-a", "Alarm/Other/Open/Silent/Automatic");
        cotTypeMap.put("b-l-o-opn-s-d", "Alarm/Other/Open/Silent/Deferred");
        cotTypeMap.put("b-l-o-opn-s-e", "Alarm/Other/Open/Silent/Early");
        cotTypeMap.put("b-l-o-opn-s-ex", "Alarm/Other/Open/Silent/Exception");
        cotTypeMap.put("b-l-o-opn-s-g", "Alarm/Other/Open/Silent/Group");
        cotTypeMap.put("b-l-o-opn-s-k", "Alarm/Other/Open/Silent/Keyswitch");
        cotTypeMap.put("b-l-o-opn-s-l", "Alarm/Other/Open/Silent/Late");
        cotTypeMap.put("b-l-o-opn-s-qa", "Alarm/Other/Open/Silent/Quick Arm");
        cotTypeMap.put("b-l-o-opn-s-r", "Alarm/Other/Open/Silent/Remote");
        cotTypeMap.put("b-l-o-opn-s-u", "Alarm/Other/Open/Silent/By User");
        cotTypeMap.put("b-l-o-opn-s-up", "Alarm/Other/Open/Silent/User on Premises");
        cotTypeMap.put("b-l-o-pan", "Alarm/Other/Panic");
        cotTypeMap.put("b-l-o-res", "Alarm/Other/Reset");
        cotTypeMap.put("b-l-o-res-s", "Alarm/Other/Reset/Silent");
        cotTypeMap.put("b-l-o-res-s-er", "Alarm/Other/Reset/Silent/Exp Mod Reset");
        cotTypeMap.put("b-l-o-spc", "Alarm/Other/Special");
        cotTypeMap.put("b-l-o-spc-a", "Alarm/Other/Special/Audible");
        cotTypeMap.put("b-l-o-spc-a-a", "Alarm/Other/Special/Audible/General Alarm");
        cotTypeMap.put("b-l-o-spc-a-e", "Alarm/Other/Special/Audible/Personal Emergency");
        cotTypeMap.put("b-l-o-spc-a-f", "Alarm/Other/Special/Audible/Air Flow Loss");
        cotTypeMap.put("b-l-o-spc-a-g", "Alarm/Other/Special/Audible/Gas Detected");
        cotTypeMap.put("b-l-o-spc-a-l", "Alarm/Other/Special/Audible/Low Gas Level");
        cotTypeMap.put("b-l-o-spc-a-w", "Alarm/Other/Special/Audible/Water Leakage");
        cotTypeMap.put("b-l-o-spc-s", "Alarm/Other/Special/Silent");
        cotTypeMap.put("b-l-o-spc-s-f", "Alarm/Other/Special/Silent/Fail to Report");
        cotTypeMap.put("b-l-o-spc-s-li", "Alarm/Other/Special/Silent/Listen in to Follow");
        cotTypeMap.put("b-l-o-sps", "Alarm/Other/Supress");
        cotTypeMap.put("b-l-o-sps-a", "Alarm/Other/Supress/Audible");
        cotTypeMap.put("b-l-o-sps-a-g", "Alarm/Other/Supress/Audible/Gate Valve Tamper");
        cotTypeMap.put("b-l-o-sps-s", "Alarm/Other/Supress/Silent");
        cotTypeMap.put("b-l-o-sps-s-ar", "Alarm/Other/Supress/Silent/Alarm Relay Disabled");
        cotTypeMap.put("b-l-o-sps-s-b1", "Alarm/Other/Supress/Silent/Bell 1 Disabled");
        cotTypeMap.put("b-l-o-sps-s-b2", "Alarm/Other/Supress/Silent/Bell 2 Disabled");
        cotTypeMap.put("b-l-o-sps-s-dd", "Alarm/Other/Supress/Silent/Dialer Disabled");
        cotTypeMap.put("b-l-o-sps-s-rr", "Alarm/Other/Supress/Silent/Reversing Relay Disabled");
        cotTypeMap.put("b-l-o-sps-s-rx", "Alarm/Other/Supress/Silent/Radio Transmitter Disabled");
        cotTypeMap.put("b-l-o-sps-s-sr", "Alarm/Other/Supress/Silent/Sounder relay Disabled");
        cotTypeMap.put("b-l-o-sps-s-tr", "Alarm/Other/Supress/Silent/Trouble Relay Disabled");
        cotTypeMap.put("b-l-o-tam", "Alarm/Other/Tamper");
        cotTypeMap.put("b-l-o-tam-a", "Alarm/Other/Tamper/Audible");
        cotTypeMap.put("b-l-o-tam-s", "Alarm/Other/Tamper/Silent");
        cotTypeMap.put("b-l-o-tam-s-e", "Alarm/Other/Tamper/Silent/Expansion Module");
        cotTypeMap.put("b-l-o-tam-s-p", "Alarm/Other/Tamper/Silent/Panel Program Change");
        cotTypeMap.put("b-l-o-tam-s-s", "Alarm/Other/Tamper/Silent/Sensor");
        cotTypeMap.put("b-l-o-tam-s-v", "Alarm/Other/Tamper/Silent/Verify");
        cotTypeMap.put("b-l-o-tbl", "Alarm/Other/Trouble");
        cotTypeMap.put("b-l-o-tbl-a", "Alarm/Other/Trouble/Audible");
        cotTypeMap.put("b-l-o-tbl-a-b", "Alarm/Other/Trouble/Audible/Foil Break");
        cotTypeMap.put("b-l-o-tbl-a-c", "Alarm/Other/Trouble/Audible/Low CO2");
        cotTypeMap.put("b-l-o-tbl-a-d", "Alarm/Other/Trouble/Audible/Day Trouble");
        cotTypeMap.put("b-l-o-tbl-a-ee", "Alarm/Other/Trouble/Audible/Exit error");
        cotTypeMap.put("b-l-o-tbl-a-l", "Alarm/Other/Trouble/Audible/Low Water Level");
        cotTypeMap.put("b-l-o-tbl-a-p", "Alarm/Other/Trouble/Audible/Low Water Pressure");
        cotTypeMap.put("b-l-o-tbl-a-u", "Alarm/Other/Trouble/Audible/Pump Failure");
        cotTypeMap.put("b-l-o-tbl-s", "Alarm/Other/Trouble/Silent");
        cotTypeMap.put("b-l-o-tbl-s-a", "Alarm/Other/Trouble/Silent/Sensor");
        cotTypeMap.put("b-l-o-tbl-s-ar", "Alarm/Other/Trouble/Silent/Alarm Relay");
        cotTypeMap.put("b-l-o-tbl-s-b", "Alarm/Other/Trouble/Silent/Low System Battery");
        cotTypeMap.put("b-l-o-tbl-s-b1", "Alarm/Other/Trouble/Silent/Bell 1");
        cotTypeMap.put("b-l-o-tbl-s-b2", "Alarm/Other/Trouble/Silent/Bell 2");
        cotTypeMap.put("b-l-o-tbl-s-bt", "Alarm/Other/Trouble/Silent/Battery Test Fail");
        cotTypeMap.put("b-l-o-tbl-s-c", "Alarm/Other/Trouble/Silent/Polling Loop Close");
        cotTypeMap.put("b-l-o-tbl-s-cf", "Alarm/Other/Trouble/Silent/Communication Fail");
        cotTypeMap.put("b-l-o-tbl-s-dc", "Alarm/Other/Trouble/Silent/Exp Mod DC Loss");
        cotTypeMap.put("b-l-o-tbl-s-dh", "Alarm/Other/Trouble/Silent/Detector Hi Sens");
        cotTypeMap.put("b-l-o-tbl-s-dl", "Alarm/Other/Trouble/Silent/Detector Low Sens");
        cotTypeMap.put("b-l-o-tbl-s-e", "Alarm/Other/Trouble/Silent/Expansion Mod Fail");
        cotTypeMap.put("b-l-o-tbl-s-e50", "Alarm/Other/Trouble/Silent/Event Log 50% Full");
        cotTypeMap.put("b-l-o-tbl-s-e90", "Alarm/Other/Trouble/Silent/Event Log 90% Full");
        cotTypeMap.put("b-l-o-tbl-s-em", "Alarm/Other/Trouble/Silent/Expansion Module Fail");
        cotTypeMap.put("b-l-o-tbl-s-eo", "Alarm/Other/Trouble/Silent/Event Log Overflow");
        cotTypeMap.put("b-l-o-tbl-s-ft", "Alarm/Other/Trouble/Silent/Fire Trouble");
        cotTypeMap.put("b-l-o-tbl-s-gf", "Alarm/Other/Trouble/Silent/Ground Fault");
        cotTypeMap.put("b-l-o-tbl-s-lb", "Alarm/Other/Trouble/Silent/Exp Mod Low Battery");
        cotTypeMap.put("b-l-o-tbl-s-lo", "Alarm/Other/Trouble/Silent/Protection Loop Open");
        cotTypeMap.put("b-l-o-tbl-s-lp", "Alarm/Other/Trouble/Silent/Lost Polling");
        cotTypeMap.put("b-l-o-tbl-s-lr", "Alarm/Other/Trouble/Silent/Long Range Radio");
        cotTypeMap.put("b-l-o-tbl-s-ls", "Alarm/Other/Trouble/Silent/Protection Loop Short");
        cotTypeMap.put("b-l-o-tbl-s-ms", "Alarm/Other/Trouble/Silent/RPM Supervision");
        cotTypeMap.put("b-l-o-tbl-s-n", "Alarm/Other/Trouble/Silent/Near Alarm");
        cotTypeMap.put("b-l-o-tbl-s-o", "Alarm/Other/Trouble/Silent/Polling Loop Open");
        cotTypeMap.put("b-l-o-tbl-s-p", "Alarm/Other/Trouble/Silent/AC Power Lost");
        cotTypeMap.put("b-l-o-tbl-s-pf", "Alarm/Other/Trouble/Silent/Printer Failure");
        cotTypeMap.put("b-l-o-tbl-s-pl", "Alarm/Other/Trouble/Silent/Protection Loop");
        cotTypeMap.put("b-l-o-tbl-s-pp", "Alarm/Other/Trouble/Silent/Printer Paper Out");
        cotTypeMap.put("b-l-o-tbl-s-ps", "Alarm/Other/Trouble/Silent/Polling Loop Short");
        cotTypeMap.put("b-l-o-tbl-s-ra", "Alarm/Other/Trouble/Silent/Radio Antenna Vswr");
        cotTypeMap.put("b-l-o-tbl-s-rf", "Alarm/Other/Trouble/Silent/Repeater Failure");
        cotTypeMap.put("b-l-o-tbl-s-rfb", "Alarm/Other/Trouble/Silent/RF Low Battery");
        cotTypeMap.put("b-l-o-tbl-s-rr", "Alarm/Other/Trouble/Silent/Reversing Relay");
        cotTypeMap.put("b-l-o-tbl-s-rs", "Alarm/Other/Trouble/Silent/RF Supervision");
        cotTypeMap.put("b-l-o-tbl-s-sh", "Alarm/Other/Trouble/Silent/Smoke Det Hi Sens");
        cotTypeMap.put("b-l-o-tbl-s-sl", "Alarm/Other/Trouble/Silent/Smoke Det Low Sens");
        cotTypeMap.put("b-l-o-tbl-s-sp", "Alarm/Other/Trouble/Silent/System Peripheral");
        cotTypeMap.put("b-l-o-tbl-s-sr", "Alarm/Other/Trouble/Silent/Sounder/Relay");
        cotTypeMap.put("b-l-o-tbl-s-ss", "Alarm/Other/Trouble/Silent/System Shutdown");
        cotTypeMap.put("b-l-o-tbl-s-st", "Alarm/Other/Trouble/Silent/Self Test Failure");
        cotTypeMap.put("b-l-o-tbl-s-t1", "Alarm/Other/Trouble/Silent/Telco Line 1");
        cotTypeMap.put("b-l-o-tbl-s-t2", "Alarm/Other/Trouble/Silent/Telco Line 2");
        cotTypeMap.put("b-l-o-tbl-s-ti", "Alarm/Other/Trouble/Silent/Time/Date Inaccurate");
        cotTypeMap.put("b-l-o-tbl-s-tr", "Alarm/Other/Trouble/Silent/Trouble Relay");
        cotTypeMap.put("b-l-o-tbl-s-x", "Alarm/Other/Trouble/Silent/RAM Checksum Bad");
        cotTypeMap.put("b-l-o-tbl-s-y", "Alarm/Other/Trouble/Silent/ROM Checksum Bad");
        cotTypeMap.put("b-l-o-tem", "Alarm/Other/Temperature");
        cotTypeMap.put("b-l-o-tem-a", "Alarm/Other/Temperature/Audible");
        cotTypeMap.put("b-l-o-tem-a-h", "Alarm/Other/Temperature/Audible/High Temperature");
        cotTypeMap.put("b-l-o-tem-a-l", "Alarm/Other/Temperature/Audible/Low Temperature");
        cotTypeMap.put("b-l-o-tem-a-lh", "Alarm/Other/Temperature/Audible/Loss of Heat");
        cotTypeMap.put("b-l-o-tem-a-r", "Alarm/Other/Temperature/Audible/Refrigeration");
        cotTypeMap.put("b-l-o-tst", "Alarm/Other/Test");
        cotTypeMap.put("b-l-o-tst-s", "Alarm/Other/Test/SIlent");
        cotTypeMap.put("b-l-o-tst-s-fa", "Alarm/Other/Test/SIlent/Fire Alarm");
        cotTypeMap.put("b-l-o-tst-s-m", "Alarm/Other/Test/SIlent/Manual");
        cotTypeMap.put("b-l-o-tst-s-pt", "Alarm/Other/Test/SIlent/Point Tested OK");
        cotTypeMap.put("b-l-o-tst-s-st", "Alarm/Other/Test/SIlent/System Remains in Trouble");
        cotTypeMap.put("b-l-o-tst-s-wm", "Alarm/Other/Test/SIlent/Walk Test Mode Entered");
        cotTypeMap.put("b-l-r", "Alarm/Rescue");
        cotTypeMap.put("b-l-s", "Alarm/Safety");
        cotTypeMap.put("b-l-t", "Alarm/Transportation");
        cotTypeMap.put("b-l-t-v", "Alarm/Transportation/Vehicle");
        cotTypeMap.put("b-l-t-v-a", "Alarm/Transportation/Vehicle/Accident");
        cotTypeMap.put("b-l-t-v-a-c", "Alarm/Transportation/Vehicle/Accident/Collision");
        cotTypeMap.put("b-l-t-v-a-c-f", "Alarm/Transportation/Vehicle/Accident/Collision/Fixed Object");
        cotTypeMap.put("b-l-t-v-a-c-n", "Alarm/Transportation/Vehicle/Accident/Collision/Non-Fixed Object");
        cotTypeMap.put("b-l-t-v-a-e", "Alarm/Transportation/Vehicle/Accident/Equipment/Cargo Loss/Shift");
        cotTypeMap.put("b-l-t-v-a-f", "Alarm/Transportation/Vehicle/Accident/Fire_Explosion");
        cotTypeMap.put("b-l-t-v-a-i", "Alarm/Transportation/Vehicle/Accident/Immersion");
        cotTypeMap.put("b-l-t-v-a-j", "Alarm/Transportation/Vehicle/Accident/Jacknife");
        cotTypeMap.put("b-l-t-v-a-o", "Alarm/Transportation/Vehicle/Accident/Object Thrown/Fallen");
        cotTypeMap.put("b-l-t-v-a-or", "Alarm/Transportation/Vehicle/Accident/Overturn_Rollover");
        cotTypeMap.put("b-l-t-v-a-p", "Alarm/Transportation/Vehicle/Accident/Person Fell or Jumped");
        cotTypeMap.put("b-l-t-v-o", "Alarm/Transportation/Vehicle/Other (Non Collision)");
        cotTypeMap.put("c", "Capability");
        cotTypeMap.put("c-c", "CAP: Communications");
        cotTypeMap.put("c-f", "CAP: Fires");
        cotTypeMap.put("c-f-d", "CAP: Direct fires");
        cotTypeMap.put("c-f-i", "CAP: Indirect fires");
        cotTypeMap.put("c-l", "CAP: Logistics (supply)");
        cotTypeMap.put("c-l-f", "CAP: Fuel");
        cotTypeMap.put("c-r", "CAP: Rescue");
        cotTypeMap.put("c-s", "CAP: Surveillance");
        cotTypeMap.put("t", "Tasking");
        cotTypeMap.put("t-a", "air");
        cotTypeMap.put("t-a-c", "close air support");
        cotTypeMap.put("t-a-d", "air drop");
        cotTypeMap.put("t-a-e", "electronic warfare");
        cotTypeMap.put("t-a-k", "strike");
        cotTypeMap.put("t-a-r", "recovery");
        cotTypeMap.put("t-a-s", "SEAD");
        cotTypeMap.put("t-k", "Strike");
        cotTypeMap.put("t-k-d", "Destroy");
        cotTypeMap.put("t-k-i", "Investigate");
        cotTypeMap.put("t-k-t", "Target");
        cotTypeMap.put("t-m", "Mensurate");
        cotTypeMap.put("t-p", "WTP: pairing");
        cotTypeMap.put("t-p-a", "WTP: air");
        cotTypeMap.put("t-p-a-c", "WTP: CAS");
        cotTypeMap.put("t-p-a-f", "WTP: refueling");
        cotTypeMap.put("t-p-a-k", "WTP: strike");
        cotTypeMap.put("t-p-k", "WTP: Strike");
        cotTypeMap.put("t-p-k-d", "WTP: Destroy");
        cotTypeMap.put("t-p-k-i", "WTP: Investigate");
        cotTypeMap.put("t-p-k-t", "WTP: Target");
        cotTypeMap.put("t-p-r", "WTP: Recovery (personnel)");
        cotTypeMap.put("t-p-s", "WTP: ISR");
        cotTypeMap.put("t-p-s-i", "WTP: imagery");
        cotTypeMap.put("t-q", "Tasking Query Capable");
        cotTypeMap.put("t-r", "Tasking Relocate");
        cotTypeMap.put("t-s", "ISR");
        cotTypeMap.put("t-s-b", "Tasking BFT info");
        cotTypeMap.put("t-s-i", "imagery desired");
        cotTypeMap.put("t-s-i-e", "ISR EO");
        cotTypeMap.put("t-s-i-i", "ISR IR");
        cotTypeMap.put("t-s-r", "ISR Radar");
        cotTypeMap.put("t-s-v", "ISR Video");
        cotTypeMap.put("t-s-v-e", "ISR Video EO");
        cotTypeMap.put("t-s-v-i", "ISR Video IR");
        cotTypeMap.put("t-u", "Tasking Update");
        cotTypeMap.put("t-u-q", "Tasking Status Query");
        cotTypeMap.put("t-u-z", "Tasking Cancel");
        cotTypeMap.put("t-x", "Tasking (Experimental)");
        cotTypeMap.put("t-x-i", "Tasking Data Retrieval");
        cotTypeMap.put("t-x-i-l", "Link Dereference");
        cotTypeMap.put("t-x-a-s", "App control- Sync (with peer)");
        cotTypeMap.put("t-x-a-s-c", "App control- Sync (subscribe)");
        cotTypeMap.put("t-x-a-f", "Filter");
        cotTypeMap.put("t-x-a-o", "Open");
        cotTypeMap.put("t-x-v-m", "Medevac");
        cotTypeMap.put("y", "Reply");
        cotTypeMap.put("y-a", "Ack");
        cotTypeMap.put("y-a-r", "Rcvd");
        cotTypeMap.put("y-a-w", "Wilco");
        cotTypeMap.put("y-c", "Tasking Complete");
        cotTypeMap.put("y-c-f", "Fail");
        cotTypeMap.put("y-c-f-a", "Fail: No Assets");
        cotTypeMap.put("y-c-f-b", "Fail: Bad Reqest (CANTPRO)");
        cotTypeMap.put("y-c-f-d", "Fail: Denied");
        cotTypeMap.put("y-c-f-i", "Fail: Insufficient Info");
        cotTypeMap.put("y-c-f-r", "Fail: Rejected");
        cotTypeMap.put("y-c-f-r-c", "Fail: C2 element");
        cotTypeMap.put("y-c-f-r-p", "Fail: Platform (CANTCO)");
        cotTypeMap.put("y-c-f-s", "Fail: Stale");
        cotTypeMap.put("y-c-f-x", "Fail: Cancelled");
        cotTypeMap.put("y-c-s", "Complete: Success");
        cotTypeMap.put("y-s", "Status: Status");
        cotTypeMap.put("y-s-c", "Status: Canceling");
        cotTypeMap.put("y-s-e", "Status: Executing");
        cotTypeMap.put("y-s-e-a", "Status: Approved");
        cotTypeMap.put("y-s-e-d", "Status: Disseminating");
        cotTypeMap.put("y-s-e-d-c", "Status: Disseminated");
        cotTypeMap.put("y-s-i", "Status: Imagery Available");
        cotTypeMap.put("y-s-m", "Status: MISREP available");
        cotTypeMap.put("y-s-p", "Status: Planning");
        cotTypeMap.put("y-s-r", "Status: Reviewing");
        cotTypeMap.put("b-g-.", "TACTICAL GRAPHICS");
        cotTypeMap.put("b-g-.-T", "TASKS");
        cotTypeMap.put("b-g-.-T-B", "BLOCK");
        cotTypeMap.put("b-g-.-T-H", "BREACH");
        cotTypeMap.put("b-g-.-T-Y", "BYPASS");
        cotTypeMap.put("b-g-.-T-C", "CANALIZE");
        cotTypeMap.put("b-g-.-T-X", "CLEAR");
        cotTypeMap.put("b-g-.-T-J", "CONTAIN");
        cotTypeMap.put("b-g-.-T-K", "COUNTERATTACK (CATK)");
        cotTypeMap.put("b-g-.-T-K-F", "COUNTERATTACK BY FIRE");
        cotTypeMap.put("b-g-.-T-L", "DELAY");
        cotTypeMap.put("b-g-.-T-D", "DESTROY");
        cotTypeMap.put("b-g-.-T-T", "DISRUPT");
        cotTypeMap.put("b-g-.-T-F", "FIX");
        cotTypeMap.put("b-g-.-T-A", "FOLLOW AND ASSUME");
        cotTypeMap.put("b-g-.-T-A-S", "FOLLOW AND SUPPORT");
        cotTypeMap.put("b-g-.-T-I", "INTERDICT");
        cotTypeMap.put("b-g-.-T-E", "ISOLATE");
        cotTypeMap.put("b-g-.-T-N", "NEUTRALIZE");
        cotTypeMap.put("b-g-.-T-O", "OCCUPY");
        cotTypeMap.put("b-g-.-T-P", "PENETRATE");
        cotTypeMap.put("b-g-.-T-R", "RELIEF IN PLACE (RIP)");
        cotTypeMap.put("b-g-.-T-Q", "RETAIN");
        cotTypeMap.put("b-g-.-T-M", "RETIREMENT");
        cotTypeMap.put("b-g-.-T-S", "SECURE");
        cotTypeMap.put("b-g-.-T-U", "SECURITY");
        cotTypeMap.put("b-g-.-T-U-S", "SCREEN");
        cotTypeMap.put("b-g-.-T-U-G", "GUARD");
        cotTypeMap.put("b-g-.-T-U-C", "COVER");
        cotTypeMap.put("b-g-.-T-Z", "SEIZE");
        cotTypeMap.put("b-g-.-T-W", "WITHDRAW");
        cotTypeMap.put("b-g-.-T-W-P", "WITHDRAW UNDER PRESSURE");
        cotTypeMap.put("b-g-.-G", "COMMAND AND CONTROL AND GENERAL MANEUVER");
        cotTypeMap.put("b-g-.-G-G", "GENERAL");
        cotTypeMap.put("b-g-.-G-G-P", "POINTS");
        cotTypeMap.put("b-g-.-G-G-P-U", "UNDER SEA WARFARE");
        cotTypeMap.put("b-g-.-G-G-P-U-U", "UNDERWATER");
        cotTypeMap.put("b-g-.-G-G-P-U-U-D", "DATUM");
        cotTypeMap.put("b-g-.-G-G-P-U-U-B", "BRIEF CONTACT");
        cotTypeMap.put("b-g-.-G-G-P-U-U-L", "LOST CONTACT");
        cotTypeMap.put("b-g-.-G-G-P-U-U-S", "SINKER");
        cotTypeMap.put("b-g-.-G-G-P-U-Y", "SONOBUOY");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-P", "PATTERN CENTER");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-D", "DIRECTIONAL FREQUENCY ANALYZING AND");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-L", "LOW FREQUENCY ANALYZING AND RECORDING");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-C", "COMMAND ACTIVE SONOBUOY SYSTEM (CASS)");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-S", "DIRECTIONAL COMMAND ACTIVE SONOBUOY SYSTEM");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-B", "BATHYTHERMOGRAPH TRANSMITTING (BT)");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-A", "ANM");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-V", "VERTICAL LINE ARRAY DIFAR (VLAD)");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-T", "ATAC");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-R", "RANGE ONLY (RO)");
        cotTypeMap.put("b-g-.-G-G-P-U-Y-K", "KINGPIN");
        cotTypeMap.put("b-g-.-G-G-P-U-S", "SEARCH");
        cotTypeMap.put("b-g-.-G-G-P-U-S-A", "SEARCH AREA");
        cotTypeMap.put("b-g-.-G-G-P-U-S-D", "DIP POSITION");
        cotTypeMap.put("b-g-.-G-G-P-U-S-C", "SEARCH CENTER");
        cotTypeMap.put("b-g-.-G-G-P-R", "REFERENCE POINT");
        cotTypeMap.put("b-g-.-G-G-P-R-S", "SPECIAL POINT");
        cotTypeMap.put("b-g-.-G-G-P-R-N", "NAV REFERENCE");
        cotTypeMap.put("b-g-.-G-G-P-R-D", "DLRP");
        cotTypeMap.put("b-g-.-G-G-P-R-I", "POINT OF INTEREST");
        cotTypeMap.put("b-g-.-G-G-P-W", "WEAPON");
        cotTypeMap.put("b-g-.-G-G-P-W-A", "AIM POINT");
        cotTypeMap.put("b-g-.-G-G-P-W-D", "DROP POINT");
        cotTypeMap.put("b-g-.-G-G-P-W-E", "ENTRY POINT");
        cotTypeMap.put("b-g-.-G-G-P-W-G", "GROUND ZERO");
        cotTypeMap.put("b-g-.-G-G-P-W-M", "MSL DETECT POINT");
        cotTypeMap.put("b-g-.-G-G-P-W-I", "IMPACT POINT");
        cotTypeMap.put("b-g-.-G-G-P-W-P", "PREDICTED IMPACT POINT");
        cotTypeMap.put("b-g-.-G-G-P-F", "FORMATION");
        cotTypeMap.put("b-g-.-G-G-P-H", "HARBOR (GENERAL)");
        cotTypeMap.put("b-g-.-G-G-P-H-Q", "POINT Q");
        cotTypeMap.put("b-g-.-G-G-P-H-A", "POINT A");
        cotTypeMap.put("b-g-.-G-G-P-H-Y", "POINT Y");
        cotTypeMap.put("b-g-.-G-G-P-H-X", "POINT X");
        cotTypeMap.put("b-g-.-G-G-P-O", "ROUTE");
        cotTypeMap.put("b-g-.-G-G-P-O-Z", "RENDEZVOUS");
        cotTypeMap.put("b-g-.-G-G-P-O-D", "DIVERSIONS");
        cotTypeMap.put("b-g-.-G-G-P-O-W", "WAYPOINT");
        cotTypeMap.put("b-g-.-G-G-P-O-P", "PIM");
        cotTypeMap.put("b-g-.-G-G-P-O-R", "POINT R");
        cotTypeMap.put("b-g-.-G-G-P-A", "AIR CONTROL");
        cotTypeMap.put("b-g-.-G-G-P-A-P", "COMBAT AIR PATROL (CAP)");
        cotTypeMap.put("b-g-.-G-G-P-A-W", "AIRBORNE EARLY WARNING (AEW)");
        cotTypeMap.put("b-g-.-G-G-P-A-T", "TACAN");
        cotTypeMap.put("b-g-.-G-G-P-A-K", "TANKING");
        cotTypeMap.put("b-g-.-G-G-P-A-A", "ANTISUBMARINE WARFARE , FIXED WING");
        cotTypeMap.put("b-g-.-G-G-P-A-H", "ANTISUBMARINE WARFARE, ROTARY WING");
        cotTypeMap.put("b-g-.-G-G-P-A-O", "TOMCAT");
        cotTypeMap.put("b-g-.-G-G-P-A-R", "RESCUE");
        cotTypeMap.put("b-g-.-G-G-P-A-L", "REPLENISH");
        cotTypeMap.put("b-g-.-G-G-P-A-M", "MARSHALL");
        cotTypeMap.put("b-g-.-G-G-P-A-S", "STRIKE IP");
        cotTypeMap.put("b-g-.-G-G-P-A-C", "CORRIDOR TAB");
        cotTypeMap.put("b-g-.-G-G-P-P", "ACTION POINTS (GENERAL)");
        cotTypeMap.put("b-g-.-G-G-P-P-K", "CHECK POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-C", "CONTACT POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-O", "COORDINATION POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-D", "DECISION POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-L", "LINKUP POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-P", "PASSAGE POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-R", "RALLY POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-E", "RELEASE POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-S", "START POINT");
        cotTypeMap.put("b-g-.-G-G-P-P-W", "WAYPOINT");
        cotTypeMap.put("b-g-.-G-G-L", "LINES");
        cotTypeMap.put("b-g-.-G-G-L-B", "BOUNDARIES");
        cotTypeMap.put("b-g-.-G-G-L-F", "FORWARD LINE OF OWN TROOPS (FLOT)");
        cotTypeMap.put("b-g-.-G-G-L-C", "LINE OF CONTACT");
        cotTypeMap.put("b-g-.-G-G-L-P", "PHASE LINE");
        cotTypeMap.put("b-g-.-G-G-L-L", "LIGHT LINE");
        cotTypeMap.put("b-g-.-G-G-A", "AREAS");
        cotTypeMap.put("b-g-.-G-G-A-G", "GENERAL AREA");
        cotTypeMap.put("b-g-.-G-G-A-A", "ASSEMBLY AREA");
        cotTypeMap.put("b-g-.-G-G-A-E", "ENGAGEMENT AREA");
        cotTypeMap.put("b-g-.-G-G-A-F", "FORTIFIED AREA");
        cotTypeMap.put("b-g-.-G-G-A-D", "DROP ZONE");
        cotTypeMap.put("b-g-.-G-G-A-X", "EXTRACTION ZONE (EZ)");
        cotTypeMap.put("b-g-.-G-G-A-L", "LANDING ZONE (LZ)");
        cotTypeMap.put("b-g-.-G-G-A-P", "PICKUP ZONE (PZ)");
        cotTypeMap.put("b-g-.-G-G-A-S", "SEARCH AREA/RECONNAISSANCE AREA");
        cotTypeMap.put("b-g-.-G-G-A-Y", "LIMITED ACCESS AREA");
        cotTypeMap.put("b-g-.-G-G-A-Z", "AIRFIELD ZONE");
        cotTypeMap.put("b-g-.-G-A", "AVIATION");
        cotTypeMap.put("b-g-.-G-A-P", "POINTS");
        cotTypeMap.put("b-g-.-G-A-P-P", "AIR CONTROL POINT (ACP)");
        cotTypeMap.put("b-g-.-G-A-P-C", "COMMUNICATIONS CHECKPOINT (CCP)");
        cotTypeMap.put("b-g-.-G-A-P-U", "POP UP POINT (PUP)");
        cotTypeMap.put("b-g-.-G-A-P-D", "DOWNED AIRCREW PICKUP POINT");
        cotTypeMap.put("b-g-.-G-A-L", "LINES");
        cotTypeMap.put("b-g-.-G-A-L-C", "AIR CORRIDOR");
        cotTypeMap.put("b-g-.-G-A-L-M", "MINIMUM RISK ROUTE (MRR)");
        cotTypeMap.put("b-g-.-G-A-L-S", "STANDARD USE ARMY AIRCRAFT FLIGHT ROUTE");
        cotTypeMap.put("b-g-.-G-A-L-U", "UNMANNED AERIAL VEHICLE (UAV) ROUTE");
        cotTypeMap.put("b-g-.-G-A-L-L", "LOW LEVEL TRANSIT ROUTE (LLTR)");
        cotTypeMap.put("b-g-.-G-A-A", "AREAS");
        cotTypeMap.put("b-g-.-G-A-A-R", "RESTRICTED OPERATIONS ZONE (ROZ)");
        cotTypeMap.put("b-g-.-G-A-A-F", "FORWARD AREA AIR DEFENSE ZONE (FAADEZ)");
        cotTypeMap.put("b-g-.-G-A-A-H", "HIGH DENSITY AIRSPACE CONTROL ZONE (HIDACZ)");
        cotTypeMap.put("b-g-.-G-A-A-M", "MISSILE ENGAGEMENT ZONE (MEZ)");
        cotTypeMap.put("b-g-.-G-A-A-M-L", "LOW ALTITUDE MEZ");
        cotTypeMap.put("b-g-.-G-A-A-M-H", "HIGH ALTITUDE MEZ");
        cotTypeMap.put("b-g-.-G-A-A-W", "WEAPONS FREE ZONE");
        cotTypeMap.put("b-g-.-G-P", "DECEPTION");
        cotTypeMap.put("b-g-.-G-P-D", "DUMMY (DECEPTION/DECOY)");
        cotTypeMap.put("b-g-.-G-P-A", "AXIS OF ADVANCE FOR FEINT");
        cotTypeMap.put("b-g-.-G-P-F", "DIRECTION OF ATTACK FOR FEINT");
        cotTypeMap.put("b-g-.-G-P-M", "DECOY MINED AREA");
        cotTypeMap.put("b-g-.-G-P-Y", "DECOY MINED AREA, FENCED");
        cotTypeMap.put("b-g-.-G-P-N", "DUMMY MINEFIELD (STATIC)");
        cotTypeMap.put("b-g-.-G-P-C", "DUMMY MINEFIELD (DYNAMIC)");
        cotTypeMap.put("b-g-.-G-D", "DEFENSE");
        cotTypeMap.put("b-g-.-G-D-P", "POINTS");
        cotTypeMap.put("b-g-.-G-D-P-T", "TARGET REFERENCE POINT (TRP)");
        cotTypeMap.put("b-g-.-G-D-P-O", "OBSERVATION POST/OUTPOST");
        cotTypeMap.put("b-g-.-G-D-P-O-C", "COMBAT OUTPOST");
        cotTypeMap.put("b-g-.-G-D-P-O-R", "OBSERVATION POST OCCUPIED BY DISMOUNTED");
        cotTypeMap.put("b-g-.-G-D-P-O-F", "FORWARD OBSERVER POSITION");
        cotTypeMap.put("b-g-.-G-D-P-O-S", "SENSOR OUTPOST/LISTENING POST (OP/LP)");
        cotTypeMap.put("b-g-.-G-D-P-O-N", "NBC OBSERVATION POST (DISMOUNTED)");
        cotTypeMap.put("b-g-.-G-D-L", "LINES");
        cotTypeMap.put("b-g-.-G-D-L-F", "FORWARD EDGE OF BATTLE AREA (FEBA)");
        cotTypeMap.put("b-g-.-G-D-L-P", "PRINCIPAL DIRECTION OF FIRE (PDF)");
        cotTypeMap.put("b-g-.-G-D-A", "AREAS");
        cotTypeMap.put("b-g-.-G-D-A-B", "BATTLE POSITION");
        cotTypeMap.put("b-g-.-G-D-A-B-P", "");
        cotTypeMap.put("b-g-.-G-D-A-E", "ENGAGEMENT AREA");
        cotTypeMap.put("b-g-.-G-O", "OFFENSE");
        cotTypeMap.put("b-g-.-G-O-P", "POINTS");
        cotTypeMap.put("b-g-.-G-O-P-P", "POINT OF");
        cotTypeMap.put("b-g-.-G-O-L", "LINES");
        cotTypeMap.put("b-g-.-G-O-L-A", "AXIS OF ADVANCE");
        cotTypeMap.put("b-g-.-G-O-L-A-V", "FRIENDLY AVIATION");
        cotTypeMap.put("b-g-.-G-O-L-A-A", "FRIENDLY AIRBORNE");
        cotTypeMap.put("b-g-.-G-O-L-A-R", "FRIENDLY ATTACK, ROTARY WING");
        cotTypeMap.put("b-g-.-G-O-L-A-G", "GROUND");
        cotTypeMap.put("b-g-.-G-O-L-A-G-M", "MAIN ATTACK");
        cotTypeMap.put("b-g-.-G-O-L-A-G-S", "SUPPORTING ATTACK");
        cotTypeMap.put("b-g-.-G-O-L-K", "DIRECTION OF ATTACK");
        cotTypeMap.put("b-g-.-G-O-L-K-A", "AVIATION");
        cotTypeMap.put("b-g-.-G-O-L-K-G", "GROUND");
        cotTypeMap.put("b-g-.-G-O-L-K-G-M", "MAIN ATTACK");
        cotTypeMap.put("b-g-.-G-O-L-K-G-S", "SUPPORTING ATTACK");
        cotTypeMap.put("b-g-.-G-O-L-F", "FINAL COORDINATION LINE");
        cotTypeMap.put("b-g-.-G-O-L-I", "INFILTRATION LINE");
        cotTypeMap.put("b-g-.-G-O-L-L", "LIMIT OF ADVANCE");
        cotTypeMap.put("b-g-.-G-O-L-T", "LINE OF");
        cotTypeMap.put("b-g-.-G-O-L-C", "LINE OF");
        cotTypeMap.put("b-g-.-G-O-L-P", "PROBABLE LINE OF DEPLOYMENT (PLD)");
        cotTypeMap.put("b-g-.-G-O-A", "AREAS");
        cotTypeMap.put("b-g-.-G-O-A-A", "ASSAULT POSITION");
        cotTypeMap.put("b-g-.-G-O-A-K", "ATTACK POSITION");
        cotTypeMap.put("b-g-.-G-O-A-F", "ATTACK BY FIRE POSITION");
        cotTypeMap.put("b-g-.-G-O-A-S", "SUPPORT BY FIRE POSITION");
        cotTypeMap.put("b-g-.-G-O-A-O", "OBJECTIVE");
        cotTypeMap.put("b-g-.-G-O-A-P", "PENETRATION BOX");
        cotTypeMap.put("b-g-.-G-S", "SPECIAL");
        cotTypeMap.put("b-g-.-G-S-L", "LINE");
        cotTypeMap.put("b-g-.-G-S-L-A", "AMBUSH");
        cotTypeMap.put("b-g-.-G-S-L-H", "HOLDING LINE");
        cotTypeMap.put("b-g-.-G-S-L-R", "RELEASE LINE");
        cotTypeMap.put("b-g-.-G-S-A", "AREA");
        cotTypeMap.put("b-g-.-G-S-A-O", "AREA OF OPERATIONS (AO)");
        cotTypeMap.put("b-g-.-G-S-A-A", "AIRHEAD");
        cotTypeMap.put("b-g-.-G-S-A-B", "BRIDGEHEAD");
        cotTypeMap.put("b-g-.-G-S-A-E", "ENCIRCLEMENT");
        cotTypeMap.put("b-g-.-G-S-A-N", "NAMED AREA OF INTEREST (NAI)");
        cotTypeMap.put("b-g-.-G-S-A-T", "TARGETED AREA OF INTEREST (TAI)");
        cotTypeMap.put("b-g-.-M", "MOBILITY/SURVIVABILITY");
        cotTypeMap.put("b-g-.-M-O", "OBSTACLES");
        cotTypeMap.put("b-g-.-M-O-G", "GENERAL");
        cotTypeMap.put("b-g-.-M-O-G-B", "BELT");
        cotTypeMap.put("b-g-.-M-O-G-L", "LINE");
        cotTypeMap.put("b-g-.-M-O-G-Z", "ZONE");
        cotTypeMap.put("b-g-.-M-O-G-F", "OBSTACLE FREE AREA");
        cotTypeMap.put("b-g-.-M-O-G-R", "OBSTACLE RESTRICTED AREA");
        cotTypeMap.put("b-g-.-M-O-S", "ABATIS");
        cotTypeMap.put("b-g-.-M-O-A", "ANTITANK OBSTACLES");
        cotTypeMap.put("b-g-.-M-O-A-D", "ANTITANK DITCH");
        cotTypeMap.put("b-g-.-M-O-A-D-U", "UNDER CONSTRUCTION");
        cotTypeMap.put("b-g-.-M-O-A-D-C", "COMPLETE");
        cotTypeMap.put("b-g-.-M-O-A-R", "ANTITANK DITCH REINFORCED WITH ANTITANK MINES");
        cotTypeMap.put("b-g-.-M-O-A-O", "ANTITANK OBSTACLES: TETRAHEDRONS, DRAGONS");
        cotTypeMap.put("b-g-.-M-O-A-O-F", "FIXED AND PREFABRICATED");
        cotTypeMap.put("b-g-.-M-O-A-O-M", "MOVEABLE");
        cotTypeMap.put("b-g-.-M-O-A-O-P", "MOVEABLE AND PREFABRICATED");
        cotTypeMap.put("b-g-.-M-O-A-W", "ANTITANK WALL");
        cotTypeMap.put("b-g-.-M-O-B", "BOOBY TRAP");
        cotTypeMap.put("b-g-.-M-O-M", "MINES");
        cotTypeMap.put("b-g-.-M-O-M-U", "UNSPECIFIED MINE");
        cotTypeMap.put("b-g-.-M-O-M-T", "ANTITANK MINE (AT)");
        cotTypeMap.put("b-g-.-M-O-M-D", "ANTITANK MINE WITH ANTIHANDLING DEVICE");
        cotTypeMap.put("b-g-.-M-O-M-E", "ANTITANK MINE (DIRECTIONAL)");
        cotTypeMap.put("b-g-.-M-O-M-P", "ANTIPERSONNEL (AP) MINES");
        cotTypeMap.put("b-g-.-M-O-M-W", "WIDE AREA MINES");
        cotTypeMap.put("b-g-.-M-O-M-C", "MINE CLUSTER");
        cotTypeMap.put("b-g-.-M-O-F", "MINEFIELDS");
        cotTypeMap.put("b-g-.-M-O-F-S", "STATIC DEPICTION");
        cotTypeMap.put("b-g-.-M-O-F-D", "DYNAMIC DEPICTION");
        cotTypeMap.put("b-g-.-M-O-F-G", "GAP");
        cotTypeMap.put("b-g-.-M-O-F-A", "MINED AREA");
        cotTypeMap.put("b-g-.-M-O-E", "OBSTACLE EFFECT");
        cotTypeMap.put("b-g-.-M-O-E-B", "BLOCK");
        cotTypeMap.put("b-g-.-M-O-E-F", "FIX");
        cotTypeMap.put("b-g-.-M-O-E-T", "TURN");
        cotTypeMap.put("b-g-.-M-O-E-D", "DISRUPT");
        cotTypeMap.put("b-g-.-M-O-U", "UNEXPLODED ORDINANCE AREA (UXO)");
        cotTypeMap.put("b-g-.-M-O-R", "ROADBLOCKS, CRATERS, AND BLOWN BRIDGES");
        cotTypeMap.put("b-g-.-M-O-R-P", "PLANNED");
        cotTypeMap.put("b-g-.-M-O-R-S", "EXPLOSIVES, STATE OF READINESS 1 (SAFE)");
        cotTypeMap.put("b-g-.-M-O-R-A", "EXPLOSIVES, STATE OF READINESS 2 (ARMED BUT");
        cotTypeMap.put("b-g-.-M-O-R-C", "ROADBLOCK COMPLETE (EXECUTED)");
        cotTypeMap.put("b-g-.-M-O-T", "TRIP WIRE");
        cotTypeMap.put("b-g-.-M-O-W", "WIRE OBSTACLE");
        cotTypeMap.put("b-g-.-M-O-W-U", "UNSPECIFIED");
        cotTypeMap.put("b-g-.-M-O-W-S", "SINGLE FENCE");
        cotTypeMap.put("b-g-.-M-O-W-D", "DOUBLE FENCE");
        cotTypeMap.put("b-g-.-M-O-W-A", "DOUBLE APRON FENCE");
        cotTypeMap.put("b-g-.-M-O-W-L", "LOW WIRE FENCE");
        cotTypeMap.put("b-g-.-M-O-W-H", "HIGH WIRE FENCE");
        cotTypeMap.put("b-g-.-M-O-W-C", "CONCERTINA");
        cotTypeMap.put("b-g-.-M-O-W-C-S", "SINGLE CONCERTINA");
        cotTypeMap.put("b-g-.-M-O-W-C-D", "DOUBLE STRAND CONCERTINA");
        cotTypeMap.put("b-g-.-M-O-W-C-T", "TRIPLE STRAND CONCERTINA");
        cotTypeMap.put("b-g-.-M-B", "OBSTACLE BYPASS");
        cotTypeMap.put("b-g-.-M-B-D", "OBSTACLE BYPASS DIFFICULTY");
        cotTypeMap.put("b-g-.-M-B-D-E", "BYPASS EASY");
        cotTypeMap.put("b-g-.-M-B-D-D", "BYPASS DIFFICULT");
        cotTypeMap.put("b-g-.-M-B-D-I", "BYPASS IMPOSSIBLE");
        cotTypeMap.put("b-g-.-M-B-C", "CROSSING SITE/WATER CROSSING");
        cotTypeMap.put("b-g-.-M-B-C-A", "ASSAULT CROSSING AREA");
        cotTypeMap.put("b-g-.-M-B-C-B", "BRIDGE OR GAP");
        cotTypeMap.put("b-g-.-M-B-C-F", "FERRY");
        cotTypeMap.put("b-g-.-M-B-C-E", "FORD EASY");
        cotTypeMap.put("b-g-.-M-B-C-D", "FORD DIFFICULT");
        cotTypeMap.put("b-g-.-M-B-C-L", "LANE");
        cotTypeMap.put("b-g-.-M-B-C-R", "RAFT SITE");
        cotTypeMap.put("b-g-.-M-B-C-P", "ENGINEER REGULATING POINT");
        cotTypeMap.put("b-g-.-M-S", "SURVIVABILITY");
        cotTypeMap.put("b-g-.-M-S-E", "EARTHWORK, SMALL TRENCH OR FORTIFICATION");
        cotTypeMap.put("b-g-.-M-S-F", "FORT");
        cotTypeMap.put("b-g-.-M-S-L", "FORTIFIED LINE");
        cotTypeMap.put("b-g-.-M-S-W", "FOXHOLE, EMPLACEMENT OR WEAPON SITE");
        cotTypeMap.put("b-g-.-M-S-P", "STRONG POINT");
        cotTypeMap.put("b-g-.-M-S-S", "SURFACE SHELTER");
        cotTypeMap.put("b-g-.-M-S-U", "UNDERGROUND SHELTER");
        cotTypeMap.put("b-g-.-M-N", "NUCLEAR, BIOLOGICAL AND CHEMICAL");
        cotTypeMap.put("b-g-.-M-N-M", "MINIMUM SAFE DISTANCE ZONES");
        cotTypeMap.put("b-g-.-M-N-Z", "NUCLEAR DETINATIONS GROUND ZERO");
        cotTypeMap.put("b-g-.-M-N-F", "FALLOUT PRODUCING");
        cotTypeMap.put("b-g-.-M-N-R", "RADIOACTIVE AREA");
        cotTypeMap.put("b-g-.-M-N-B", "BIOLOGICALLY CONTAMINATED AREA");
        cotTypeMap.put("b-g-.-M-N-C", "CHEMICALLY CONTAMINATED AREA");
        cotTypeMap.put("b-g-.-M-N-E", "RELEASE EVENTS");
        cotTypeMap.put("b-g-.-M-N-E-B", "BIOLOGICAL");
        cotTypeMap.put("b-g-.-M-N-E-C", "CHEMICAL");
        cotTypeMap.put("b-g-.-M-N-D", "DECONTAMINATION (DECON) POINTS");
        cotTypeMap.put("b-g-.-M-N-D-P", "DECON SITE/POINT (UNSPECIFIED)");
        cotTypeMap.put("b-g-.-M-N-D-A", "ALTERNATE DECON SITE/POINT (UNSPECIFIED)");
        cotTypeMap.put("b-g-.-M-N-D-T", "DECON SITE/POINT (TROOPS)");
        cotTypeMap.put("b-g-.-M-N-D-E", "DECON SITE/POINT (EQUIPMENT)");
        cotTypeMap.put("b-g-.-M-N-D-B", "DECON SITE/POINT (EQUIPMENT AND TROOPS)");
        cotTypeMap.put("b-g-.-M-N-D-O", "DECON SITE/POINT (OPERATIONAL");
        cotTypeMap.put("b-g-.-M-N-D-D", "DECON SITE/POINT (THOROUGH DECONTAMINATION)");
        cotTypeMap.put("b-g-.-M-N-L", "DOSE RATE CONTOUR LINES");
        cotTypeMap.put("b-g-.-F", "FIRE SUPPORT");
        cotTypeMap.put("b-g-.-F-P", "POINT");
        cotTypeMap.put("b-g-.-F-P-T", "TARGET");
        cotTypeMap.put("b-g-.-F-P-T-S", "POINT/SINGLE TARGET");
        cotTypeMap.put("b-g-.-F-P-T-N", "NUCLEAR TARGET");
        cotTypeMap.put("b-g-.-F-P-T-C", "CIRCULAR TARGET");
        cotTypeMap.put("b-g-.-F-P-T-R", "RECTANGULAR TARGET");
        cotTypeMap.put("b-g-.-F-P-S", "FIRE SUPPORT STATION");
        cotTypeMap.put("b-g-.-F-L", "LINES");
        cotTypeMap.put("b-g-.-F-L-F", "FIRE SUPPORT COORDINATION LINE (FSCL)");
        cotTypeMap.put("b-g-.-F-L-C", "COORDINATED FIRE LINE (CFL)");
        cotTypeMap.put("b-g-.-F-L-N", "NO FIRE LINE (NFL)");
        cotTypeMap.put("b-g-.-F-L-R", "RESTRICTIVE FIRE LINE (RFL)");
        cotTypeMap.put("b-g-.-F-L-L", "LINEAR TARGET");
        cotTypeMap.put("b-g-.-F-L-P", "FINAL PROTECTIVE FIRE (FPF)");
        cotTypeMap.put("b-g-.-F-L-S", "LINEAR SMOKE TARGET");
        cotTypeMap.put("b-g-.-F-A", "AREAS");
        cotTypeMap.put("b-g-.-F-A-A", "FIRE SUPPORT AREA (FSA)");
        cotTypeMap.put("b-g-.-F-A-C", "AIRSPACE COORDINATION AREA (ACA)");
        cotTypeMap.put("b-g-.-F-A-T", "AREA TARGET");
        cotTypeMap.put("b-g-.-F-A-K", "SMOKE");
        cotTypeMap.put("b-g-.-F-A-S", "SERIES OR GROUP OF TARGETS");
        cotTypeMap.put("b-g-.-F-A-B", "BOMB AREA");
        cotTypeMap.put("b-g-.-F-A-F", "FREE FIRE AREA (FFA)");
        cotTypeMap.put("b-g-.-F-A-N", "NO FIRE AREA (NFA)");
        cotTypeMap.put("b-g-.-F-A-R", "RESTRICTIVE FIRE AREA (RFA)");
        cotTypeMap.put("b-g-.-F-A-P", "POSITION AREA FOR ARTILLERY (PAA)");
        cotTypeMap.put("b-g-.-S", "COMBAT SERVICE SUPPORT");
        cotTypeMap.put("b-g-.-S-P", "POINTS");
        cotTypeMap.put("b-g-.-S-P-X", "AMBULANCE EXCHANGE POINT");
        cotTypeMap.put("b-g-.-S-P-C", "CANNIBALIZATION POINT");
        cotTypeMap.put("b-g-.-S-P-Y", "CASUALTY COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-T", "CIVILIAN COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-D", "DETAINEE COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-E", "ENEMY PRISONER OF WAR (EPW) COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-L", "LOGISTICS RELEASE POINT (LRP)");
        cotTypeMap.put("b-g-.-S-P-M", "MAINTENANCE COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-R", "REARM, REFUEL AND RESUPPLY POINT");
        cotTypeMap.put("b-g-.-S-P-U", "REFUEL ON THE MOVE (ROM) POINT");
        cotTypeMap.put("b-g-.-S-P-O", "TRAFFIC CONTROL POST (TCP)");
        cotTypeMap.put("b-g-.-S-P-I", "TRAILER TRANSFER POINT");
        cotTypeMap.put("b-g-.-S-P-N", "UNIT MAINTENANCE COLLECTION POINT");
        cotTypeMap.put("b-g-.-S-P-S", "SUPPLY POINTS");
        cotTypeMap.put("b-g-.-S-P-S-Z", "GENERAL");
        cotTypeMap.put("b-g-.-S-P-S-A", "CLASS 1");
        cotTypeMap.put("b-g-.-S-P-S-B", "CLASS II");
        cotTypeMap.put("b-g-.-S-P-S-C", "CLASS III");
        cotTypeMap.put("b-g-.-S-P-S-D", "CLASS IV");
        cotTypeMap.put("b-g-.-S-P-S-E", "CLASS V");
        cotTypeMap.put("b-g-.-S-P-S-F", "CLASS VI");
        cotTypeMap.put("b-g-.-S-P-S-G", "CLASS VII");
        cotTypeMap.put("b-g-.-S-P-S-H", "CLASS VIII");
        cotTypeMap.put("b-g-.-S-P-S-I", "CLASS IX");
        cotTypeMap.put("b-g-.-S-P-S-J", "CLASS X");
        cotTypeMap.put("b-g-.-S-P-A", "AMMUNITION POINTS");
        cotTypeMap.put("b-g-.-S-P-A-S", "AMMUNITION SUPPLY POINT (ASP)");
        cotTypeMap.put("b-g-.-S-P-A-T", "AMMUNITION TRANSFER POINT (ATP)");
        cotTypeMap.put("b-g-.-S-L", "LINES");
        cotTypeMap.put("b-g-.-S-L-C", "CONVOYS");
        cotTypeMap.put("b-g-.-S-L-C-M", "MOVING CONVOY");
        cotTypeMap.put("b-g-.-S-L-C-H", "HALTED CONVOY");
        cotTypeMap.put("b-g-.-S-L-R", "SUPPLY ROUTES");
        cotTypeMap.put("b-g-.-S-L-R-M", "MAIN SUPPLY ROUTE");
        cotTypeMap.put("b-g-.-S-L-R-A", "ALTERNATE SUPPLY ROUTE");
        cotTypeMap.put("b-g-.-S-L-R-O", "ONE WAY TRAFFIC");
        cotTypeMap.put("b-g-.-S-L-R-T", "ALTERNATING TRAFFIC");
        cotTypeMap.put("b-g-.-S-L-R-W", "TWO WAY TRAFFIC");
        cotTypeMap.put("b-g-.-S-A", "AREA");
        cotTypeMap.put("b-g-.-S-A-D", "DETAINEE HOLDING AREA");
        cotTypeMap.put("b-g-.-S-A-E", "ENEMY PRISONER OF WAR (EPW) HOLDING AREA");
        cotTypeMap.put("b-g-.-S-A-R", "FORWARD ARMING AND REFUELING AREA (FARP)");
        cotTypeMap.put("b-g-.-S-A-H", "REFUGEE HOLDING AREA");
        cotTypeMap.put("b-g-.-S-A-S", "SUPPORT AREAS");
        cotTypeMap.put("b-g-.-S-A-S-B", "BRIGADE (BSA)");
        cotTypeMap.put("b-g-.-S-A-S-D", "DIVISION (DSA)");
        cotTypeMap.put("b-g-.-S-A-S-R", "REGIMENTAL (RSA)");
        cotTypeMap.put("b-g-.-O", "OTHER");
        cotTypeMap.put("b-g-.-O-E", "EMERGENCY");
        cotTypeMap.put("b-g-.-O-E-D", "DITCHED AIRCRAFT");
        cotTypeMap.put("b-g-.-O-E-P", "PERSON IN WATER");
        cotTypeMap.put("b-g-.-O-E-V", "DISTRESSED VESSEL");
        cotTypeMap.put("b-g-.-O-H", "HAZARD");
        cotTypeMap.put("b-g-.-O-H-M", "SEA MINE LIKE");
        cotTypeMap.put("b-g-.-O-H-N", "NAVIGATIONAL");
        cotTypeMap.put("b-g-.-O-H-I", "ICEBERG");
        cotTypeMap.put("b-g-.-O-H-O", "OIL RIG");
        cotTypeMap.put("b-g-.-O-S", "SEA SUBSURFACE RETURNS");
        cotTypeMap.put("b-g-.-O-S-B", "BOTTOM RETURN/NOMBO");
        cotTypeMap.put("b-g-.-O-S-B-M", "INSTALLATION/MANMADE");
        cotTypeMap.put("b-g-.-O-S-B-N", "SEABED ROCK/STONE, OBSTACLE, OTHER");
        cotTypeMap.put("b-g-.-O-S-B-W", "WRECK");
        cotTypeMap.put("b-g-.-O-S-M", "MARINE LIFE");
        cotTypeMap.put("b-g-.-O-S-S", "SEA ANOMALY (WAKE, CURRENT, KNUCKLE)");
        cotTypeMap.put("b-g-.-O-B", "BEARING LINE");
        cotTypeMap.put("b-g-.-O-B-E", "ELECTRONIC");
        cotTypeMap.put("b-g-.-O-B-A", "ACOUSTIC");
        cotTypeMap.put("b-g-.-O-B-T", "TORPEDO");
        cotTypeMap.put("b-g-.-O-B-O", "ELECTRO OPTICAL INTERCEPT");
        cotTypeMap.put("b-g-.-O-F", "FIX");
        cotTypeMap.put("b-g-.-O-F-A", "ACOUSTIC");
        cotTypeMap.put("b-g-.-O-F-E", "ELECTRO MAGNETIC");
        cotTypeMap.put("b-g-.-O-F-O", "ELECTRO OPTICAL");
        cotTypeMap.put("b-w", "METOC");
        cotTypeMap.put("b-w-A", "ATMOSPHERIC");
        cotTypeMap.put("b-w-A-t", "Temperature");
        cotTypeMap.put("b-w-A-P", "PRESSURE SYSTEMS");
        cotTypeMap.put("b-w-A-P-L", "LOW PRESSURE CENTER");
        cotTypeMap.put("b-w-A-P-H", "HIGH PRESSURE CENTER");
        cotTypeMap.put("b-w-A-P-F", "FRONTAL SYSTEMS");
        cotTypeMap.put("b-w-A-P-F-C", "COLD FRONT");
        cotTypeMap.put("b-w-A-P-F-C-U", "UPPER COLD FRONT");
        cotTypeMap.put("b-w-A-P-F-W", "WARM FRONT");
        cotTypeMap.put("b-w-A-P-F-W-U", "UPPER WARM FRONT");
        cotTypeMap.put("b-w-A-P-F-O", "OCCLUDED FRONT");
        cotTypeMap.put("b-w-A-P-F-S", "STATIONARY FRONT");
        cotTypeMap.put("b-w-A-P-X", "LINES");
        cotTypeMap.put("b-w-A-P-X-T", "TROUGH LINE");
        cotTypeMap.put("b-w-A-P-X-R", "RIDGE LINE");
        cotTypeMap.put("b-w-A-P-X-S", "SQUALL LINE");
        cotTypeMap.put("b-w-A-T", "TURBULENCE");
        cotTypeMap.put("b-w-A-T-L", "LIGHT TURBULENCE");
        cotTypeMap.put("b-w-A-T-M", "MODERATE TURBULENCE");
        cotTypeMap.put("b-w-A-T-S", "SEVERE TURBULENCE");
        cotTypeMap.put("b-w-A-T-E", "EXTREME TURBULENCE");
        cotTypeMap.put("b-w-A-I", "ICING");
        cotTypeMap.put("b-w-A-I-C", "CLEAR ICING");
        cotTypeMap.put("b-w-A-I-C-L", "LIGHT CLEAR ICING");
        cotTypeMap.put("b-w-A-I-C-M", "MODERATE CLEAR ICING");
        cotTypeMap.put("b-w-A-I-C-S", "SEVERE CLEAR ICING");
        cotTypeMap.put("b-w-A-I-R", "RIME ICING");
        cotTypeMap.put("b-w-A-I-R-L", "LIGHT RIME ICING");
        cotTypeMap.put("b-w-A-I-R-M", "MODERATE RIME ICING");
        cotTypeMap.put("b-w-A-I-R-S", "SEVERE RIME ICING");
        cotTypeMap.put("b-w-A-I-M", "MIXED ICING");
        cotTypeMap.put("b-w-A-I-M-L", "LIGHT MIXED ICING");
        cotTypeMap.put("b-w-A-I-M-M", "MODERATE MIXED ICING");
        cotTypeMap.put("b-w-A-I-M-S", "SEVERE MIXED ICING");
        cotTypeMap.put("b-w-A-W", "WIND BARB");
        cotTypeMap.put("b-w-A-W-J", "JET STREAM");
        cotTypeMap.put("b-w-A-F", "FLIGHT RULES");
        cotTypeMap.put("b-w-A-F-I", "INSTRUMENT CEILING");
        cotTypeMap.put("b-w-A-F-V", "VISUAL CEILING");
        cotTypeMap.put("b-w-A-C", "COVERAGE SYMBOLS");
        cotTypeMap.put("b-w-A-C-t", "Cloud tops");
        cotTypeMap.put("b-w-A-C-b", "Cloud base");
        cotTypeMap.put("b-w-A-C-c", "Cloud ceiling");
        cotTypeMap.put("b-w-A-C-a", "Total Cloud Coverage");
        cotTypeMap.put("b-w-A-C-C", "CLEAR SKY (SKC)");
        cotTypeMap.put("b-w-A-C-S", "SCATTERED SKY (SCT)");
        cotTypeMap.put("b-w-A-C-B", "BROKEN SKY (BKN)");
        cotTypeMap.put("b-w-A-C-W", "OVERCAST WITH BREAKS");
        cotTypeMap.put("b-w-A-C-O", "OVERCAST (OVC)");
        cotTypeMap.put("b-w-A-C-P", "SKY OBSCURED OR PARTIALLY OBSCURED");
        cotTypeMap.put("b-w-A-R", "PRECIPITATION");
        cotTypeMap.put("b-w-A-R-R", "RAIN (RA)");
        cotTypeMap.put("b-w-A-R-R-S", "RAIN SHOWER");
        cotTypeMap.put("b-w-A-R-R-F", "FREEZING RAIN (FZRA)");
        cotTypeMap.put("b-w-A-R-R-D", "DRIZZLE (DZ)");
        cotTypeMap.put("b-w-A-R-R-D-F", "FREEZING DRIZZLE (FZDZ)");
        cotTypeMap.put("b-w-A-R-S", "SNOW(SN)");
        cotTypeMap.put("b-w-A-R-S-S", "SNOW SHOWERS");
        cotTypeMap.put("b-w-A-R-S-G", "SNOW GRAINS (SG)");
        cotTypeMap.put("b-w-A-R-H", "HAIL");
        cotTypeMap.put("b-w-A-R-I", "ICE PELLETS (PE)");
        cotTypeMap.put("b-w-A-R-C", "ICE CRYSTALS (IC)");
        cotTypeMap.put("b-w-A-S", "STORMS");
        cotTypeMap.put("b-w-A-S-T", "THUNDERSTORMS (TS)");
        cotTypeMap.put("b-w-A-S-T-R", "THUNDERSTORM (TS) WITH RAIN (RA) ");
        cotTypeMap.put("b-w-A-S-T-F", "FUNNEL CLOUD (FC)/TORNADO/WATERSPOUT");
        cotTypeMap.put("b-w-A-S-T-L", "LIGHTNING (LTG)");
        cotTypeMap.put("b-w-A-S-S", "STORM SYSTEMS");
        cotTypeMap.put("b-w-A-S-S-T", "TROPICAL STORM");
        cotTypeMap.put("b-w-A-S-S-H", "HURRICANE");
        cotTypeMap.put("b-w-A-O", "OBSTRUCTIONS TO VISIBILITY");
        cotTypeMap.put("b-w-A-O-S", "BLOWING SNOW (BLSN)");
        cotTypeMap.put("b-w-A-O-F", "FOG (FG)");
        cotTypeMap.put("b-w-A-O-F-F", "FREEZING FOG (FZFG)");
        cotTypeMap.put("b-w-A-O-T", "DUST/SAND STORM");
        cotTypeMap.put("b-w-A-O-D", "DUST DEVIL");
        cotTypeMap.put("b-w-A-O-K", "SMOKE (FU)");
        cotTypeMap.put("b-w-A-O-H", "HAZE (HZ)");
        cotTypeMap.put("b-w-A-O-B", "BLOWING DUST OR SAND");
        cotTypeMap.put("b-w-O", "OCEANIC");
        cotTypeMap.put("b-w-S", "SPACE");
        cotTypeMap.put("b-r-.-h-c", "HEALTH/CASUALTY");
        cotTypeMap.put("b-r-.-I", "SIGNALS INTELLIGENCE");
        cotTypeMap.put("b-r-.-I-P", "SPACE TRACK");
        cotTypeMap.put("b-r-.-I-P-S", "SIGNAL INTERCEPT");
        cotTypeMap.put("b-r-.-I-P-S-C", "COMMUNICATIONS");
        cotTypeMap.put("b-r-.-I-P-S-C-D", "SATELLITE DOWN LINK");
        cotTypeMap.put("b-r-.-I-P-S-R", "RADAR");
        cotTypeMap.put("b-r-.-I-P-S-R-D", "DATA TRANSMISSION");
        cotTypeMap.put("b-r-.-I-P-S-R-E", "EARTH SURVEILLANCE");
        cotTypeMap.put("b-r-.-I-P-S-R-I", "IFF (TRANSPONDER)");
        cotTypeMap.put("b-r-.-I-P-S-R-M", "MULTI FUNCTION");
        cotTypeMap.put("b-r-.-I-P-S-R-T", "TARGET ACQUISITION");
        cotTypeMap.put("b-r-.-I-P-S-R-S", "SPACE");
        cotTypeMap.put("b-r-.-I-P-S-R-U", "UNKNOWN");
        cotTypeMap.put("b-r-.-I-A", "AIR TRACK");
        cotTypeMap.put("b-r-.-I-A-S", "SIGNAL INTERCEPT");
        cotTypeMap.put("b-r-.-I-A-S-C", "COMMUNICATIONS");
        cotTypeMap.put("b-r-.-I-A-S-C-C", "CELLULAR/MOBILE");
        cotTypeMap.put("b-r-.-I-A-S-C-O", "OMNI LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-A-S-C-P", "POINT TO POINT LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-A-S-C-S", "SATELLITE UP LINK");
        cotTypeMap.put("b-r-.-I-A-S-R", "RADAR");
        cotTypeMap.put("b-r-.-I-A-S-R-A-I", "AIRBORNE INTERCEPT");
        cotTypeMap.put("b-r-.-I-A-S-R-A-S", "AIRBORNE SEARCH + BOMBING");
        cotTypeMap.put("b-r-.-I-A-S-R-C", "CONTROLLED INTERCEPT");
        cotTypeMap.put("b-r-.-I-A-S-R-D", "DATA TRANSMISSION");
        cotTypeMap.put("b-r-.-I-A-S-R-E", "EARLY WARNING");
        cotTypeMap.put("b-r-.-I-A-S-R-F", "FIRE CONTROL");
        cotTypeMap.put("b-r-.-I-A-S-R-I", "IFF (TRANSPONDER)");
        cotTypeMap.put("b-r-.-I-A-S-R-M-A", "MISSILE ACQUISITION");
        cotTypeMap.put("b-r-.-I-A-S-R-M-D", "MISSILE DOWNLINK");
        cotTypeMap.put("b-r-.-I-A-S-R-M-G", "MISSILE GUIDANCE");
        cotTypeMap.put("b-r-.-I-A-S-R-M-T", "MISSILE TRACKING");
        cotTypeMap.put("b-r-.-I-A-S-R-M-F", "MULTI FUNCTION");
        cotTypeMap.put("b-r-.-I-A-S-R-T-I", "TARGET ILLUMINATOR");
        cotTypeMap.put("b-r-.-I-A-S-R-T-A", "TARGET ACQUISITION");
        cotTypeMap.put("b-r-.-I-A-S-R-T-T", "TARGET TRACKING");
        cotTypeMap.put("b-r-.-I-A-S-R-U", "UNKNOWN");
        cotTypeMap.put("b-r-.-I-G", "GROUND TRACK");
        cotTypeMap.put("b-r-.-I-G-S", "SIGNAL INTERCEPT");
        cotTypeMap.put("b-r-.-I-G-S-C", "COMMUNICATIONS");
        cotTypeMap.put("b-r-.-I-G-S-C-C", "CELLULAR/MOBILE");
        cotTypeMap.put("b-r-.-I-G-S-C-O", "OMNI LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-G-S-C-P", "POINT TO POINT LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-G-S-C-S", "SATELLITE UP LINK");
        cotTypeMap.put("b-r-.-I-G-S-C-T", "TROPOSPHERIC SCATTER");
        cotTypeMap.put("b-r-.-I-G-S-R", "RADAR");
        cotTypeMap.put("b-r-.-I-G-S-R-A-T", "AIR TRAFFIC CONTROL");
        cotTypeMap.put("b-r-.-I-G-S-R-A-A", "ANTI AIRCRAFT");
        cotTypeMap.put("b-r-.-I-G-S-R-B", "BATTLEFIELD SURVEILLANCE");
        cotTypeMap.put("b-r-.-I-G-S-R-C-S", "COASTAL SURVEILLANCE");
        cotTypeMap.put("b-r-.-I-G-S-R-C-A", "CONTROLLED APPROACH");
        cotTypeMap.put("b-r-.-I-G-S-R-D", "DATA TRANSMISSION");
        cotTypeMap.put("b-r-.-I-G-S-R-E", "EARLY WARNING");
        cotTypeMap.put("b-r-.-I-G-S-R-F", "FIRE CONTROL");
        cotTypeMap.put("b-r-.-I-G-S-R-H", "HEIGHT FINDING");
        cotTypeMap.put("b-r-.-I-G-S-R-I", "IDENTIFICATION FRIEND/FOE (INTERROGATOR)");
        cotTypeMap.put("b-r-.-I-G-S-R-M-M", "METEOROLOGICAL (MILITARY)");
        cotTypeMap.put("b-r-.-I-G-S-R-M-A", "MISSILE ACQUISITION");
        cotTypeMap.put("b-r-.-I-G-S-R-M-G", "MISSILE GUIDANCE");
        cotTypeMap.put("b-r-.-I-G-S-R-M-T", "MISSILE TRACKING");
        cotTypeMap.put("b-r-.-I-G-S-R-M-F", "MULTI FUNCTION");
        cotTypeMap.put("b-r-.-I-G-S-R-S", "SHELL TRACKING");
        cotTypeMap.put("b-r-.-I-G-S-R-T-A", "TARGET ACQUISITION");
        cotTypeMap.put("b-r-.-I-G-S-R-T-I", "TARGET ILLUMINATOR");
        cotTypeMap.put("b-r-.-I-G-S-R-T-T", "TARGET TRACKING");
        cotTypeMap.put("b-r-.-I-G-S-R-U", "UNKNOWN");
        cotTypeMap.put("b-r-.-I-S", "SEA SURFACE TRACK");
        cotTypeMap.put("b-r-.-I-S-S", "SIGNAL INTERCEPT");
        cotTypeMap.put("b-r-.-I-S-S-C", "COMMUNICATIONS");
        cotTypeMap.put("b-r-.-I-S-S-C-C", "CELLULAR/MOBILE");
        cotTypeMap.put("b-r-.-I-S-S-C-O", "OMNI LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-S-S-C-P", "POINT TO POINT LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-S-S-C-S", "SATELLITE UP LINK");
        cotTypeMap.put("b-r-.-I-S-S-R", "RADAR");
        cotTypeMap.put("b-r-.-I-S-S-R-A-T", "AIR TRAFFIC CONTROL");
        cotTypeMap.put("b-r-.-I-S-S-R-A-A", "ANTI AIRCRAFT");
        cotTypeMap.put("b-r-.-I-S-S-R-C-A", "CONTROLLED APPROACH");
        cotTypeMap.put("b-r-.-I-S-S-R-C-I", "CONTROLLED INTERCEPT");
        cotTypeMap.put("b-r-.-I-S-S-R-D", "DATA TRANSMISSION");
        cotTypeMap.put("b-r-.-I-S-S-R-E", "EARLY WARNING");
        cotTypeMap.put("b-r-.-I-S-S-R-F", "FIRE CONTROL");
        cotTypeMap.put("b-r-.-I-S-S-R-H", "HEIGHT FINDING");
        cotTypeMap.put("b-r-.-I-S-S-R-I", "IDENTIFICATION FRIEND/FOE (INTERROGATOR)");
        cotTypeMap.put("b-r-.-I-S-S-R-M-M", "METEOROLOGICAL (MILITARY)");
        cotTypeMap.put("b-r-.-I-S-S-R-M-A", "MISSILE ACQUISITION");
        cotTypeMap.put("b-r-.-I-S-S-R-M-G", "MISSILE GUIDANCE");
        cotTypeMap.put("b-r-.-I-S-S-R-M-T", "MISSILE TRACKING");
        cotTypeMap.put("b-r-.-I-S-S-R-M-F", "MULTI FUNCTION");
        cotTypeMap.put("b-r-.-I-S-S-R-S", "SURFACE SEARCH");
        cotTypeMap.put("b-r-.-I-S-S-R-T-A", "TARGET ACQUISITION");
        cotTypeMap.put("b-r-.-I-S-S-R-T-I", "TARGET ILLUMINATOR");
        cotTypeMap.put("b-r-.-I-S-S-R-T-T", "TARGET TRACKING");
        cotTypeMap.put("b-r-.-I-S-S-R-U", "UNKNOWN");
        cotTypeMap.put("b-r-.-I-U", "SUBSURFACE TRACK");
        cotTypeMap.put("b-r-.-I-U-S", "SIGNAL INTERCEPT");
        cotTypeMap.put("b-r-.-I-U-S-C", "COMMUNICATIONS");
        cotTypeMap.put("b-r-.-I-U-S-C-O", "OMNI LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-U-S-C-P", "POINT TO POINT LINE OF SIGHT (LOS)");
        cotTypeMap.put("b-r-.-I-U-S-C-S", "SATELLITE UP LINK");
        cotTypeMap.put("b-r-.-I-U-S-R", "RADAR");
        cotTypeMap.put("b-r-.-I-U-S-R-D", "DATA TRANSMISSION");
        cotTypeMap.put("b-r-.-I-U-S-R-E", "EARLY WARNING");
        cotTypeMap.put("b-r-.-I-U-S-R-M", "MULTI FUNCTION");
        cotTypeMap.put("b-r-.-I-U-S-R-S", "SURFACE SEARCH");
        cotTypeMap.put("b-r-.-I-U-S-R-T", "TARGET ACQUISITION");
        cotTypeMap.put("b-r-.-I-U-S-R-U", "UNKNOWN");
        cotTypeMap.put("b-r-.-O", "MILITARY OPERATIONS OTHER THAN WAR (MOOTW)");
        cotTypeMap.put("b-r-.-O-V", "VIOLENT ACTIVITIES (DEATH CAUSING)");
        cotTypeMap.put("b-r-.-O-V-A", "ARSON/FIRE");
        cotTypeMap.put("b-r-.-O-V-M", "ASSASSINATION/MURDER/EXECUTION");
        cotTypeMap.put("b-r-.-O-V-B", "BOMB/BOMBING");
        cotTypeMap.put("b-r-.-O-V-Y", "BOOBY TRAP");
        cotTypeMap.put("b-r-.-O-V-D", "DRIVE BY SHOOTING");
        cotTypeMap.put("b-r-.-O-V-S", "SNIPING");
        cotTypeMap.put("b-r-.-O-V-P", "POISONING");
        cotTypeMap.put("b-r-.-O-L", "LOCATIONS");
        cotTypeMap.put("b-r-.-O-L-B", "BLACK LIST LOCATION");
        cotTypeMap.put("b-r-.-O-L-G", "GRAY LIST LOCATION");
        cotTypeMap.put("b-r-.-O-L-W", "WHITE LIST LOCATION");
        cotTypeMap.put("b-r-.-O-O", "OPERATIONS");
        cotTypeMap.put("b-r-.-O-O-P", "PATROLLING");
        cotTypeMap.put("b-r-.-O-O-R", "RECRUITMENT");
        cotTypeMap.put("b-r-.-O-O-R-W", "RECRUITMENT (WILLING)");
        cotTypeMap.put("b-r-.-O-O-R-C", "RECRUITMENT (COERCED/IMPRESSED)");
        cotTypeMap.put("b-r-.-O-O-D", "DEMONSTRATION");
        cotTypeMap.put("b-r-.-O-O-M", "MINE LAYING");
        cotTypeMap.put("b-r-.-O-O-Y", "PSYCHOLOGICAL OPERATIONS (PSYOP)");
        cotTypeMap.put("b-r-.-O-O-Y-T", "PSYOP (TV AND RADIO PROPAGANDA)");
        cotTypeMap.put("b-r-.-O-O-Y-W", "PSYOP (WRITTEN PROPAGANA)");
        cotTypeMap.put("b-r-.-O-O-Y-H", "HOUSE TO HOUSE PROPAGANDA");
        cotTypeMap.put("b-r-.-O-O-F", "FORAGING/SEARCHING");
        cotTypeMap.put("b-r-.-O-O-S", "SPY");
        cotTypeMap.put("b-r-.-O-O-O", "FOOD DISTRIBUTION");
        cotTypeMap.put("b-r-.-O-O-E", "EXTORTION");
        cotTypeMap.put("b-r-.-O-O-H", "HIJACKING");
        cotTypeMap.put("b-r-.-O-O-H-T", "HIJACKING (VEHICLE)");
        cotTypeMap.put("b-r-.-O-O-H-A", "HIJACKING (AIRPLANE)");
        cotTypeMap.put("b-r-.-O-O-H-V", "HIJACKING (BOAT)");
        cotTypeMap.put("b-r-.-O-O-K", "KIDNAPPING");
        cotTypeMap.put("b-r-.-O-O-A", "ARREST");
        cotTypeMap.put("b-r-.-O-O-U", "DRUG OPERATION");
        cotTypeMap.put("b-r-.-O-I", "ITEMS");
        cotTypeMap.put("b-r-.-O-I-R", "REFUGEES");
        cotTypeMap.put("b-r-.-O-I-S", "SAFE HOUSE");
        cotTypeMap.put("b-r-.-O-I-G", "GRAFITTI");
        cotTypeMap.put("b-r-.-O-I-V", "VANDALISM/RAPE/LOOT/RANSACK/PLUNDER/SACK");
        cotTypeMap.put("b-r-.-O-I-I", "KNOWN INSURGENT VEHICLE");
        cotTypeMap.put("b-r-.-O-I-D", "DRUG VEHICLE");
        cotTypeMap.put("b-r-.-O-I-F", "INTERNAL SECURITY FORCE");
        cotTypeMap.put("b-x.*", "NON-COT OBJECT");
        cotTypeMap.put("r-c-x-c", "Chemical attack");
        cotTypeMap.put("r-c-x-c-s", "blister: aka vesicants. Chemicals that severely blister the eyes, respiratory tract, and skin on contact");
        cotTypeMap.put("r-c-x-c-s-m", "mustards");
        cotTypeMap.put("r-c-x-c-s-m-hd", "istilled mustard (HD)");
        cotTypeMap.put("r-c-x-c-s-m-h", "Mustard gas (H) (sulfur mustard)");
        cotTypeMap.put("r-c-x-c-s-m-hl", "ustard/lewisite (HL)");
        cotTypeMap.put("r-c-x-c-s-m-t", "Mustard/T");
        cotTypeMap.put("r-c-x-c-s-m-n", "Nitrogen mustard (HN-1, HN-2, HN-3)");
        cotTypeMap.put("r-c-x-c-s-m-q", "Sesqui mustard");
        cotTypeMap.put("r-c-x-c-s-m-s", "Sulfur mustard (H) (mustard gas)");
        cotTypeMap.put("r-c-x-c-s-cx", "hosgene oxime (CX)");
        cotTypeMap.put("r-c-x-c-s-l", "Lewisites/chloroarsine agents");
        cotTypeMap.put("r-c-x-c-s-l-l", "Lewisite (L, L-1, L-2, L-3)");
        cotTypeMap.put("r-c-x-c-s-l-hl", "Mustard/lewisite (HL)");
        cotTypeMap.put("r-c-x-c-b", "blood: Poisons that affect the body by being absorbed into the blood");
        cotTypeMap.put("r-c-x-c-b-a", "Arsine (SA)");
        cotTypeMap.put("r-c-x-c-b-m", "Carbon Monoxide");
        cotTypeMap.put("r-c-x-c-b-c", "Cyanide");
        cotTypeMap.put("r-c-x-c-b-c-c", "Cyanogen chloride (CK)");
        cotTypeMap.put("r-c-x-c-b-c-h", "Hydrogen cyanide (AC)");
        cotTypeMap.put("r-c-x-c-b-c-p", "Potassium cyanide (KCN)");
        cotTypeMap.put("r-c-x-c-b-c-s", "Sodium cyanide (NaCN)");
        cotTypeMap.put("r-c-x-c-b-s", "Sodium monofluoroacetate (compound 1080)");
        cotTypeMap.put("r-c-x-c-c", "caustics (Acids): Chemicals that burn or corrode people's skin, eyes, and mucus membranes (lining of the nose, mouth, throat, and lungs) on contact");
        cotTypeMap.put("r-c-x-c-c-h", "Hydrofluoric acid (hydrogen fluoride)");
        cotTypeMap.put("r-c-x-c-p", "Choking/Lung/Pulmonary Agents: Chemicals that cause severe irritation or swelling of the respiratory tract (lining of the nose, throat, and lungs)");
        cotTypeMap.put("r-c-x-c-p-a", "Ammonia");
        cotTypeMap.put("r-c-x-c-p-b", "Bromine (CA)");
        cotTypeMap.put("r-c-x-c-p-c", "Chlorine (CL)");
        cotTypeMap.put("r-c-x-c-p-h", "Hydrogen chloride");
        cotTypeMap.put("r-c-x-c-p-m", "Methyl bromide");
        cotTypeMap.put("r-c-x-c-p-i", "Methyl isocyanate");
        cotTypeMap.put("r-c-x-c-p-t", "Osmium tetroxide");
        cotTypeMap.put("r-c-x-c-p-p", "Phosgene");
        cotTypeMap.put("r-c-x-c-p-p-d", "Diphosgene (DP)");
        cotTypeMap.put("r-c-x-c-p-p-c", "Phosgene (CG)");
        cotTypeMap.put("r-c-x-c-p-o", "Phosphine");
        cotTypeMap.put("r-c-x-c-p-r", "Phosphorus, elemental");
        cotTypeMap.put("r-c-x-c-p-r-w", "white");
        cotTypeMap.put("r-c-x-c-p-r-y", "yellow");
        cotTypeMap.put("r-c-x-c-p-s", "Sulfuryl fluoride");
        cotTypeMap.put("r-c-x-c-i", "Incapacitating Agents: Drugs that make people unable to think clearly or that cause an altered state of consciousness (possibly unconsciousness)");
        cotTypeMap.put("r-c-x-c-i-b", "BZ");
        cotTypeMap.put("r-c-x-c-i-o", "opioids: chemical substance that causes a morphine-like action");
        cotTypeMap.put("r-c-x-c-i-o-f", "Fentanyls");
        cotTypeMap.put("r-c-x-c-a", "Long-Acting Anticoagulants: Poisons that prevent blood from clotting properly, which can lead to uncontrolled bleeding");
        cotTypeMap.put("r-c-x-c-a-w", "Super warfarin");
        cotTypeMap.put("r-c-x-c-m", "Metals: Agents that consist of metallic poisons");
        cotTypeMap.put("r-c-x-c-m-a", "Arsenic");
        cotTypeMap.put("r-c-x-c-m-b", "Barium");
        cotTypeMap.put("r-c-x-c-m-m", "Mercury");
        cotTypeMap.put("r-c-x-c-m-t", "Thallium");
        cotTypeMap.put("r-c-x-c-n", "Nerve Agents: Highly poisonous chemicals that work by preventing the nervous system from working properly");
        cotTypeMap.put("r-c-x-c-n-g", "G agents");
        cotTypeMap.put("r-c-x-c-n-g-b", "Sarin (GB)");
        cotTypeMap.put("r-c-x-c-n-g-d", "Soman (GD)");
        cotTypeMap.put("r-c-x-c-n-g-a", "Tabun (GA)");
        cotTypeMap.put("r-c-x-c-n-v", "V agents");
        cotTypeMap.put("r-c-x-c-n-v-x", "VX");
        cotTypeMap.put("r-c-x-c-o", "Organic Solvents: Agents that damage the tissues of living things by dissolving fats and oils");
        cotTypeMap.put("r-c-x-c-o-b", "Benzene");
        cotTypeMap.put("r-c-x-c-g", "Riot Control Agents/Tear Gas: Highly irritating agents normally used by law enforcement for crowd control or by individuals for protection (for example, mace)");
        cotTypeMap.put("r-c-x-c-g-b", "Bromobenzylcyanide (CA)");
        cotTypeMap.put("r-c-x-c-g-n", "Chloroacetophenone (CN)");
        cotTypeMap.put("r-c-x-c-g-s", "Chlorobenzylidenemalononitrile (CS)");
        cotTypeMap.put("r-c-x-c-g-p", "Chloropicrin (PS)");
        cotTypeMap.put("r-c-x-c-g-d", "Dibenzoxazepine (CR)");
        cotTypeMap.put("r-c-x-c-h", "Toxic Alcohols: Poisonous alcohols that can damage the heart, kidneys, and nervous system");
        cotTypeMap.put("r-c-x-c-h-e", "Ethylene glycol");
        cotTypeMap.put("r-c-x-c-v", "Vomiting Agents: Chemicals that cause nausea and vomiting");
        cotTypeMap.put("r-c-x-c-v-a", "Adamsite (DM)");
        cotTypeMap.put("r-c-x-b", "Biological attack");
        cotTypeMap.put("r-c-x-b-m", "Mycotic biological agents");
        cotTypeMap.put("r-c-x-b-m-c", "OC, Coccidioides mycosis");
        cotTypeMap.put("r-c-x-b-b", "Bacterial biological agents");
        cotTypeMap.put("r-c-x-b-b-a", "anthrax");
        cotTypeMap.put("r-c-x-b-b-a-n", "N anthrax");
        cotTypeMap.put("r-c-x-b-b-a-t", "TR anthrax");
        cotTypeMap.put("r-c-x-b-b-p", "LE, plague");
        cotTypeMap.put("r-c-x-b-b-t", "tularemia");
        cotTypeMap.put("r-c-x-b-b-t-u", "UL, tularemia (schu S4)");
        cotTypeMap.put("r-c-x-b-b-t-w", "TT, wet-type UL");
        cotTypeMap.put("r-c-x-b-b-t-d", "ZZ, dry-type UL");
        cotTypeMap.put("r-c-x-b-b-t-s", "SR, tularemia");
        cotTypeMap.put("r-c-x-b-b-t-j", "JT, tularemia (425)");
        cotTypeMap.put("r-c-x-b-b-c", "HO, cholera");
        cotTypeMap.put("r-c-x-b-b-b", "brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-b", "AB, bovine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-p", "porcine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-p-u", "US, porcine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-p-n", "NX, porcine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-c", "caprine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-c-a", "AM, caprine brucellosis");
        cotTypeMap.put("r-c-x-b-b-b-c-b", "BX, caprine brucellosis");
        cotTypeMap.put("r-c-x-b-b-y", "Y, bacterial dysentery");
        cotTypeMap.put("r-c-x-b-b-g", "LA, Glanders");
        cotTypeMap.put("r-c-x-b-b-m", "HI, Melioidosis");
        cotTypeMap.put("r-c-x-b-b-d", "DK, diphtheria");
        cotTypeMap.put("r-c-x-b-b-l", "TQ, listeriosis");
        cotTypeMap.put("r-c-x-b-c", "Chlamydial biological agents");
        cotTypeMap.put("r-c-x-b-c-s", "SI - psittacosis");
        cotTypeMap.put("r-c-x-b-r", "Rickettsial biological agents");
        cotTypeMap.put("r-c-x-b-r-r", "rocky mountain spotted fever");
        cotTypeMap.put("r-c-x-b-r-r-r", "RI, rocky mountain spotted fever");
        cotTypeMap.put("r-c-x-b-r-r-u", "UY, rocky mountain spotted fever");
        cotTypeMap.put("r-c-x-b-r-q", "OU, Q fever");
        cotTypeMap.put("r-c-x-b-r-q-w", "MN, wet-type OU");
        cotTypeMap.put("r-c-x-b-r-q-d", "NT, dry-type OU");
        cotTypeMap.put("r-c-x-b-r-t", "typhus");
        cotTypeMap.put("r-c-x-b-r-t-h", "YE, human typhus");
        cotTypeMap.put("r-c-x-b-r-t-m", "AV, murine typhus");
        cotTypeMap.put("r-c-x-b-v", "Viral biological agents");
        cotTypeMap.put("r-c-x-b-v-y", "yellow fever");
        cotTypeMap.put("r-c-x-b-v-y-o", "OJ, yellow fever");
        cotTypeMap.put("r-c-x-b-v-y-u", "UT, yellow fever");
        cotTypeMap.put("r-c-x-b-v-y-l", "LU, yellow fever");
        cotTypeMap.put("r-c-x-b-v-r", "FA, rift valley fever");
        cotTypeMap.put("r-c-x-b-v-e", "Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e", "Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e-v", "Venezuelan Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e-v-n", "NU, Venezuelan Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e-v-t", "TD, Venezuelan Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e-v-f", "FX, Venezuelan Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-e-e", "ZX, Eastern Equine Encephalitis");
        cotTypeMap.put("r-c-x-b-v-e-j", "AN, Japanese B Encephalitis");
        cotTypeMap.put("r-c-x-b-v-s", "ZL, smallpox");
        cotTypeMap.put("r-c-x-b-t", "Biological Toxins");
        cotTypeMap.put("r-c-x-b-t-a", "Abrin");
        cotTypeMap.put("r-c-x-b-t-v", "Brevetoxin");
        cotTypeMap.put("r-c-x-b-t-b", "botulinum toxin A");
        cotTypeMap.put("r-c-x-b-t-b-x", "X, botulinum toxin A");
        cotTypeMap.put("r-c-x-b-t-b-xr", "XR, partially purified botulinum toxin A");
        cotTypeMap.put("r-c-x-b-t-r", "ricin");
        cotTypeMap.put("r-c-x-b-t-r-w", "W, ricin");
        cotTypeMap.put("r-c-x-b-t-r-wa", "WA, ricin");
        cotTypeMap.put("r-c-x-b-t-e", "staphylococcal enterotoxin B");
        cotTypeMap.put("r-c-x-b-t-e-uc", "UC, staphylococcal enterotoxin B");
        cotTypeMap.put("r-c-x-b-t-e-pg", "PG, staphylococcal enterotoxin B");
        cotTypeMap.put("r-c-x-b-t-s", "saxitoxin");
        cotTypeMap.put("r-c-x-b-t-s-t", "TZ, saxitoxin");
        cotTypeMap.put("r-c-x-b-t-s-s", "SS, saxitoxin");
        cotTypeMap.put("r-c-x-b-t-c", "Colchicine");
        cotTypeMap.put("r-c-x-b-t-y", "Strychnine");
        cotTypeMap.put("r-c-x-b-t-n", "Nicotine");
        cotTypeMap.put("r-c-x-b-t-d", "Digitalis");
        cotTypeMap.put("r-c-x-b-t-t", "PP, tetrodotoxin");
        cotTypeMap.put("r-c-x-b-t-3", "Trichothecene");
        cotTypeMap.put("r-c-x-b-s", "Simuants");
        cotTypeMap.put("r-c-x-b-s-m", "MR, molasis residium");
        cotTypeMap.put("r-c-x-b-s-g", "Bacillus globigii");
        cotTypeMap.put("r-c-x-b-s-g-bg", "BG, Bacillus globigii");
        cotTypeMap.put("r-c-x-b-s-g-bs", "BS, Bacillus globigii");
        cotTypeMap.put("r-c-x-b-s-g-u", "U, Bacillus globigii");
        cotTypeMap.put("r-c-x-b-s-s", "Serratia marescens");
        cotTypeMap.put("r-c-x-b-s-s-s", "SM, Serratia marescens");
        cotTypeMap.put("r-c-x-b-s-s-p", "P, Serratia marescens");
        cotTypeMap.put("r-c-x-b-s-a", "AF, Aspergillus fumigatus mutant C-2");
        cotTypeMap.put("r-c-x-b-s-e", "EC, E. coli");
        cotTypeMap.put("r-c-x-b-s-t", "BT, Bacillus thursidius");
        cotTypeMap.put("r-c-x-b-s-w", "EH, Erwinia hebicola");
        cotTypeMap.put("r-c-x-b-s-f", "FP, fluorescent particle");
    }//CotTypes constructor

    public String getDescription(String type) {
        String description = cotTypeMap.get(type);
        if (description != null) {
            return description;
        } else {
            return "null";
        }
    }//getDescription
}//CotTypes
